diff -pruN 3.0.1-1/.circleci/config.yml 3.0.2-1/.circleci/config.yml
--- 3.0.1-1/.circleci/config.yml	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.circleci/config.yml	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,302 @@
+version: 2.1
+
+orbs:
+  codecov: codecov/codecov@4.1.0
+
+executors:
+  docker_executor:
+    parameters:
+      base_image:
+        default: "python"
+        type: enum
+        enum: ["python", "pypy"]
+      version:
+        default: "3.11"
+        type: enum
+        enum: ["3.8", "3.9", "3.10", "3.11", "3.12"]
+    docker:
+      - image: "<<parameters.base_image>>:<<parameters.version>>-bookworm"
+        auth:
+          username: $DOCKERHUB_USERNAME
+          password: $DOCKERHUB_PASSWORD
+    resource_class: small
+    environment:
+      FORCE_COLOR: "1"
+      PIP_CACHE_DIR: ~/caches/pip
+      PIP_DISABLE_PIP_VERSION_CHECK: "1"
+      PIP_ROOT_USER_ACTION: "ignore"
+
+commands:
+  setup_venv:
+    description: "Setup and install virtualenv, with caching"
+    parameters:
+      install:
+        type: steps
+        default: []
+    steps:
+      - restore_cache:
+          keys:
+            - venv-v1-{{ .Environment.CIRCLE_JOB }}
+      - run: mkdir -p ~/caches/pip
+      - run: python -m venv ~/venv
+      - run: . ~/venv/bin/activate
+      - steps: <<parameters.install>>
+      - save_cache:
+          key: venv-v1-{{ .Environment.CIRCLE_JOB }}
+          paths:
+            - "~/venv"
+            - ~/caches/pip
+
+jobs:
+  build:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - setup_venv:
+          install:
+            - run: python -m pip install build
+      - run: python -m build .
+      - persist_to_workspace:
+          root: dist
+          paths:
+            - aiosmtplib-*.tar.gz
+            - aiosmtplib-*.whl
+      - store_artifacts:
+          path: dist/
+
+  buildcheck:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - attach_workspace:
+          at: dist
+      - setup_venv:
+          install:
+            - run: python -m pip install check-wheel-contents
+            - run: python -m pip install twine
+      - run: check-wheel-contents dist/*.whl
+      - run: twine check --strict dist/*
+
+  typecheck:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - setup_venv:
+          install:
+            - run: python -m pip install mypy
+            - run: python -m pip install pyright
+      - run: python -m mypy src/aiosmtplib
+      - run: python -m pyright tests/pyright.py
+  security:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - setup_venv:
+          install:
+            - run: python -m pip install bandit
+      - run: python -m bandit -n 10 -x tests -r src/aiosmtplib
+
+  docs:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - attach_workspace:
+          at: dist
+      - setup_venv:
+          install:
+            - run: python -m pip install -r docs/requirements.txt
+            - run: python -m pip install -r requirements-dev.txt
+      - run: python -m pip install dist/*.whl
+      - run: python -m sphinx -nWT -b doctest -d ./docs/build/doctrees ./docs ./docs/build/html
+      - run: python -m sphinx -nWT -b dummy -d ./docs/build/doctrees ./docs ./docs/build/html
+
+  test:
+    executor:
+      name: docker_executor
+      base_image: <<parameters.base_image>>
+      version: <<parameters.python_version>>
+    parameters:
+      base_image:
+        type: enum
+        enum: ["python", "pypy"]
+      python_version:
+        type: enum
+        description: "python version"
+        enum: ["3.8", "3.9", "3.10", "3.11", "3.12"]
+      event_loop:
+        type: enum
+        description: "event loop type"
+        enum: ["asyncio", "uvloop"]
+    environment:
+      COVERAGE_FILE: "coverage-results/.coverage.<<parameters.base_image>><<parameters.python_version>>-<<parameters.event_loop>>"
+      HYPOTHESIS_PROFILE: "ci"
+      BIND_ADDR: "127.0.0.1"
+    steps:
+      - checkout
+      - attach_workspace:
+          at: dist
+      - restore_cache:
+          keys:
+            - "hypothesis-v7-<<parameters.python_version>>"
+      - setup_venv:
+          install:
+            - run: python -m pip install -r requirements-dev.txt
+      # package install happens outside of venv cache
+      - when:
+          condition:
+            equal: ["uvloop", <<parameters.event_loop>>]
+          steps:
+            - run: python -m pip install $(find dist -name aiosmtplib-*.whl)[uvloop]
+      - unless:
+          condition:
+            equal: ["uvloop", <<parameters.event_loop>>]
+          steps:
+            - run: python -m pip install $(find dist -name aiosmtplib-*.whl)
+      - run: |
+          python -m pytest --cov --cov-report= \
+            --cov-config=pyproject.toml \
+            --junitxml=test-results/$CIRCLE_JOB/results.xml \
+            --event-loop=<<parameters.event_loop>> \
+            --bind-addr=$BIND_ADDR \
+            --hypothesis-profile $HYPOTHESIS_PROFILE
+      - store_artifacts:
+          path: test-results
+      - store_test_results:
+          path: test-results
+      - persist_to_workspace:
+          root: coverage-results
+          paths:
+            - .coverage.*
+      - save_cache:
+          key: "hypothesis-v7-<<parameters.python_version>>"
+          paths:
+            - .hypothesis
+  coverage:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    environment:
+      COVERAGE_FILE: .coverage
+    steps:
+      - checkout
+      - attach_workspace:
+          at: coverage-results
+      - run: cp coverage-results/.coverage.* ./
+      - setup_venv:
+          install:
+            - run: python -m pip install coverage[toml]
+      - run: python -m coverage combine
+      - run: python -m coverage xml
+      - run: python -m coverage html
+      - run: python -m coverage report --fail-under=90
+      - store_artifacts:
+          path: coverage.xml
+      - store_artifacts:
+          path: htmlcov
+      - codecov/upload:
+          file: coverage.xml
+
+  deploy:
+    executor:
+      name: docker_executor
+      version: "3.11"
+    steps:
+      - checkout
+      - attach_workspace:
+          at: dist
+      - setup_venv:
+          install:
+            - run: python -m pip install twine
+      - run: twine upload -r testpypi --username $TESTPYPI_USERNAME --password $TESTPYPI_PASSWORD dist/*
+      - run: twine upload -r pypi --username $PYPI_USERNAME --password $PYPI_PASSWORD dist/*
+
+workflows:
+  build_test_deploy:
+    jobs:
+      - build: &base-job
+          context:
+            - docker-hub-credentials
+          filters:
+            tags:
+              only: /.*/
+      - typecheck:
+          <<: *base-job
+      - security:
+          <<: *base-job
+      - buildcheck:
+          <<: *base-job
+          requires:
+            - build
+      - docs:
+          <<: *base-job
+          requires:
+            - build
+      - test:
+          <<: *base-job
+          name: "test-asyncio-<< matrix.python_version >>"
+          matrix:
+            alias: "test-cpython-asyncio"
+            parameters:
+              event_loop: ["asyncio"]
+              base_image: ["python"]
+              python_version: ["3.8", "3.9", "3.10", "3.11", "3.12"]
+          requires:
+            - build
+      - test:
+          <<: *base-job
+          name: "test-pypy-<< matrix.python_version >>"
+          matrix:
+            alias: "test-pypy-asyncio"
+            parameters:
+              event_loop: ["asyncio"]
+              base_image: ["pypy"]
+              python_version: ["3.9", "3.10"]
+          requires:
+            - build
+            - "test-asyncio-<< matrix.python_version >>"
+      - test:
+          <<: *base-job
+          name: "test-uvloop-<< matrix.python_version >>"
+          matrix:
+            alias: "test-cpython-uvloop"
+            parameters:
+              event_loop: ["uvloop"]
+              base_image: ["python"]
+              python_version: ["3.8", "3.9", "3.10", "3.11", "3.12"]
+          requires:
+            - build
+            - "test-asyncio-<< matrix.python_version >>"
+      - coverage:
+          <<: *base-job
+          requires:
+            - test-cpython-asyncio
+            - test-cpython-uvloop
+            - test-pypy-asyncio
+      - deploy:
+          requires:
+            - build
+            - typecheck
+            - buildcheck
+            - security
+            - docs
+            - test-cpython-asyncio
+            - test-cpython-uvloop
+            - test-pypy-asyncio
+          context:
+            - docker-hub-credentials
+          filters:
+            branches:
+              ignore: /.*/
+            tags:
+              only: /^v.*/
+# VS Code Extension Version: 1.5.1
diff -pruN 3.0.1-1/.github/ISSUE_TEMPLATE/bug_report.md 3.0.2-1/.github/ISSUE_TEMPLATE/bug_report.md
--- 3.0.1-1/.github/ISSUE_TEMPLATE/bug_report.md	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.github/ISSUE_TEMPLATE/bug_report.md	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,22 @@
+---
+name: Bug report
+about: "Something is broken \U0001F631"
+title: ''
+labels: '🐛 bug'
+assignees: ''
+
+---
+
+**Describe the bug**
+A clear and concise description of what the bug is.
+
+**To Reproduce**
+Steps to reproduce the behaviour:
+1. Call ``aiosmtplib.send`` with the following params: ...
+2. ...
+
+**Expected behaviour**
+A clear and concise description of what you expected to happen.
+
+**Additional context**
+Add any other context about the problem here.
diff -pruN 3.0.1-1/.github/ISSUE_TEMPLATE/feature_request.md 3.0.2-1/.github/ISSUE_TEMPLATE/feature_request.md
--- 3.0.1-1/.github/ISSUE_TEMPLATE/feature_request.md	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.github/ISSUE_TEMPLATE/feature_request.md	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,20 @@
+---
+name: Feature request
+about: "Suggest a feature \U0001F680"
+title: ''
+labels: '🚀 feature'
+assignees: ''
+
+---
+
+**Is your feature request related to a problem? Please describe.**
+A clear and concise description of what the problem is. Ex. I'm always frustrated when [...]
+
+**Describe the solution you'd like**
+A clear and concise description of what you want to happen.
+
+**Describe alternatives you've considered**
+A clear and concise description of any alternative solutions or features you've considered.
+
+**Additional context**
+Add any other context about the feature request here.
diff -pruN 3.0.1-1/.gitignore 3.0.2-1/.gitignore
--- 3.0.1-1/.gitignore	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.gitignore	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,30 @@
+.bandit
+.cache/
+.coverage
+.coverage.*
+.coveralls.yml
+.eggs/
+.hypothesis/
+.idea/
+.mypy_cache/
+.mutmut-cache
+.pytest_cache/
+.python-version
+.tox/
+.tool-versions
+.venv
+.vscode/
+aiosmtplib.code-workspace
+aiosmtplib.egg-info/
+aiosmtplib.sublime-project
+aiosmtplib.sublime-workspace
+build/
+coverage.xml
+coverage/
+dist/
+docs/_build/
+htmlcov/
+test-results/
+TODO
+venv/
+poetry.lock
diff -pruN 3.0.1-1/.pre-commit-config.yaml 3.0.2-1/.pre-commit-config.yaml
--- 3.0.1-1/.pre-commit-config.yaml	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.pre-commit-config.yaml	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,18 @@
+repos:
+  - repo: https://github.com/pre-commit/pre-commit-hooks
+    rev: v4.6.0
+    hooks:
+      - id: check-ast
+      - id: check-added-large-files
+      - id: fix-byte-order-marker
+      - id: check-toml
+      - id: check-yaml
+      - id: debug-statements
+      - id: check-merge-conflict
+      - id: trailing-whitespace
+      - id: end-of-file-fixer
+  - repo: https://github.com/astral-sh/ruff-pre-commit
+    rev: 'v0.5.5'
+    hooks:
+      - id: ruff
+      - id: ruff-format
diff -pruN 3.0.1-1/.readthedocs.yml 3.0.2-1/.readthedocs.yml
--- 3.0.1-1/.readthedocs.yml	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/.readthedocs.yml	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,19 @@
+# Read the Docs configuration file
+# See https://docs.readthedocs.io/en/stable/config-file/v2.html for details
+version: 2
+
+build:
+  os: ubuntu-22.04
+  tools:
+    python: "3.11"
+
+python:
+  install:
+    - requirements: docs/requirements.txt
+    - requirements: requirements-dev.txt
+    - method: pip
+      path: ./
+
+sphinx:
+  configuration: docs/conf.py
+  fail_on_warning: true
diff -pruN 3.0.1-1/CHANGELOG.rst 3.0.2-1/CHANGELOG.rst
--- 3.0.1-1/CHANGELOG.rst	2023-11-02 18:36:16.215468600 +0000
+++ 3.0.2-1/CHANGELOG.rst	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,13 @@
 Changelog
 =========
 
+3.0.2
+-----
+
+- Bugfix: Type of "send" is partially unknown with pyright
+- Change: Switched from Poetry to build/hatch/twine for packaging.
+
+
 3.0.1
 -----
 
diff -pruN 3.0.1-1/CODE_OF_CONDUCT.md 3.0.2-1/CODE_OF_CONDUCT.md
--- 3.0.1-1/CODE_OF_CONDUCT.md	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/CODE_OF_CONDUCT.md	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,128 @@
+# Contributor Covenant Code of Conduct
+
+## Our Pledge
+
+We as members, contributors, and leaders pledge to make participation in our
+community a harassment-free experience for everyone, regardless of age, body
+size, visible or invisible disability, ethnicity, sex characteristics, gender
+identity and expression, level of experience, education, socio-economic status,
+nationality, personal appearance, race, religion, or sexual identity
+and orientation.
+
+We pledge to act and interact in ways that contribute to an open, welcoming,
+diverse, inclusive, and healthy community.
+
+## Our Standards
+
+Examples of behavior that contributes to a positive environment for our
+community include:
+
+* Demonstrating empathy and kindness toward other people
+* Being respectful of differing opinions, viewpoints, and experiences
+* Giving and gracefully accepting constructive feedback
+* Accepting responsibility and apologizing to those affected by our mistakes,
+  and learning from the experience
+* Focusing on what is best not just for us as individuals, but for the
+  overall community
+
+Examples of unacceptable behavior include:
+
+* The use of sexualized language or imagery, and sexual attention or
+  advances of any kind
+* Trolling, insulting or derogatory comments, and personal or political attacks
+* Public or private harassment
+* Publishing others' private information, such as a physical or email
+  address, without their explicit permission
+* Other conduct which could reasonably be considered inappropriate in a
+  professional setting
+
+## Enforcement Responsibilities
+
+Community leaders are responsible for clarifying and enforcing our standards of
+acceptable behavior and will take appropriate and fair corrective action in
+response to any behavior that they deem inappropriate, threatening, offensive,
+or harmful.
+
+Community leaders have the right and responsibility to remove, edit, or reject
+comments, commits, code, wiki edits, issues, and other contributions that are
+not aligned to this Code of Conduct, and will communicate reasons for moderation
+decisions when appropriate.
+
+## Scope
+
+This Code of Conduct applies within all community spaces, and also applies when
+an individual is officially representing the community in public spaces.
+Examples of representing our community include using an official e-mail address,
+posting via an official social media account, or acting as an appointed
+representative at an online or offline event.
+
+## Enforcement
+
+Instances of abusive, harassing, or otherwise unacceptable behavior may be
+reported to the community leaders responsible for enforcement at
+hi@colemaclean.dev.
+All complaints will be reviewed and investigated promptly and fairly.
+
+All community leaders are obligated to respect the privacy and security of the
+reporter of any incident.
+
+## Enforcement Guidelines
+
+Community leaders will follow these Community Impact Guidelines in determining
+the consequences for any action they deem in violation of this Code of Conduct:
+
+### 1. Correction
+
+**Community Impact**: Use of inappropriate language or other behavior deemed
+unprofessional or unwelcome in the community.
+
+**Consequence**: A private, written warning from community leaders, providing
+clarity around the nature of the violation and an explanation of why the
+behavior was inappropriate. A public apology may be requested.
+
+### 2. Warning
+
+**Community Impact**: A violation through a single incident or series
+of actions.
+
+**Consequence**: A warning with consequences for continued behavior. No
+interaction with the people involved, including unsolicited interaction with
+those enforcing the Code of Conduct, for a specified period of time. This
+includes avoiding interactions in community spaces as well as external channels
+like social media. Violating these terms may lead to a temporary or
+permanent ban.
+
+### 3. Temporary Ban
+
+**Community Impact**: A serious violation of community standards, including
+sustained inappropriate behavior.
+
+**Consequence**: A temporary ban from any sort of interaction or public
+communication with the community for a specified period of time. No public or
+private interaction with the people involved, including unsolicited interaction
+with those enforcing the Code of Conduct, is allowed during this period.
+Violating these terms may lead to a permanent ban.
+
+### 4. Permanent Ban
+
+**Community Impact**: Demonstrating a pattern of violation of community
+standards, including sustained inappropriate behavior,  harassment of an
+individual, or aggression toward or disparagement of classes of individuals.
+
+**Consequence**: A permanent ban from any sort of public interaction within
+the community.
+
+## Attribution
+
+This Code of Conduct is adapted from the [Contributor Covenant][homepage],
+version 2.0, available at
+https://www.contributor-covenant.org/version/2/0/code_of_conduct.html.
+
+Community Impact Guidelines were inspired by [Mozilla's code of conduct
+enforcement ladder](https://github.com/mozilla/diversity).
+
+[homepage]: https://www.contributor-covenant.org
+
+For answers to common questions about this code of conduct, see the FAQ at
+https://www.contributor-covenant.org/faq. Translations are available at
+https://www.contributor-covenant.org/translations.
diff -pruN 3.0.1-1/Makefile 3.0.2-1/Makefile
--- 3.0.1-1/Makefile	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/Makefile	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,30 @@
+.PHONY: build install test isort black flake8 pyupgrade typecheck bandit lint security poetry-check docs
+
+POETRY := poetry
+PRECOMMIT := pre-commit
+
+build:
+	$(POETRY) build
+install:
+	$(POETRY) install
+test:
+	$(POETRY) run pytest
+isort:
+	$(PRECOMMIT) run isort --all-files --show-diff-on-failure
+black:
+	$(PRECOMMIT) run black --all-files --show-diff-on-failure
+flake8:
+	$(PRECOMMIT) run flake8 --all-files --show-diff-on-failure
+pyupgrade:
+	$(PRECOMMIT) run pyupgrade --all-files --show-diff-on-failure
+typecheck:
+	$(POETRY) run mypy aiosmtplib
+bandit:
+	$(POETRY) run bandit -n 10 -x tests ./
+poetry-check:
+	$(POETRY) check
+lint: flake8 isort black pyupgrade poetry-check
+security: bandit
+docs:
+	$(POETRY) run sphinx-build -nWT -b doctest -d ./docs/build/doctrees ./docs ./docs/build/html
+	$(POETRY) run sphinx-build -nWT -b html -d ./docs/build/doctrees ./docs ./docs/build/html
diff -pruN 3.0.1-1/PKG-INFO 3.0.2-1/PKG-INFO
--- 3.0.1-1/PKG-INFO	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/PKG-INFO	2020-02-02 00:00:00.000000000 +0000
@@ -1,39 +1,33 @@
-Metadata-Version: 2.1
+Metadata-Version: 2.3
 Name: aiosmtplib
-Version: 3.0.1
+Version: 3.0.2
 Summary: asyncio SMTP client
-Home-page: https://github.com/cole/aiosmtplib
+Project-URL: Documentation, https://aiosmtplib.readthedocs.io/en/stable/
+Project-URL: Changelog, https://github.com/cole/aiosmtplib/blob/main/CHANGELOG.rst
+Project-URL: GitHub, https://github.com/cole/aiosmtplib
+Author-email: Cole Maclean <hi@colemaclean.dev>
 License: MIT
-Keywords: smtp,email,asyncio
-Author: Cole Maclean
-Author-email: hi@colemaclean.dev
-Requires-Python: >=3.8,<4.0
+License-File: LICENSE.txt
+Keywords: asyncio,email,smtp
 Classifier: Development Status :: 5 - Production/Stable
 Classifier: Environment :: No Input/Output (Daemon)
 Classifier: Framework :: AsyncIO
 Classifier: Intended Audience :: Developers
-Classifier: License :: OSI Approved :: MIT License
 Classifier: Natural Language :: English
 Classifier: Operating System :: OS Independent
-Classifier: Programming Language :: Python :: 3
-Classifier: Programming Language :: Python :: 3.8
-Classifier: Programming Language :: Python :: 3.9
-Classifier: Programming Language :: Python :: 3.10
-Classifier: Programming Language :: Python :: 3.11
 Classifier: Topic :: Communications
 Classifier: Topic :: Communications :: Email
 Classifier: Topic :: Software Development :: Libraries
 Classifier: Topic :: Software Development :: Libraries :: Python Modules
 Classifier: Typing :: Typed
+Requires-Python: >=3.8
 Provides-Extra: docs
+Requires-Dist: furo>=2023.9.10; extra == 'docs'
+Requires-Dist: sphinx-autodoc-typehints>=1.24.0; extra == 'docs'
+Requires-Dist: sphinx-copybutton>=0.5.0; extra == 'docs'
+Requires-Dist: sphinx>=7.0.0; extra == 'docs'
 Provides-Extra: uvloop
-Requires-Dist: furo (>=2023.9.10,<2024.0.0) ; extra == "docs"
-Requires-Dist: sphinx (>=7.0.0,<8.0.0) ; extra == "docs"
-Requires-Dist: sphinx-copybutton (>=0.5.0,<0.6.0) ; extra == "docs"
-Requires-Dist: sphinx_autodoc_typehints (>=1.24.0,<2.0.0) ; extra == "docs"
-Requires-Dist: uvloop (>=0.18,<0.19) ; extra == "uvloop"
-Project-URL: Documentation, https://aiosmtplib.readthedocs.io/en/stable/
-Project-URL: Repository, https://github.com/cole/aiosmtplib
+Requires-Dist: uvloop>=0.18; extra == 'uvloop'
 Description-Content-Type: text/x-rst
 
 aiosmtplib
@@ -117,4 +111,3 @@ Bug reports (and feature requests) are w
                   :target: https://results.pre-commit.ci/latest/github/cole/aiosmtplib/main
                   :alt: "pre-commit.ci status"
 .. _Read The Docs: https://aiosmtplib.readthedocs.io/en/stable/
-
diff -pruN 3.0.1-1/aiosmtplib/__init__.py 3.0.2-1/aiosmtplib/__init__.py
--- 3.0.1-1/aiosmtplib/__init__.py	2023-11-02 18:36:16.215468600 +0000
+++ 3.0.2-1/aiosmtplib/__init__.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,60 +0,0 @@
-"""
-aiosmtplib
-==========
-
-An asyncio SMTP client.
-
-Originally based on smtplib from the Python 3 standard library by:
-The Dragon De Monsyne <dragondm@integral.org>
-
-Author: Cole Maclean <hi@colemaclean.dev>
-"""
-from .api import send
-from .errors import (
-    SMTPAuthenticationError,
-    SMTPConnectError,
-    SMTPConnectTimeoutError,
-    SMTPDataError,
-    SMTPException,
-    SMTPHeloError,
-    SMTPNotSupported,
-    SMTPReadTimeoutError,
-    SMTPRecipientRefused,
-    SMTPRecipientsRefused,
-    SMTPResponseException,
-    SMTPSenderRefused,
-    SMTPServerDisconnected,
-    SMTPTimeoutError,
-    SMTPConnectResponseError,
-)
-from .response import SMTPResponse
-from .smtp import SMTP
-from .typing import SMTPStatus
-
-
-__title__ = "aiosmtplib"
-__version__ = "3.0.1"
-__author__ = "Cole Maclean"
-__license__ = "MIT"
-__copyright__ = "Copyright 2022 Cole Maclean"
-__all__ = (
-    "send",
-    "SMTP",
-    "SMTPResponse",
-    "SMTPStatus",
-    "SMTPAuthenticationError",
-    "SMTPConnectError",
-    "SMTPDataError",
-    "SMTPException",
-    "SMTPHeloError",
-    "SMTPNotSupported",
-    "SMTPRecipientRefused",
-    "SMTPRecipientsRefused",
-    "SMTPResponseException",
-    "SMTPSenderRefused",
-    "SMTPServerDisconnected",
-    "SMTPTimeoutError",
-    "SMTPConnectTimeoutError",
-    "SMTPReadTimeoutError",
-    "SMTPConnectResponseError",
-)
diff -pruN 3.0.1-1/aiosmtplib/__main__.py 3.0.2-1/aiosmtplib/__main__.py
--- 3.0.1-1/aiosmtplib/__main__.py	2023-11-02 18:36:16.215468600 +0000
+++ 3.0.2-1/aiosmtplib/__main__.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,30 +0,0 @@
-from aiosmtplib.smtp import SMTP, SMTP_PORT
-
-
-raw_hostname = input("SMTP server hostname [localhost]: ")  # nosec
-raw_port = input(f"SMTP server port [{SMTP_PORT}]: ")  # nosec
-raw_sender = input("From: ")  # nosec
-raw_recipients = input("To: ")  # nosec
-
-hostname = raw_hostname or "localhost"
-port = int(raw_port) if raw_port else SMTP_PORT
-recipients = raw_recipients.split(",")
-lines = []
-
-print("Enter message, end with ^D:")
-while True:
-    try:
-        lines.append(input())  # nosec
-    except EOFError:
-        break
-
-message = "\n".join(lines)
-message_len = len(message.encode("utf-8"))
-print(f"Message length (bytes): {message_len}")
-
-smtp_client = SMTP(hostname=hostname or "localhost", port=port, start_tls=False)
-sendmail_errors, sendmail_response = smtp_client.sendmail_sync(
-    raw_sender, recipients, message
-)
-
-print(f"Server response: {sendmail_response}")
diff -pruN 3.0.1-1/aiosmtplib/api.py 3.0.2-1/aiosmtplib/api.py
--- 3.0.1-1/aiosmtplib/api.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/api.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,139 +0,0 @@
-"""
-Main public API.
-"""
-import email.message
-import socket
-import ssl
-from typing import Dict, Optional, Sequence, Tuple, Union, cast
-
-from .response import SMTPResponse
-from .smtp import DEFAULT_TIMEOUT, SMTP
-from .typing import SocketPathType
-
-
-__all__ = ("send",)
-
-
-async def send(
-    message: Union[email.message.EmailMessage, email.message.Message, str, bytes],
-    /,
-    *,
-    sender: Optional[str] = None,
-    recipients: Optional[Union[str, Sequence[str]]] = None,
-    mail_options: Optional[Sequence[str]] = None,
-    rcpt_options: Optional[Sequence[str]] = None,
-    hostname: Optional[str] = "localhost",
-    port: Optional[int] = None,
-    username: Optional[Union[str, bytes]] = None,
-    password: Optional[Union[str, bytes]] = None,
-    local_hostname: Optional[str] = None,
-    source_address: Optional[Tuple[str, int]] = None,
-    timeout: Optional[float] = DEFAULT_TIMEOUT,
-    use_tls: bool = False,
-    start_tls: Optional[bool] = None,
-    validate_certs: bool = True,
-    client_cert: Optional[str] = None,
-    client_key: Optional[str] = None,
-    tls_context: Optional[ssl.SSLContext] = None,
-    cert_bundle: Optional[str] = None,
-    socket_path: Optional[SocketPathType] = None,
-    sock: Optional[socket.socket] = None,
-) -> Tuple[Dict[str, SMTPResponse], str]:
-    """
-    Send an email message. On await, connects to the SMTP server using the details
-    provided, sends the message, then disconnects.
-
-    :param message:  Message text. Either an :py:class:`email.message.EmailMessage`
-        object, ``str`` or ``bytes``. If an :py:class:`email.message.EmailMessage`
-        object is provided, sender and recipients set in the message headers will be
-        used, unless overridden by the respective keyword arguments.
-    :keyword sender:  From email address. Not required if an
-        :py:class:`email.message.EmailMessage` object is provided for the `message`
-        argument.
-    :keyword recipients: Recipient email addresses. Not required if an
-        :py:class:`email.message.EmailMessage` object is provided for the `message`
-        argument.
-    :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
-    :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
-        ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
-    :keyword username:  Username to login as after connect.
-    :keyword password:  Password for login after connect.
-    :keyword local_hostname: The hostname of the client.  If specified, used as the
-        FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
-        :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
-    :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
-        as its source address before connecting. If the host is '' and port is 0,
-        the OS default behavior will be used.
-    :keyword timeout: Default timeout value for the connection, in seconds.
-        Defaults to 60.
-    :keyword use_tls: If True, make the initial connection to the server
-        over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
-        STARTTLS, ``use_tls`` should be ``False``.
-    :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
-        If ``None`` (the default), upgrade will be initiated if supported by the
-        server.
-        If ``True``, and upgrade will be initiated regardless of server support.
-        If ``False``, no upgrade will occur.
-        Mutually exclusive with ``use_tls``.
-    :keyword validate_certs: Determines if server certificates are
-        validated. Defaults to ``True``.
-    :keyword client_cert: Path to client side certificate, for TLS.
-    :keyword client_key: Path to client side key, for TLS.
-    :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
-        Mutually exclusive with ``client_cert``/``client_key``.
-    :keyword cert_bundle: Path to certificate bundle, for TLS verification.
-    :keyword socket_path: Path to a Unix domain socket. Not compatible with
-        hostname or port. Accepts str, bytes, or a pathlike object.
-    :keyword sock: An existing, connected socket object. If given, none of
-        hostname, port, or socket_path should be provided.
-
-    :raises ValueError: required arguments missing or mutually exclusive options
-        provided
-    """
-    if not isinstance(message, (email.message.EmailMessage, email.message.Message)):
-        if not recipients:
-            raise ValueError("Recipients must be provided with raw messages.")
-        if not sender:
-            raise ValueError("Sender must be provided with raw messages.")
-
-    sender = cast(str, sender)
-    recipients = cast(Union[str, Sequence[str]], recipients)
-
-    client = SMTP(
-        hostname=hostname,
-        port=port,
-        local_hostname=local_hostname,
-        source_address=source_address,
-        timeout=timeout,
-        use_tls=use_tls,
-        start_tls=start_tls,
-        validate_certs=validate_certs,
-        client_cert=client_cert,
-        client_key=client_key,
-        tls_context=tls_context,
-        cert_bundle=cert_bundle,
-        socket_path=socket_path,
-        sock=sock,
-        username=username,
-        password=password,
-    )
-
-    async with client:
-        if isinstance(message, (email.message.EmailMessage, email.message.Message)):
-            result = await client.send_message(
-                message,
-                sender=sender,
-                recipients=recipients,
-                mail_options=mail_options,
-                rcpt_options=rcpt_options,
-            )
-        else:
-            result = await client.sendmail(
-                sender,
-                recipients,
-                message,
-                mail_options=mail_options,
-                rcpt_options=rcpt_options,
-            )
-
-    return result
diff -pruN 3.0.1-1/aiosmtplib/auth.py 3.0.2-1/aiosmtplib/auth.py
--- 3.0.1-1/aiosmtplib/auth.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/auth.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,72 +0,0 @@
-"""
-Authentication related methods.
-"""
-import base64
-import hmac
-from typing import Tuple, Union
-
-
-__all__ = ("auth_crammd5_verify", "auth_plain_encode", "auth_login_encode")
-
-
-def _ensure_bytes(value: Union[str, bytes]) -> bytes:
-    if isinstance(value, bytes):
-        return value
-
-    return value.encode("utf-8")
-
-
-def auth_crammd5_verify(
-    username: Union[str, bytes],
-    password: Union[str, bytes],
-    challenge: Union[str, bytes],
-    /,
-) -> bytes:
-    """
-    CRAM-MD5 auth uses the password as a shared secret to MD5 the server's
-    response, and sends the username combined with that (base64 encoded).
-    """
-    username_bytes = _ensure_bytes(username)
-    password_bytes = _ensure_bytes(password)
-    decoded_challenge = base64.b64decode(challenge)
-
-    md5_digest = hmac.new(password_bytes, msg=decoded_challenge, digestmod="md5")
-    verification = username_bytes + b" " + md5_digest.hexdigest().encode("ascii")
-    encoded_verification = base64.b64encode(verification)
-
-    return encoded_verification
-
-
-def auth_plain_encode(
-    username: Union[str, bytes],
-    password: Union[str, bytes],
-    /,
-) -> bytes:
-    """
-    PLAIN auth base64 encodes the username and password together.
-    """
-    username_bytes = _ensure_bytes(username)
-    password_bytes = _ensure_bytes(password)
-
-    username_and_password = b"\0" + username_bytes + b"\0" + password_bytes
-    encoded = base64.b64encode(username_and_password)
-
-    return encoded
-
-
-def auth_login_encode(
-    username: Union[str, bytes],
-    password: Union[str, bytes],
-    /,
-) -> Tuple[bytes, bytes]:
-    """
-    LOGIN auth base64 encodes the username and password and sends them
-    in sequence.
-    """
-    username_bytes = _ensure_bytes(username)
-    password_bytes = _ensure_bytes(password)
-
-    encoded_username = base64.b64encode(username_bytes)
-    encoded_password = base64.b64encode(password_bytes)
-
-    return encoded_username, encoded_password
diff -pruN 3.0.1-1/aiosmtplib/email.py 3.0.2-1/aiosmtplib/email.py
--- 3.0.1-1/aiosmtplib/email.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/email.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,186 +0,0 @@
-"""
-Email message and address formatting/parsing functions.
-"""
-import copy
-import email.charset
-import email.generator
-import email.header
-import email.headerregistry
-import email.message
-import email.policy
-import email.utils
-import io
-import re
-from typing import List, Optional, Tuple, Union
-
-
-__all__ = (
-    "extract_recipients",
-    "extract_sender",
-    "flatten_message",
-    "parse_address",
-    "quote_address",
-)
-
-
-LINE_SEP = "\r\n"
-SPECIALS_REGEX = re.compile(r'[][\\()<>@,:;".]')
-ESCAPES_REGEX = re.compile(r'[\\"]')
-UTF8_CHARSET = email.charset.Charset("utf-8")
-
-
-def parse_address(address: str) -> str:
-    """
-    Parse an email address, falling back to the raw string given.
-    """
-    display_name, parsed_address = email.utils.parseaddr(address)
-
-    return parsed_address or address.strip()
-
-
-def quote_address(address: str) -> str:
-    """
-    Quote a subset of the email addresses defined by RFC 821.
-    """
-    parsed_address = parse_address(address)
-    return f"<{parsed_address}>"
-
-
-def formataddr(pair: Tuple[str, str]) -> str:
-    """
-    Copied from the standard library, and modified to handle international (UTF-8)
-    email addresses.
-
-    The inverse of parseaddr(), this takes a 2-tuple of the form
-    (realname, email_address) and returns the string value suitable
-    for an RFC 2822 From, To or Cc header.
-    If the first element of pair is false, then the second element is
-    returned unmodified.
-    """
-    name, address = pair
-    if name:
-        encoded_name = UTF8_CHARSET.header_encode(name)
-        return f"{encoded_name} <{address}>"
-    else:
-        quotes = ""
-        if SPECIALS_REGEX.search(name):
-            quotes = '"'
-            name = ESCAPES_REGEX.sub(r"\\\g<0>", name)
-            return f"{quotes}{name}{quotes} <{address}>"
-
-    return address
-
-
-def flatten_message(
-    message: Union[email.message.EmailMessage, email.message.Message],
-    /,
-    *,
-    utf8: bool = False,
-    cte_type: str = "8bit",
-) -> bytes:
-    # Make a local copy so we can delete the bcc headers.
-    message_copy = copy.copy(message)
-    del message_copy["Bcc"]
-    del message_copy["Resent-Bcc"]
-
-    if isinstance(message, email.message.EmailMessage):
-        # New message class, default policy
-        policy = email.policy.default.clone(
-            linesep=LINE_SEP,
-            utf8=utf8,
-            cte_type=cte_type,
-        )
-    else:
-        # Old message class, Compat32 policy.
-        # Compat32 cannot use UTF8
-        policy = email.policy.compat32.clone(linesep=LINE_SEP, cte_type=cte_type)
-
-    with io.BytesIO() as messageio:
-        generator = email.generator.BytesGenerator(messageio, policy=policy)
-        generator.flatten(message_copy)
-        flat_message = messageio.getvalue()
-
-    return flat_message
-
-
-def extract_addresses(
-    header: Union[str, email.headerregistry.AddressHeader, email.header.Header],
-    /,
-) -> List[str]:
-    """
-    Convert address headers into raw email addresses, suitable for use in
-    low level SMTP commands.
-    """
-    addresses = []
-    if isinstance(header, email.headerregistry.AddressHeader):
-        for address in header.addresses:
-            # If the object has been assigned an iterable, it's possible to get
-            # a string here
-            if isinstance(address, email.headerregistry.Address):
-                addresses.append(address.addr_spec)
-            else:
-                addresses.append(parse_address(address))
-    elif isinstance(header, email.header.Header):
-        for address_bytes, charset in email.header.decode_header(header):
-            if charset is None:
-                charset = "ascii"
-            addresses.append(parse_address(str(address_bytes, encoding=charset)))
-    else:
-        addresses.extend(addr for _, addr in email.utils.getaddresses([header]))
-
-    return addresses
-
-
-def extract_sender(
-    message: Union[email.message.EmailMessage, email.message.Message],
-    /,
-) -> Optional[str]:
-    """
-    Extract the sender from the message object given.
-    """
-    resent_dates = message.get_all("Resent-Date")
-
-    if resent_dates is not None and len(resent_dates) > 1:
-        raise ValueError("Message has more than one 'Resent-' header block")
-    elif resent_dates:
-        sender_header_name = "Resent-Sender"
-        from_header_name = "Resent-From"
-    else:
-        sender_header_name = "Sender"
-        from_header_name = "From"
-
-    # Prefer the sender field per RFC 2822:3.6.2.
-    if sender_header_name in message:
-        sender_header = message[sender_header_name]
-    else:
-        sender_header = message[from_header_name]
-
-    if sender_header is None:
-        return None
-
-    return extract_addresses(sender_header)[0]
-
-
-def extract_recipients(
-    message: Union[email.message.EmailMessage, email.message.Message],
-    /,
-) -> List[str]:
-    """
-    Extract the recipients from the message object given.
-    """
-    recipients: List[str] = []
-
-    resent_dates = message.get_all("Resent-Date")
-
-    if resent_dates is not None and len(resent_dates) > 1:
-        raise ValueError("Message has more than one 'Resent-' header block")
-    elif resent_dates:
-        recipient_headers = ("Resent-To", "Resent-Cc", "Resent-Bcc")
-    else:
-        recipient_headers = ("To", "Cc", "Bcc")
-
-    for header in recipient_headers:
-        for recipient in message.get_all(header, failobj=[]):
-            recipients.extend(extract_addresses(recipient))
-
-    return recipients
diff -pruN 3.0.1-1/aiosmtplib/errors.py 3.0.2-1/aiosmtplib/errors.py
--- 3.0.1-1/aiosmtplib/errors.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/errors.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,137 +0,0 @@
-from asyncio import TimeoutError
-from typing import List
-
-
-__all__ = (
-    "SMTPAuthenticationError",
-    "SMTPConnectError",
-    "SMTPDataError",
-    "SMTPException",
-    "SMTPHeloError",
-    "SMTPNotSupported",
-    "SMTPRecipientRefused",
-    "SMTPRecipientsRefused",
-    "SMTPResponseException",
-    "SMTPSenderRefused",
-    "SMTPServerDisconnected",
-    "SMTPTimeoutError",
-    "SMTPConnectTimeoutError",
-    "SMTPReadTimeoutError",
-    "SMTPConnectResponseError",
-)
-
-
-class SMTPException(Exception):
-    """
-    Base class for all SMTP exceptions.
-    """
-
-    def __init__(self, message: str, /) -> None:
-        self.message = message
-        self.args = (message,)
-
-
-class SMTPServerDisconnected(SMTPException, ConnectionError):
-    """
-    The connection was lost unexpectedly, or a command was run that requires
-    a connection.
-    """
-
-
-class SMTPConnectError(SMTPException, ConnectionError):
-    """
-    An error occurred while connecting to the SMTP server.
-    """
-
-
-class SMTPTimeoutError(SMTPException, TimeoutError):
-    """
-    A timeout occurred while performing a network operation.
-    """
-
-
-class SMTPConnectTimeoutError(SMTPTimeoutError, SMTPConnectError):
-    """
-    A timeout occurred while connecting to the SMTP server.
-    """
-
-
-class SMTPReadTimeoutError(SMTPTimeoutError):
-    """
-    A timeout occurred while waiting for a response from the SMTP server.
-    """
-
-
-class SMTPNotSupported(SMTPException):
-    """
-    A command or argument sent to the SMTP server is not supported.
-    """
-
-
-class SMTPResponseException(SMTPException):
-    """
-    Base class for all server responses with error codes.
-    """
-
-    def __init__(self, code: int, message: str, /) -> None:
-        self.code = code
-        self.message = message
-        self.args = (code, message)
-
-
-class SMTPConnectResponseError(SMTPResponseException, SMTPConnectError):
-    """
-    The SMTP server returned an invalid response code after connecting.
-    """
-
-
-class SMTPHeloError(SMTPResponseException):
-    """
-    Server refused HELO or EHLO.
-    """
-
-
-class SMTPDataError(SMTPResponseException):
-    """
-    Server refused DATA content.
-    """
-
-
-class SMTPAuthenticationError(SMTPResponseException):
-    """
-    Server refused our AUTH request; may be caused by invalid credentials.
-    """
-
-
-class SMTPSenderRefused(SMTPResponseException):
-    """
-    SMTP server refused the message sender.
-    """
-
-    def __init__(self, code: int, message: str, sender: str, /) -> None:
-        self.code = code
-        self.message = message
-        self.sender = sender
-        self.args = (code, message, sender)
-
-
-class SMTPRecipientRefused(SMTPResponseException):
-    """
-    SMTP server refused a message recipient.
-    """
-
-    def __init__(self, code: int, message: str, recipient: str, /) -> None:
-        self.code = code
-        self.message = message
-        self.recipient = recipient
-        self.args = (code, message, recipient)
-
-
-class SMTPRecipientsRefused(SMTPException):
-    """
-    SMTP server refused multiple recipients.
-    """
-
-    def __init__(self, recipients: List[SMTPRecipientRefused], /) -> None:
-        self.recipients = recipients
-        self.args = (recipients,)
diff -pruN 3.0.1-1/aiosmtplib/esmtp.py 3.0.2-1/aiosmtplib/esmtp.py
--- 3.0.1-1/aiosmtplib/esmtp.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/esmtp.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,72 +0,0 @@
-"""
-ESMTP utils
-"""
-import re
-from typing import Dict, List, Tuple
-
-
-__all__ = ("parse_esmtp_extensions",)
-
-
-OLDSTYLE_AUTH_REGEX = re.compile(r"auth=(?P<auth>.*)", flags=re.I)
-EXTENSIONS_REGEX = re.compile(r"(?P<ext>[A-Za-z0-9][A-Za-z0-9\-]*) ?")
-
-
-def parse_esmtp_extensions(message: str) -> Tuple[Dict[str, str], List[str]]:
-    """
-    Parse an EHLO response from the server into a dict of {extension: params}
-    and a list of auth method names.
-
-    It might look something like:
-
-         220 size.does.matter.af.MIL (More ESMTP than Crappysoft!)
-         EHLO heaven.af.mil
-         250-size.does.matter.af.MIL offers FIFTEEN extensions:
-         250-8BITMIME
-         250-PIPELINING
-         250-DSN
-         250-ENHANCEDSTATUSCODES
-         250-EXPN
-         250-HELP
-         250-SAML
-         250-SEND
-         250-SOML
-         250-TURN
-         250-XADR
-         250-XSTA
-         250-ETRN
-         250-XGEN
-         250 SIZE 51200000
-    """
-    esmtp_extensions = {}
-    auth_types: List[str] = []
-
-    response_lines = message.split("\n")
-
-    # ignore the first line
-    for line in response_lines[1:]:
-        # To be able to communicate with as many SMTP servers as possible,
-        # we have to take the old-style auth advertisement into account,
-        # because:
-        # 1) Else our SMTP feature parser gets confused.
-        # 2) There are some servers that only advertise the auth methods we
-        #    support using the old style.
-        auth_match = OLDSTYLE_AUTH_REGEX.match(line)
-        if auth_match is not None:
-            auth_type = auth_match.group("auth")
-            auth_types.append(auth_type.lower().strip())
-
-        # RFC 1869 requires a space between ehlo keyword and parameters.
-        # It's actually stricter, in that only spaces are allowed between
-        # parameters, but were not going to check for that here.  Note
-        # that the space isn't present if there are no parameters.
-        extensions = EXTENSIONS_REGEX.match(line)
-        if extensions is not None:
-            extension = extensions.group("ext").lower()
-            params = extensions.string[extensions.end("ext") :].strip()
-            esmtp_extensions[extension] = params
-
-            if extension == "auth":
-                auth_types.extend([param.strip().lower() for param in params.split()])
-
-    return esmtp_extensions, auth_types
diff -pruN 3.0.1-1/aiosmtplib/protocol.py 3.0.2-1/aiosmtplib/protocol.py
--- 3.0.1-1/aiosmtplib/protocol.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/protocol.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,373 +0,0 @@
-"""
-An ``asyncio.Protocol`` subclass for lower level IO handling.
-"""
-import asyncio
-import collections
-import re
-import ssl
-from typing import Deque, Optional, cast
-
-from .errors import (
-    SMTPDataError,
-    SMTPReadTimeoutError,
-    SMTPResponseException,
-    SMTPServerDisconnected,
-    SMTPTimeoutError,
-)
-from .response import SMTPResponse
-from .typing import SMTPStatus
-
-
-__all__ = ("SMTPProtocol",)
-
-
-MAX_LINE_LENGTH = 8192
-LINE_ENDINGS_REGEX = re.compile(rb"(?:\r\n|\n|\r(?!\n))")
-PERIOD_REGEX = re.compile(rb"(?m)^\.")
-
-
-class FlowControlMixin(asyncio.Protocol):
-    """
-    Reusable flow control logic for StreamWriter.drain().
-    This implements the protocol methods pause_writing(),
-    resume_writing() and connection_lost().  If the subclass overrides
-    these it must call the super methods.
-    StreamWriter.drain() must wait for _drain_helper() coroutine.
-
-    Copied from stdlib as per recommendation: https://bugs.python.org/msg343685.
-    Logging and asserts removed, type annotations added.
-    """
-
-    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None):
-        if loop is None:
-            self._loop = asyncio.get_event_loop()
-        else:
-            self._loop = loop
-
-        self._paused = False
-        self._drain_waiters: Deque[asyncio.Future[None]] = collections.deque()
-        self._connection_lost = False
-
-    def pause_writing(self) -> None:
-        self._paused = True
-
-    def resume_writing(self) -> None:
-        self._paused = False
-
-        for waiter in self._drain_waiters:
-            if not waiter.done():
-                waiter.set_result(None)
-
-    def connection_lost(self, exc: Optional[Exception]) -> None:
-        self._connection_lost = True
-        # Wake up the writer(s) if currently paused.
-        if not self._paused:
-            return
-
-        for waiter in self._drain_waiters:
-            if not waiter.done():
-                if exc is None:
-                    waiter.set_result(None)
-                else:
-                    waiter.set_exception(exc)
-
-    async def _drain_helper(self) -> None:
-        if self._connection_lost:
-            raise ConnectionResetError("Connection lost")
-        if not self._paused:
-            return
-        waiter = self._loop.create_future()
-        self._drain_waiters.append(waiter)
-        try:
-            await waiter
-        finally:
-            self._drain_waiters.remove(waiter)
-
-    def _get_close_waiter(self, stream: asyncio.StreamWriter) -> "asyncio.Future[None]":
-        raise NotImplementedError
-
-
-class SMTPProtocol(FlowControlMixin, asyncio.BaseProtocol):
-    def __init__(
-        self,
-        loop: Optional[asyncio.AbstractEventLoop] = None,
-    ) -> None:
-        super().__init__(loop=loop)
-        self._over_ssl = False
-        self._buffer = bytearray()
-        self._response_waiter: Optional[asyncio.Future[SMTPResponse]] = None
-
-        self.transport: Optional[asyncio.BaseTransport] = None
-        self._command_lock: Optional[asyncio.Lock] = None
-        self._closed: "asyncio.Future[None]" = self._loop.create_future()
-        self._quit_sent = False
-
-    def _get_close_waiter(self, stream: asyncio.StreamWriter) -> "asyncio.Future[None]":
-        return self._closed
-
-    def __del__(self) -> None:
-        # Avoid 'Future exception was never retrieved' warnings
-        # Some unknown race conditions can sometimes trigger these :(
-        self._retrieve_response_exception()
-
-    @property
-    def is_connected(self) -> bool:
-        """
-        Check if our transport is still connected.
-        """
-        return bool(self.transport is not None and not self.transport.is_closing())
-
-    def connection_made(self, transport: asyncio.BaseTransport) -> None:
-        self.transport = cast(asyncio.Transport, transport)
-        self._over_ssl = transport.get_extra_info("sslcontext") is not None
-        self._response_waiter = self._loop.create_future()
-        self._command_lock = asyncio.Lock()
-        self._quit_sent = False
-
-    def connection_lost(self, exc: Optional[Exception]) -> None:
-        super().connection_lost(exc)
-
-        if not self._quit_sent:
-            smtp_exc = SMTPServerDisconnected("Connection lost")
-            if exc:
-                smtp_exc.__cause__ = exc
-
-            if self._response_waiter and not self._response_waiter.done():
-                self._response_waiter.set_exception(smtp_exc)
-
-        self.transport = None
-        self._command_lock = None
-
-    def data_received(self, data: bytes) -> None:
-        if self._response_waiter is None:
-            raise RuntimeError(
-                f"data_received called without a response waiter set: {data!r}"
-            )
-        elif self._response_waiter.done():
-            # We got a response without issuing a command; ignore it.
-            return
-
-        self._buffer.extend(data)
-
-        # If we got an obvious partial message, don't try to parse the buffer
-        last_linebreak = data.rfind(b"\n")
-        if (
-            last_linebreak == -1
-            or data[last_linebreak + 3 : last_linebreak + 4] == b"-"
-        ):
-            return
-
-        try:
-            response = self._read_response_from_buffer()
-        except Exception as exc:
-            self._response_waiter.set_exception(exc)
-        else:
-            if response is not None:
-                self._response_waiter.set_result(response)
-
-    def eof_received(self) -> bool:
-        exc = SMTPServerDisconnected("Unexpected EOF received")
-        if self._response_waiter and not self._response_waiter.done():
-            self._response_waiter.set_exception(exc)
-
-        # Returning false closes the transport
-        return False
-
-    def _retrieve_response_exception(self) -> Optional[BaseException]:
-        """
-        Return any exception that has been set on the response waiter.
-
-        Used to avoid 'Future exception was never retrieved' warnings
-        """
-        if (
-            self._response_waiter
-            and self._response_waiter.done()
-            and not self._response_waiter.cancelled()
-        ):
-            return self._response_waiter.exception()
-
-        return None
-
-    def _read_response_from_buffer(self) -> Optional[SMTPResponse]:
-        """Parse the actual response (if any) from the data buffer"""
-        code = -1
-        message = bytearray()
-        offset = 0
-        message_complete = False
-
-        while True:
-            line_end_index = self._buffer.find(b"\n", offset)
-            if line_end_index == -1:
-                break
-
-            line = bytes(self._buffer[offset : line_end_index + 1])
-
-            if len(line) > MAX_LINE_LENGTH:
-                raise SMTPResponseException(
-                    SMTPStatus.unrecognized_command, "Response too long"
-                )
-
-            try:
-                code = int(line[:3])
-            except ValueError:
-                raise SMTPResponseException(
-                    SMTPStatus.invalid_response.value,
-                    f"Malformed SMTP response line: {line!r}",
-                ) from None
-
-            offset += len(line)
-            if len(message):
-                message.extend(b"\n")
-            message.extend(line[4:].strip(b" \t\r\n"))
-            if line[3:4] != b"-":
-                message_complete = True
-                break
-
-        if message_complete:
-            response = SMTPResponse(
-                code, bytes(message).decode("utf-8", "surrogateescape")
-            )
-            del self._buffer[:offset]
-            return response
-        else:
-            return None
-
-    async def read_response(self, timeout: Optional[float] = None) -> SMTPResponse:
-        """
-        Get a status response from the server.
-
-        This method must be awaited once per command sent; if multiple commands
-        are written to the transport without awaiting, response data will be lost.
-
-        Returns an :class:`.response.SMTPResponse` namedtuple consisting of:
-          - server response code (e.g. 250, or such, if all goes well)
-          - server response string (multiline responses are converted to a
-            single, multiline string).
-        """
-        if self._response_waiter is None:
-            raise SMTPServerDisconnected("Connection lost")
-
-        try:
-            result = await asyncio.wait_for(self._response_waiter, timeout)
-        except (TimeoutError, asyncio.TimeoutError) as exc:
-            raise SMTPReadTimeoutError("Timed out waiting for server response") from exc
-        finally:
-            # If we were disconnected, don't create a new waiter
-            if self.transport is None:
-                self._response_waiter = None
-            else:
-                self._response_waiter = self._loop.create_future()
-
-        return result
-
-    def write(self, data: bytes) -> None:
-        if self.transport is None or self.transport.is_closing():
-            raise SMTPServerDisconnected("Connection lost")
-        if not hasattr(self.transport, "write"):
-            raise RuntimeError(
-                f"Transport {self.transport!r} does not support writing."
-            )
-
-        self.transport.write(data)  # type: ignore
-
-    async def execute_command(
-        self, *args: bytes, timeout: Optional[float] = None
-    ) -> SMTPResponse:
-        """
-        Sends an SMTP command along with any args to the server, and returns
-        a response.
-        """
-        if self._command_lock is None:
-            raise SMTPServerDisconnected("Server not connected")
-        command = b" ".join(args) + b"\r\n"
-
-        async with self._command_lock:
-            self.write(command)
-
-            if command == b"QUIT\r\n":
-                self._quit_sent = True
-
-            response = await self.read_response(timeout=timeout)
-
-        return response
-
-    async def execute_data_command(
-        self, message: bytes, timeout: Optional[float] = None
-    ) -> SMTPResponse:
-        """
-        Sends an SMTP DATA command to the server, followed by encoded message content.
-
-        Automatically quotes lines beginning with a period per RFC821.
-        Lone \\\\r and \\\\n characters are converted to \\\\r\\\\n
-        characters.
-        """
-        if self._command_lock is None:
-            raise SMTPServerDisconnected("Server not connected")
-
-        message = LINE_ENDINGS_REGEX.sub(b"\r\n", message)
-        message = PERIOD_REGEX.sub(b"..", message)
-        if not message.endswith(b"\r\n"):
-            message += b"\r\n"
-        message += b".\r\n"
-
-        async with self._command_lock:
-            self.write(b"DATA\r\n")
-            start_response = await self.read_response(timeout=timeout)
-            if start_response.code != SMTPStatus.start_input:
-                raise SMTPDataError(start_response.code, start_response.message)
-
-            self.write(message)
-            response = await self.read_response(timeout=timeout)
-            if response.code != SMTPStatus.completed:
-                raise SMTPDataError(response.code, response.message)
-
-        return response
-
-    async def start_tls(
-        self,
-        tls_context: ssl.SSLContext,
-        server_hostname: Optional[str] = None,
-        timeout: Optional[float] = None,
-    ) -> SMTPResponse:
-        """
-        Puts the connection to the SMTP server into TLS mode.
-        """
-        if self._over_ssl:
-            raise RuntimeError("Already using TLS.")
-        if self._command_lock is None:
-            raise SMTPServerDisconnected("Server not connected")
-
-        async with self._command_lock:
-            self.write(b"STARTTLS\r\n")
-            response = await self.read_response(timeout=timeout)
-            if response.code != SMTPStatus.ready:
-                raise SMTPResponseException(response.code, response.message)
-
-            # Check for disconnect after response
-            if self.transport is None or self.transport.is_closing():
-                raise SMTPServerDisconnected("Connection lost")
-
-            try:
-                tls_transport = await self._loop.start_tls(
-                    self.transport,
-                    self,
-                    tls_context,
-                    server_side=False,
-                    server_hostname=server_hostname,
-                    ssl_handshake_timeout=timeout,
-                )
-            except (TimeoutError, asyncio.TimeoutError) as exc:
-                raise SMTPTimeoutError("Timed out while upgrading transport") from exc
-            # SSLProtocol only raises ConnectionAbortedError on timeout
-            except ConnectionAbortedError as exc:
-                raise SMTPTimeoutError(exc.args[0]) from exc
-            except ConnectionResetError as exc:
-                if exc.args:
-                    message = exc.args[0]
-                else:
-                    message = "Connection was reset while upgrading transport"
-                raise SMTPServerDisconnected(message) from exc
-
-            self.transport = tls_transport
-
-        return response
diff -pruN 3.0.1-1/aiosmtplib/py.typed 3.0.2-1/aiosmtplib/py.typed
--- 3.0.1-1/aiosmtplib/py.typed	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/py.typed	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-This file exists to help mypy (and other tools) find inline type hints. See PR #141 and PEP 561.
diff -pruN 3.0.1-1/aiosmtplib/response.py 3.0.2-1/aiosmtplib/response.py
--- 3.0.1-1/aiosmtplib/response.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/response.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,33 +0,0 @@
-"""
-SMTPResponse class, a simple namedtuple of (code, message).
-"""
-from typing import NamedTuple
-
-
-__all__ = ("SMTPResponse",)
-
-
-class SMTPResponse(NamedTuple):
-    """
-    NamedTuple of server response code and server response message.
-
-    ``code`` and ``message`` can be accessed via attributes or indexes:
-
-        >>> response = SMTPResponse(200, "OK")
-        >>> response.message
-        'OK'
-        >>> response[0]
-        200
-        >>> response.code
-        200
-
-    """
-
-    code: int
-    message: str
-
-    def __repr__(self) -> str:
-        return f"({self.code}, {self.message})"
-
-    def __str__(self) -> str:
-        return f"{self.code} {self.message}"
diff -pruN 3.0.1-1/aiosmtplib/smtp.py 3.0.2-1/aiosmtplib/smtp.py
--- 3.0.1-1/aiosmtplib/smtp.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/smtp.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,1459 +0,0 @@
-"""
-Main SMTP client class.
-
-Implements SMTP, ESMTP & Auth methods.
-"""
-import asyncio
-import email.message
-import socket
-import ssl
-from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Type, Union
-
-from .auth import auth_crammd5_verify, auth_login_encode, auth_plain_encode
-from .email import (
-    extract_recipients,
-    extract_sender,
-    flatten_message,
-    parse_address,
-    quote_address,
-)
-from .errors import (
-    SMTPAuthenticationError,
-    SMTPConnectError,
-    SMTPConnectTimeoutError,
-    SMTPException,
-    SMTPHeloError,
-    SMTPNotSupported,
-    SMTPRecipientRefused,
-    SMTPRecipientsRefused,
-    SMTPResponseException,
-    SMTPSenderRefused,
-    SMTPServerDisconnected,
-    SMTPTimeoutError,
-    SMTPConnectResponseError,
-)
-from .esmtp import parse_esmtp_extensions
-from .protocol import SMTPProtocol
-from .response import SMTPResponse
-from .typing import Default, SMTPStatus, SocketPathType, _default
-
-
-__all__ = ("SMTP", "SMTP_PORT", "SMTP_TLS_PORT", "SMTP_STARTTLS_PORT")
-
-SMTP_PORT = 25
-SMTP_TLS_PORT = 465
-SMTP_STARTTLS_PORT = 587
-DEFAULT_TIMEOUT = 60
-
-
-class SMTP:
-    """
-    Main SMTP client class.
-
-    Basic usage:
-
-        >>> event_loop = asyncio.get_event_loop()
-        >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
-        >>> event_loop.run_until_complete(smtp.connect())
-        (220, ...)
-        >>> sender = "root@localhost"
-        >>> recipients = ["somebody@localhost"]
-        >>> message = "Hello World"
-        >>> send = smtp.sendmail(sender, recipients, "Hello World")
-        >>> event_loop.run_until_complete(send)
-        ({}, 'OK')
-
-    Keyword arguments can be provided either on :meth:`__init__` or when
-    calling the :meth:`connect` method. Note that in both cases these options,
-    except for ``timeout``, are saved for later use; subsequent calls to
-    :meth:`connect` will use the same options, unless new ones are provided.
-    ``timeout`` is saved for later use when provided on :meth:`__init__`, but
-    not when calling the :meth:`connect` method.
-    """
-
-    # Preferred methods first
-    AUTH_METHODS: Tuple[str, ...] = (
-        "cram-md5",
-        "plain",
-        "login",
-    )
-
-    def __init__(
-        self,
-        *,
-        hostname: Optional[str] = "localhost",
-        port: Optional[int] = None,
-        username: Optional[Union[str, bytes]] = None,
-        password: Optional[Union[str, bytes]] = None,
-        local_hostname: Optional[str] = None,
-        source_address: Optional[Tuple[str, int]] = None,
-        timeout: Optional[float] = DEFAULT_TIMEOUT,
-        use_tls: bool = False,
-        start_tls: Optional[bool] = None,
-        validate_certs: bool = True,
-        client_cert: Optional[str] = None,
-        client_key: Optional[str] = None,
-        tls_context: Optional[ssl.SSLContext] = None,
-        cert_bundle: Optional[str] = None,
-        socket_path: Optional[SocketPathType] = None,
-        sock: Optional[socket.socket] = None,
-    ) -> None:
-        """
-        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
-        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
-            ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
-        :keyword username:  Username to login as after connect.
-        :keyword password:  Password for login after connect.
-        :keyword local_hostname: The hostname of the client.  If specified, used as the
-            FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
-            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
-        :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
-            as its source address before connecting. If the host is '' and port is 0,
-            the OS default behavior will be used.
-        :keyword timeout: Default timeout value for the connection, in seconds.
-            Defaults to 60.
-        :keyword use_tls: If True, make the initial connection to the server
-            over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
-            STARTTLS, ``use_tls`` should be ``False``.
-        :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
-            If ``None`` (the default), upgrade will be initiated if supported by the
-            server.
-            If ``True``, and upgrade will be initiated regardless of server support.
-            If ``False``, no upgrade will occur.
-            Mutually exclusive with ``use_tls``.
-        :keyword validate_certs: Determines if server certificates are
-            validated. Defaults to ``True``.
-        :keyword client_cert: Path to client side certificate, for TLS.
-        :keyword client_key: Path to client side key, for TLS.
-        :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
-            Mutually exclusive with ``client_cert``/``client_key``.
-        :keyword cert_bundle: Path to certificate bundle, for TLS verification.
-        :keyword socket_path: Path to a Unix domain socket. Not compatible with
-            hostname or port. Accepts str, bytes, or a pathlike object.
-        :keyword sock: An existing, connected socket object. If given, none of
-            hostname, port, or socket_path should be provided.
-
-        :raises ValueError: mutually exclusive options provided
-        """
-        self.protocol: Optional[SMTPProtocol] = None
-        self.transport: Optional[asyncio.BaseTransport] = None
-
-        # Kwarg defaults are provided here, and saved for connect.
-        self.hostname = hostname
-        self.port = port
-        self._login_username = username
-        self._login_password = password
-        self._local_hostname = local_hostname
-        self.timeout = timeout
-        self.use_tls = use_tls
-        self._start_tls_on_connect = start_tls
-        self.validate_certs = validate_certs
-        self.client_cert = client_cert
-        self.client_key = client_key
-        self.tls_context = tls_context
-        self.cert_bundle = cert_bundle
-        self.socket_path = socket_path
-        self.sock = sock
-        self.source_address = source_address
-
-        self.loop: Optional[asyncio.AbstractEventLoop] = None
-        self._connect_lock: Optional[asyncio.Lock] = None
-        self.last_helo_response: Optional[SMTPResponse] = None
-        self._last_ehlo_response: Optional[SMTPResponse] = None
-        self.esmtp_extensions: Dict[str, str] = {}
-        self.supports_esmtp = False
-        self.server_auth_methods: List[str] = []
-        self._sendmail_lock: Optional[asyncio.Lock] = None
-
-        self._validate_config()
-
-    async def __aenter__(self) -> "SMTP":
-        if not self.is_connected:
-            await self.connect()
-
-        return self
-
-    async def __aexit__(
-        self, exc_type: Type[BaseException], exc: BaseException, traceback: Any
-    ) -> None:
-        if isinstance(exc, (ConnectionError, TimeoutError)):
-            self.close()
-            return
-
-        try:
-            await self.quit()
-        except (SMTPServerDisconnected, SMTPResponseException, SMTPTimeoutError):
-            pass
-
-    @property
-    def is_connected(self) -> bool:
-        """
-        Check if our transport is still connected.
-        """
-        return bool(self.protocol is not None and self.protocol.is_connected)
-
-    @property
-    def local_hostname(self) -> str:
-        """
-        Get the system hostname to be sent to the SMTP server.
-        Simply caches the result of :func:`socket.getfqdn`.
-        """
-        if self._local_hostname is None:
-            self._local_hostname = socket.getfqdn()
-
-        return self._local_hostname
-
-    @property
-    def last_ehlo_response(self) -> Union[SMTPResponse, None]:
-        return self._last_ehlo_response
-
-    @last_ehlo_response.setter
-    def last_ehlo_response(self, response: SMTPResponse) -> None:
-        """
-        When setting the last EHLO response, parse the message for supported
-        extensions and auth methods.
-        """
-        extensions, auth_methods = parse_esmtp_extensions(response.message)
-        self._last_ehlo_response = response
-        self.esmtp_extensions = extensions
-        self.server_auth_methods = auth_methods
-        self.supports_esmtp = True
-
-    @property
-    def is_ehlo_or_helo_needed(self) -> bool:
-        """
-        Check if we've already received a response to an EHLO or HELO command.
-        """
-        return self.last_ehlo_response is None and self.last_helo_response is None
-
-    @property
-    def supported_auth_methods(self) -> List[str]:
-        """
-        Get all AUTH methods supported by the both server and by us.
-        """
-        return [auth for auth in self.AUTH_METHODS if auth in self.server_auth_methods]
-
-    def _update_settings_from_kwargs(
-        self,
-        hostname: Optional[Union[str, Default]] = _default,
-        port: Optional[Union[int, Default]] = _default,
-        username: Optional[Union[str, bytes, Default]] = _default,
-        password: Optional[Union[str, bytes, Default]] = _default,
-        local_hostname: Optional[Union[str, Default]] = _default,
-        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
-        use_tls: Optional[bool] = None,
-        start_tls: Optional[Union[bool, Default]] = _default,
-        validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        socket_path: Optional[Union[SocketPathType, Default]] = _default,
-        sock: Optional[Union[socket.socket, Default]] = _default,
-    ) -> None:
-        """Update our configuration from the kwargs provided.
-
-        This method can be called multiple times.
-        """
-        if hostname is not _default:
-            self.hostname = hostname
-        if use_tls is not None:
-            self.use_tls = use_tls
-        if start_tls is not _default:
-            self._start_tls_on_connect = start_tls
-        if validate_certs is not None:
-            self.validate_certs = validate_certs
-        if port is not _default:
-            self.port = port
-        if username is not _default:
-            self._login_username = username
-        if password is not _default:
-            self._login_password = password
-
-        if local_hostname is not _default:
-            self._local_hostname = local_hostname
-        if source_address is not _default:
-            self.source_address = source_address
-        if client_cert is not _default:
-            self.client_cert = client_cert
-        if client_key is not _default:
-            self.client_key = client_key
-        if tls_context is not _default:
-            self.tls_context = tls_context
-        if cert_bundle is not _default:
-            self.cert_bundle = cert_bundle
-        if socket_path is not _default:
-            self.socket_path = socket_path
-        if sock is not _default:
-            self.sock = sock
-
-    def _validate_config(self) -> None:
-        if self._start_tls_on_connect and self.use_tls:
-            raise ValueError("The start_tls and use_tls options are not compatible.")
-
-        if self.tls_context is not None and self.client_cert is not None:
-            raise ValueError(
-                "Either a TLS context or a certificate/key must be provided"
-            )
-
-        if self.sock is not None and any([self.hostname, self.port, self.socket_path]):
-            raise ValueError(
-                "The socket option is not compatible with hostname, port or socket_path"
-            )
-
-        if self.socket_path is not None and any([self.hostname, self.port]):
-            raise ValueError(
-                "The socket_path option is not compatible with hostname/port"
-            )
-
-        if self._local_hostname is not None and (
-            "\r" in self._local_hostname or "\n" in self._local_hostname
-        ):
-            raise ValueError(
-                "The local_hostname param contains prohibited newline characters"
-            )
-
-        if self.hostname is not None and (
-            "\r" in self.hostname or "\n" in self.hostname
-        ):
-            raise ValueError(
-                "The hostname param contains prohibited newline characters"
-            )
-
-    def _get_default_port(self) -> int:
-        """
-        Return an appropriate default port, based on options selected.
-        """
-        if self.use_tls:
-            return SMTP_TLS_PORT
-        elif self._start_tls_on_connect:
-            return SMTP_STARTTLS_PORT
-
-        return SMTP_PORT
-
-    async def connect(
-        self,
-        *,
-        hostname: Optional[Union[str, Default]] = _default,
-        port: Optional[Union[int, Default]] = _default,
-        username: Optional[Union[str, bytes, Default]] = _default,
-        password: Optional[Union[str, bytes, Default]] = _default,
-        local_hostname: Optional[Union[str, Default]] = _default,
-        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
-        timeout: Optional[Union[float, Default]] = _default,
-        use_tls: Optional[bool] = None,
-        start_tls: Optional[Union[bool, Default]] = _default,
-        validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        socket_path: Optional[Union[SocketPathType, Default]] = _default,
-        sock: Optional[Union[socket.socket, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Initialize a connection to the server. Options provided to
-        :meth:`.connect` take precedence over those used to initialize the
-        class.
-
-        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
-        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
-            ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
-        :keyword username:  Username to login as after connect.
-        :keyword password:  Password for login after connect.
-        :keyword local_hostname: The hostname of the client.  If specified, used as the
-            FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
-            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
-        :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
-            as its source address before connecting. If the host is '' and port is 0,
-            the OS default behavior will be used.
-        :keyword timeout: Default timeout value for the connection, in seconds.
-            Defaults to 60.
-        :keyword use_tls: If True, make the initial connection to the server
-            over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
-            STARTTLS, ``use_tls`` should be ``False``.
-        :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
-            If ``None`` (the default), upgrade will be initiated if supported by the
-            server.
-            If ``True``, and upgrade will be initiated regardless of server support.
-            If ``False``, no upgrade will occur.
-            Mutually exclusive with ``use_tls``.
-        :keyword validate_certs: Determines if server certificates are
-            validated. Defaults to ``True``.
-        :keyword client_cert: Path to client side certificate, for TLS.
-        :keyword client_key: Path to client side key, for TLS.
-        :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
-            Mutually exclusive with ``client_cert``/``client_key``.
-        :keyword cert_bundle: Path to certificate bundle, for TLS verification.
-        :keyword socket_path: Path to a Unix domain socket. Not compatible with
-            hostname or port. Accepts str, bytes, or a pathlike object.
-        :keyword sock: An existing, connected socket object. If given, none of
-            hostname, port, or socket_path should be provided.
-
-        :raises ValueError: mutually exclusive options provided
-        """
-        self._update_settings_from_kwargs(
-            hostname=hostname,
-            port=port,
-            local_hostname=local_hostname,
-            source_address=source_address,
-            use_tls=use_tls,
-            start_tls=start_tls,
-            validate_certs=validate_certs,
-            client_cert=client_cert,
-            client_key=client_key,
-            tls_context=tls_context,
-            cert_bundle=cert_bundle,
-            socket_path=socket_path,
-            sock=sock,
-            username=username,
-            password=password,
-        )
-        self._validate_config()
-
-        self.loop = asyncio.get_running_loop()
-        if self._connect_lock is None:
-            self._connect_lock = asyncio.Lock()
-        await self._connect_lock.acquire()
-
-        # Set default port last in case use_tls or start_tls is provided,
-        # and only if we're not using a socket.
-        if self.port is None and self.sock is None and self.socket_path is None:
-            self.port = self._get_default_port()
-
-        try:
-            response = await self._create_connection(
-                timeout=self.timeout if timeout is _default else timeout
-            )
-        except Exception as exc:
-            self.close()  # Reset our state to disconnected
-            raise exc
-
-        await self._maybe_start_tls_on_connect()
-        await self._maybe_login_on_connect()
-
-        return response
-
-    async def _create_connection(self, timeout: Optional[float]) -> SMTPResponse:
-        if self.loop is None:
-            raise RuntimeError("No event loop set")
-
-        protocol = SMTPProtocol(loop=self.loop)
-
-        tls_context: Optional[ssl.SSLContext] = None
-        ssl_handshake_timeout: Optional[float] = None
-        if self.use_tls:
-            tls_context = self._get_tls_context()
-            ssl_handshake_timeout = timeout
-
-        if self.sock is not None:
-            connect_coro = self.loop.create_connection(
-                lambda: protocol,
-                sock=self.sock,
-                ssl=tls_context,
-                ssl_handshake_timeout=ssl_handshake_timeout,
-            )
-        elif self.socket_path is not None:
-            connect_coro = self.loop.create_unix_connection(
-                lambda: protocol,
-                path=self.socket_path,  # type: ignore
-                ssl=tls_context,
-                ssl_handshake_timeout=ssl_handshake_timeout,
-            )
-        else:
-            if self.hostname is None:
-                raise RuntimeError("No hostname provided; default should have been set")
-            if self.port is None:
-                raise RuntimeError("No port provided; default should have been set")
-
-            connect_coro = self.loop.create_connection(
-                lambda: protocol,
-                host=self.hostname,
-                port=self.port,
-                ssl=tls_context,
-                ssl_handshake_timeout=ssl_handshake_timeout,
-                local_addr=self.source_address,
-            )
-
-        try:
-            transport, _ = await asyncio.wait_for(connect_coro, timeout=timeout)
-        except (TimeoutError, asyncio.TimeoutError) as exc:
-            raise SMTPConnectTimeoutError(
-                f"Timed out connecting to {self.hostname} on port {self.port}"
-            ) from exc
-        except OSError as exc:
-            raise SMTPConnectError(
-                f"Error connecting to {self.hostname} on port {self.port}: {exc}"
-            ) from exc
-
-        self.protocol = protocol
-        self.transport = transport
-
-        try:
-            response = await protocol.read_response(timeout=timeout)
-        except SMTPServerDisconnected as exc:
-            raise SMTPConnectError(
-                f"Error connecting to {self.hostname} on port {self.port}: {exc}"
-            ) from exc
-        except SMTPTimeoutError as exc:
-            raise SMTPConnectTimeoutError(
-                "Timed out waiting for server ready message"
-            ) from exc
-
-        if response.code != SMTPStatus.ready:
-            raise SMTPConnectResponseError(response.code, response.message)
-
-        return response
-
-    def _connection_lost(self, waiter: "asyncio.Future[None]") -> None:
-        self.close()
-
-    async def _maybe_start_tls_on_connect(self) -> None:
-        """
-        Depending on config, upgrade the connection via STARTTLS.
-        """
-        if self._start_tls_on_connect is True:
-            await self.starttls()
-        # If _start_tls_on_connect hasn't been set either way,
-        # try to STARTTLS if supported, with graceful failure handling
-        elif self._start_tls_on_connect is None:
-            already_using_tls = self.get_transport_info("sslcontext") is not None
-            if not (self.use_tls or already_using_tls):
-                await self._ehlo_or_helo_if_needed()
-                if self.supports_extension("starttls"):
-                    await self.starttls()
-
-    async def _maybe_login_on_connect(self) -> None:
-        """
-        Depending on config, login after connecting.
-        """
-        if self._login_username is not None:
-            login_password = (
-                self._login_password if self._login_password is not None else ""
-            )
-            await self.login(self._login_username, login_password)
-
-    async def execute_command(
-        self, *args: bytes, timeout: Optional[Union[float, Default]] = _default
-    ) -> SMTPResponse:
-        """
-        Check that we're connected, if we got a timeout value, and then
-        pass the command to the protocol.
-
-        :raises SMTPServerDisconnected: connection lost
-        """
-        if self.protocol is None:
-            raise SMTPServerDisconnected("Server not connected")
-
-        response = await self.protocol.execute_command(
-            *args, timeout=self.timeout if timeout is _default else timeout
-        )
-
-        # If the server is unavailable, be nice and close the connection
-        if response.code == SMTPStatus.domain_unavailable:
-            self.close()
-
-        return response
-
-    def _get_tls_context(self) -> ssl.SSLContext:
-        """
-        Build an SSLContext object from the options we've been given.
-        """
-        if self.tls_context is not None:
-            context = self.tls_context
-        else:
-            # SERVER_AUTH is what we want for a client side socket
-            context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
-            context.check_hostname = bool(self.validate_certs)
-            if self.validate_certs:
-                context.verify_mode = ssl.CERT_REQUIRED
-            else:
-                context.verify_mode = ssl.CERT_NONE
-
-            if self.cert_bundle is not None:
-                context.load_verify_locations(cafile=self.cert_bundle)
-
-            if self.client_cert is not None:
-                context.load_cert_chain(self.client_cert, keyfile=self.client_key)
-
-        return context
-
-    def close(self) -> None:
-        """
-        Closes the connection.
-        """
-        if self.transport is not None and not self.transport.is_closing():
-            self.transport.close()
-
-        if self._connect_lock is not None and self._connect_lock.locked():
-            self._connect_lock.release()
-
-        self.protocol = None
-        self.transport = None
-
-        # Reset ESMTP state
-        self._reset_server_state()
-
-    def get_transport_info(self, key: str) -> Any:
-        """
-        Get extra info from the transport.
-        Supported keys:
-
-            - ``peername``
-            - ``socket``
-            - ``sockname``
-            - ``compression``
-            - ``cipher``
-            - ``peercert``
-            - ``sslcontext``
-            - ``sslobject``
-
-        :raises SMTPServerDisconnected: connection lost
-        """
-        if not (self.is_connected and self.transport):
-            raise SMTPServerDisconnected("Server not connected")
-
-        return self.transport.get_extra_info(key)
-
-    # Base SMTP commands #
-
-    async def helo(
-        self,
-        *,
-        hostname: Optional[str] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send the SMTP HELO command.
-        Hostname to send for this command defaults to the FQDN of the local
-        host.
-
-        :raises SMTPHeloError: on unexpected server response code
-        """
-        response = self.last_helo_response = await self.execute_command(
-            b"HELO", (hostname or self.local_hostname).encode("ascii"), timeout=timeout
-        )
-
-        if response.code != SMTPStatus.completed:
-            raise SMTPHeloError(response.code, response.message)
-
-        return response
-
-    async def help(self, *, timeout: Optional[Union[float, Default]] = _default) -> str:
-        """
-        Send the SMTP HELP command, which responds with help text.
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        response = await self.execute_command(b"HELP", timeout=timeout)
-        if response.code not in (
-            SMTPStatus.system_status_ok,
-            SMTPStatus.help_message,
-            SMTPStatus.completed,
-        ):
-            raise SMTPResponseException(response.code, response.message)
-
-        return response.message
-
-    async def rset(
-        self, *, timeout: Optional[Union[float, Default]] = _default
-    ) -> SMTPResponse:
-        """
-        Send an SMTP RSET command, which resets the server's envelope
-        (the envelope contains the sender, recipient, and mail data).
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        response = await self.execute_command(b"RSET", timeout=timeout)
-        if response.code != SMTPStatus.completed:
-            raise SMTPResponseException(response.code, response.message)
-
-        return response
-
-    async def noop(
-        self, *, timeout: Optional[Union[float, Default]] = _default
-    ) -> SMTPResponse:
-        """
-        Send an SMTP NOOP command, which does nothing.
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        response = await self.execute_command(b"NOOP", timeout=timeout)
-        if response.code != SMTPStatus.completed:
-            raise SMTPResponseException(response.code, response.message)
-
-        return response
-
-    async def vrfy(
-        self,
-        address: str,
-        /,
-        *,
-        options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send an SMTP VRFY command, which tests an address for validity.
-        Not many servers support this command.
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if options is None:
-            options = []
-
-        parsed_address = parse_address(address)
-        if any(option.lower() == "smtputf8" for option in options):
-            if not self.supports_extension("smtputf8"):
-                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
-            addr_bytes = parsed_address.encode("utf-8")
-        else:
-            addr_bytes = parsed_address.encode("ascii")
-        options_bytes = [option.encode("ascii") for option in options]
-
-        response = await self.execute_command(
-            b"VRFY", addr_bytes, *options_bytes, timeout=timeout
-        )
-
-        if response.code not in (
-            SMTPStatus.completed,
-            SMTPStatus.will_forward,
-            SMTPStatus.cannot_vrfy,
-        ):
-            raise SMTPResponseException(response.code, response.message)
-
-        return response
-
-    async def expn(
-        self,
-        address: str,
-        /,
-        *,
-        options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send an SMTP EXPN command, which expands a mailing list.
-        Not many servers support this command.
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if options is None:
-            options = []
-
-        parsed_address = parse_address(address)
-        if any(option.lower() == "smtputf8" for option in options):
-            if not self.supports_extension("smtputf8"):
-                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
-            addr_bytes = parsed_address.encode("utf-8")
-        else:
-            addr_bytes = parsed_address.encode("ascii")
-        options_bytes = [option.encode("ascii") for option in options]
-
-        response = await self.execute_command(
-            b"EXPN", addr_bytes, *options_bytes, timeout=timeout
-        )
-
-        if response.code != SMTPStatus.completed:
-            raise SMTPResponseException(response.code, response.message)
-
-        return response
-
-    async def quit(
-        self, *, timeout: Optional[Union[float, Default]] = _default
-    ) -> SMTPResponse:
-        """
-        Send the SMTP QUIT command, which closes the connection.
-        Also closes the connection from our side after a response is received.
-
-        :raises SMTPResponseException: on unexpected server response code
-        """
-        response = await self.execute_command(b"QUIT", timeout=timeout)
-        if response.code != SMTPStatus.closing:
-            raise SMTPResponseException(response.code, response.message)
-
-        self.close()
-
-        return response
-
-    async def mail(
-        self,
-        sender: str,
-        /,
-        *,
-        options: Optional[Iterable[str]] = None,
-        encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send an SMTP MAIL command, which specifies the message sender and
-        begins a new mail transfer session ("envelope").
-
-        :raises SMTPSenderRefused: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if options is None:
-            options = []
-
-        quoted_sender = quote_address(sender)
-        addr_bytes = quoted_sender.encode(encoding)
-        options_bytes = [option.encode("ascii") for option in options]
-
-        response = await self.execute_command(
-            b"MAIL", b"FROM:" + addr_bytes, *options_bytes, timeout=timeout
-        )
-
-        if response.code != SMTPStatus.completed:
-            raise SMTPSenderRefused(response.code, response.message, sender)
-
-        return response
-
-    async def rcpt(
-        self,
-        recipient: str,
-        /,
-        *,
-        options: Optional[Iterable[str]] = None,
-        encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send an SMTP RCPT command, which specifies a single recipient for
-        the message. This command is sent once per recipient and must be
-        preceded by 'MAIL'.
-
-        :raises SMTPRecipientRefused: on unexpected server response code
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if options is None:
-            options = []
-
-        quoted_recipient = quote_address(recipient)
-        addr_bytes = quoted_recipient.encode(encoding)
-        options_bytes = [option.encode("ascii") for option in options]
-
-        response = await self.execute_command(
-            b"RCPT", b"TO:" + addr_bytes, *options_bytes, timeout=timeout
-        )
-
-        if response.code not in (SMTPStatus.completed, SMTPStatus.will_forward):
-            raise SMTPRecipientRefused(response.code, response.message, recipient)
-
-        return response
-
-    async def data(
-        self,
-        message: Union[str, bytes],
-        /,
-        *,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send an SMTP DATA command, followed by the message given.
-        This method transfers the actual email content to the server.
-
-        :raises SMTPDataError: on unexpected server response code
-        :raises SMTPServerDisconnected: connection lost
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        # As data accesses protocol directly, some handling is required
-        if self.protocol is None:
-            raise SMTPServerDisconnected("Connection lost")
-
-        if timeout is _default:
-            timeout = self.timeout
-
-        if isinstance(message, str):
-            message = message.encode("ascii")
-
-        return await self.protocol.execute_data_command(message, timeout=timeout)
-
-    # ESMTP commands #
-
-    async def ehlo(
-        self,
-        *,
-        hostname: Optional[str] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Send the SMTP EHLO command.
-        Hostname to send for this command defaults to the FQDN of the local
-        host.
-
-        :raises SMTPHeloError: on unexpected server response code
-        """
-        if hostname is None:
-            hostname = self.local_hostname
-
-        response = await self.execute_command(
-            b"EHLO", hostname.encode("ascii"), timeout=timeout
-        )
-        self.last_ehlo_response = response
-
-        if response.code != SMTPStatus.completed:
-            raise SMTPHeloError(response.code, response.message)
-
-        return response
-
-    def supports_extension(self, extension: str, /) -> bool:
-        """
-        Tests if the server supports the ESMTP service extension given.
-        """
-        return extension.lower() in self.esmtp_extensions
-
-    async def _ehlo_or_helo_if_needed(self) -> None:
-        """
-        Call self.ehlo() and/or self.helo() if needed.
-
-        If there has been no previous EHLO or HELO command this session, this
-        method tries ESMTP EHLO first.
-        """
-        if self.is_ehlo_or_helo_needed:
-            try:
-                await self.ehlo()
-            except SMTPHeloError as exc:
-                if self.is_connected:
-                    await self.helo()
-                else:
-                    raise exc
-
-    def _reset_server_state(self) -> None:
-        """
-        Clear stored information about the server.
-        """
-        self.last_helo_response = None
-        self._last_ehlo_response = None
-        self.esmtp_extensions = {}
-        self.supports_esmtp = False
-        self.server_auth_methods = []
-
-    async def starttls(
-        self,
-        *,
-        server_hostname: Optional[str] = None,
-        validate_certs: Optional[bool] = None,
-        client_cert: Optional[Union[str, Default]] = _default,
-        client_key: Optional[Union[str, Default]] = _default,
-        cert_bundle: Optional[Union[str, Default]] = _default,
-        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Puts the connection to the SMTP server into TLS mode.
-
-        If there has been no previous EHLO or HELO command this session, this
-        method tries ESMTP EHLO first.
-
-        If the server supports TLS, this will encrypt the rest of the SMTP
-        session. If you provide the keyfile and certfile parameters,
-        the identity of the SMTP server and client can be checked (if
-        validate_certs is True). You can also provide a custom SSLContext
-        object. If no certs or SSLContext is given, and TLS config was
-        provided when initializing the class, STARTTLS will use to that,
-        otherwise it will use the Python defaults.
-
-        :raises SMTPException: server does not support STARTTLS
-        :raises SMTPServerDisconnected: connection lost
-        :raises ValueError: invalid options provided
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if self.protocol is None:
-            raise SMTPServerDisconnected("Server not connected")
-
-        if self.get_transport_info("sslcontext") is not None:
-            raise SMTPException("Connection already using TLS")
-
-        self._update_settings_from_kwargs(
-            validate_certs=validate_certs,
-            client_cert=client_cert,
-            client_key=client_key,
-            cert_bundle=cert_bundle,
-            tls_context=tls_context,
-        )
-        self._validate_config()
-
-        if server_hostname is None:
-            server_hostname = self.hostname
-
-        if timeout is _default:
-            timeout = self.timeout
-
-        tls_context = self._get_tls_context()
-
-        if not self.supports_extension("starttls"):
-            raise SMTPException("SMTP STARTTLS extension not supported by server.")
-
-        response = await self.protocol.start_tls(
-            tls_context, server_hostname=server_hostname, timeout=timeout
-        )
-        if self.protocol is None:
-            raise SMTPServerDisconnected("Connection lost")
-        # Update our transport reference
-        self.transport = self.protocol.transport
-
-        # RFC 3207 part 4.2:
-        # The client MUST discard any knowledge obtained from the server, such
-        # as the list of SMTP service extensions, which was not obtained from
-        # the TLS negotiation itself.
-        self._reset_server_state()
-
-        return response
-
-    # Auth commands
-
-    async def login(
-        self,
-        username: Union[str, bytes],
-        password: Union[str, bytes],
-        /,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        Tries to login with supported auth methods.
-
-        Some servers advertise authentication methods they don't really
-        support, so if authentication fails, we continue until we've tried
-        all methods.
-        """
-        await self._ehlo_or_helo_if_needed()
-
-        if not self.supports_extension("auth"):
-            if self.is_connected and self.get_transport_info("sslcontext") is None:
-                raise SMTPException(
-                    "The SMTP AUTH extension is not supported by this server. Try "
-                    "connecting via TLS (or STARTTLS)."
-                )
-            raise SMTPException(
-                "The SMTP AUTH extension is not supported by this server."
-            )
-
-        response: Optional[SMTPResponse] = None
-        exception: Optional[SMTPAuthenticationError] = None
-        for auth_name in self.supported_auth_methods:
-            method_name = f'auth_{auth_name.replace("-", "")}'
-            try:
-                auth_method = getattr(self, method_name)
-            except AttributeError as err:
-                raise RuntimeError(
-                    f"Missing handler for auth method {auth_name}"
-                ) from err
-            try:
-                response = await auth_method(username, password, timeout=timeout)
-            except SMTPAuthenticationError as exc:
-                exception = exc
-            else:
-                # No exception means we're good
-                break
-
-        if response is None:
-            raise exception or SMTPException("No suitable authentication method found.")
-
-        return response
-
-    async def auth_crammd5(
-        self,
-        username: Union[str, bytes],
-        password: Union[str, bytes],
-        /,
-        *,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        CRAM-MD5 auth uses the password as a shared secret to MD5 the server's
-        response.
-
-        Example::
-
-            250 AUTH CRAM-MD5
-            auth cram-md5
-            334 PDI0NjA5LjEwNDc5MTQwNDZAcG9wbWFpbC5TcGFjZS5OZXQ+
-            dGltIGI5MTNhNjAyYzdlZGE3YTQ5NWI0ZTZlNzMzNGQzODkw
-
-        """
-        initial_response = await self.execute_command(
-            b"AUTH", b"CRAM-MD5", timeout=timeout
-        )
-
-        if initial_response.code != SMTPStatus.auth_continue:
-            raise SMTPAuthenticationError(
-                initial_response.code, initial_response.message
-            )
-
-        verification_bytes = auth_crammd5_verify(
-            username, password, initial_response.message
-        )
-        response = await self.execute_command(verification_bytes)
-
-        if response.code != SMTPStatus.auth_successful:
-            raise SMTPAuthenticationError(response.code, response.message)
-
-        return response
-
-    async def auth_plain(
-        self,
-        username: Union[str, bytes],
-        password: Union[str, bytes],
-        /,
-        *,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        PLAIN auth encodes the username and password in one Base64 encoded
-        string. No verification message is required.
-
-        Example::
-
-            220-esmtp.example.com
-            AUTH PLAIN dGVzdAB0ZXN0AHRlc3RwYXNz
-            235 ok, go ahead (#2.0.0)
-
-        """
-        encoded = auth_plain_encode(username, password)
-        response = await self.execute_command(
-            b"AUTH", b"PLAIN", encoded, timeout=timeout
-        )
-
-        if response.code != SMTPStatus.auth_successful:
-            raise SMTPAuthenticationError(response.code, response.message)
-
-        return response
-
-    async def auth_login(
-        self,
-        username: Union[str, bytes],
-        password: Union[str, bytes],
-        /,
-        *,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> SMTPResponse:
-        """
-        LOGIN auth sends the Base64 encoded username and password in sequence.
-
-        Example::
-
-            250 AUTH LOGIN PLAIN CRAM-MD5
-            auth login avlsdkfj
-            334 UGFzc3dvcmQ6
-            avlsdkfj
-
-        Note that there is an alternate version sends the username
-        as a separate command::
-
-            250 AUTH LOGIN PLAIN CRAM-MD5
-            auth login
-            334 VXNlcm5hbWU6
-            avlsdkfj
-            334 UGFzc3dvcmQ6
-            avlsdkfj
-
-        However, since most servers seem to support both, we send the username
-        with the initial request.
-        """
-        encoded_username, encoded_password = auth_login_encode(username, password)
-        initial_response = await self.execute_command(
-            b"AUTH", b"LOGIN", encoded_username, timeout=timeout
-        )
-
-        if initial_response.code != SMTPStatus.auth_continue:
-            raise SMTPAuthenticationError(
-                initial_response.code, initial_response.message
-            )
-
-        response = await self.execute_command(encoded_password, timeout=timeout)
-
-        if response.code != SMTPStatus.auth_successful:
-            raise SMTPAuthenticationError(response.code, response.message)
-
-        return response
-
-    async def sendmail(
-        self,
-        sender: str,
-        recipients: Union[str, Sequence[str]],
-        message: Union[str, bytes],
-        /,
-        *,
-        mail_options: Optional[Iterable[str]] = None,
-        rcpt_options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
-        """
-        This command performs an entire mail transaction.
-
-        The arguments are:
-            - sender: The address sending this mail.
-            - recipients: A list of addresses to send this mail to.  A bare
-                string will be treated as a list with 1 address.
-            - message: The message string to send.
-            - mail_options: List of options (such as ESMTP 8bitmime) for the
-                MAIL command.
-            - rcpt_options: List of options (such as DSN commands) for all the
-                RCPT commands.
-
-        message must be a string containing characters in the ASCII range.
-        The string is encoded to bytes using the ascii codec, and lone \\\\r
-        and \\\\n characters are converted to \\\\r\\\\n characters.
-
-        If there has been no previous HELO or EHLO command this session, this
-        method tries EHLO first.
-
-        This method will return normally if the mail is accepted for at least
-        one recipient.  It returns a tuple consisting of:
-
-            - an error dictionary, with one entry for each recipient that was
-                refused.  Each entry contains a tuple of the SMTP error code
-                and the accompanying error message sent by the server.
-            - the message sent by the server in response to the DATA command
-                (often containing a message id)
-
-        Example:
-
-             >>> event_loop = asyncio.get_event_loop()
-             >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
-             >>> event_loop.run_until_complete(smtp.connect())
-             (220, ...)
-             >>> recipients = ["one@one.org", "two@two.org", "3@three.org"]
-             >>> message = "From: Me@my.org\\nSubject: testing\\nHello World"
-             >>> send_coro = smtp.sendmail("me@my.org", recipients, message)
-             >>> event_loop.run_until_complete(send_coro)
-             ({}, 'OK')
-             >>> event_loop.run_until_complete(smtp.quit())
-             (221, Bye)
-
-        In the above example, the message was accepted for delivery for all
-        three addresses. If delivery had been only successful to two
-        of the three addresses, and one was rejected, the response would look
-        something like::
-
-            (
-                {"nobody@three.org": (550, "User unknown")},
-                "Written safely to disk. #902487694.289148.12219.",
-            )
-
-
-        If delivery is not successful to any addresses,
-        :exc:`.SMTPRecipientsRefused` is raised.
-
-        If :exc:`.SMTPResponseException` is raised by this method, we try to
-        send an RSET command to reset the server envelope automatically for
-        the next attempt.
-
-        :raises SMTPRecipientsRefused: delivery to all recipients failed
-        :raises SMTPResponseException: on invalid response
-        """
-        if isinstance(recipients, str):
-            recipients = [recipients]
-        if mail_options is None:
-            mail_options = []
-        else:
-            mail_options = list(mail_options)
-        if rcpt_options is None:
-            rcpt_options = []
-        else:
-            rcpt_options = list(rcpt_options)
-
-        if any(option.lower() == "smtputf8" for option in mail_options):
-            mailbox_encoding = "utf-8"
-        else:
-            mailbox_encoding = "ascii"
-
-        if self._sendmail_lock is None:
-            self._sendmail_lock = asyncio.Lock()
-
-        async with self._sendmail_lock:
-            # Make sure we've done an EHLO for extension checks
-            await self._ehlo_or_helo_if_needed()
-
-            if mailbox_encoding == "utf-8" and not self.supports_extension("smtputf8"):
-                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
-
-            if self.supports_extension("size"):
-                message_len = len(message)
-                size_option = f"size={message_len}"
-                mail_options.insert(0, size_option)
-
-            try:
-                await self.mail(
-                    sender,
-                    options=mail_options,
-                    encoding=mailbox_encoding,
-                    timeout=timeout,
-                )
-                recipient_errors = await self._send_recipients(
-                    recipients, rcpt_options, encoding=mailbox_encoding, timeout=timeout
-                )
-                response = await self.data(message, timeout=timeout)
-            except (SMTPResponseException, SMTPRecipientsRefused) as exc:
-                # If we got an error, reset the envelope.
-                try:
-                    await self.rset(timeout=timeout)
-                except (ConnectionError, SMTPResponseException):
-                    # If we're disconnected on the reset, or we get a bad
-                    # status, don't raise that as it's confusing
-                    pass
-                raise exc
-
-        return recipient_errors, response.message
-
-    async def _send_recipients(
-        self,
-        recipients: Sequence[str],
-        options: Iterable[str],
-        encoding: str = "ascii",
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Dict[str, SMTPResponse]:
-        """
-        Send the recipients given to the server. Used as part of
-        :meth:`.sendmail`.
-        """
-        recipient_errors = []
-        for address in recipients:
-            try:
-                await self.rcpt(
-                    address, options=options, encoding=encoding, timeout=timeout
-                )
-            except SMTPRecipientRefused as exc:
-                recipient_errors.append(exc)
-
-        if len(recipient_errors) == len(recipients):
-            raise SMTPRecipientsRefused(recipient_errors)
-
-        formatted_errors = {
-            err.recipient: SMTPResponse(err.code, err.message)
-            for err in recipient_errors
-        }
-
-        return formatted_errors
-
-    async def send_message(
-        self,
-        message: Union[email.message.EmailMessage, email.message.Message],
-        /,
-        *,
-        sender: Optional[str] = None,
-        recipients: Optional[Union[str, Sequence[str]]] = None,
-        mail_options: Optional[Iterable[str]] = None,
-        rcpt_options: Optional[Iterable[str]] = None,
-        timeout: Optional[Union[float, Default]] = _default,
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
-        r"""
-        Sends an :py:class:`email.message.EmailMessage` object.
-
-        Arguments are as for :meth:`.sendmail`, except that message is an
-        :py:class:`email.message.EmailMessage` object.  If sender is None or
-        recipients is None, these arguments are taken from the headers of the
-        EmailMessage as described in RFC 2822.  Regardless of the values of sender
-        and recipients, any Bcc field (or Resent-Bcc field, when the message is a
-        resent) of the EmailMessage object will not be transmitted.  The EmailMessage
-        object is then serialized using :py:class:`email.generator.Generator` and
-        :meth:`.sendmail` is called to transmit the message.
-
-        'Resent-Date' is a mandatory field if the message is resent (RFC 2822
-        Section 3.6.6). In such a case, we use the 'Resent-\*' fields.
-        However, if there is more than one 'Resent-' block there's no way to
-        unambiguously determine which one is the most recent in all cases,
-        so rather than guess we raise a ``ValueError`` in that case.
-
-        :raises ValueError:
-            on more than one Resent header block
-            on no sender kwarg or From header in message
-            on no recipients kwarg or To, Cc or Bcc header in message
-        :raises SMTPRecipientsRefused: delivery to all recipients failed
-        :raises SMTPResponseException: on invalid response
-        """
-        if mail_options is None:
-            mail_options = []
-        else:
-            mail_options = list(mail_options)
-
-        if sender is None:
-            sender = extract_sender(message)
-        if sender is None:
-            raise ValueError("No From header provided in message")
-
-        if isinstance(recipients, str):
-            recipients = [recipients]
-        elif recipients is None:
-            recipients = extract_recipients(message)
-        if not recipients:
-            raise ValueError("No recipient headers provided in message")
-
-        # Make sure we've done an EHLO for extension checks
-        await self._ehlo_or_helo_if_needed()
-
-        try:
-            sender.encode("ascii")
-            "".join(recipients).encode("ascii")
-        except UnicodeEncodeError:
-            utf8_required = True
-        else:
-            utf8_required = False
-
-        if utf8_required:
-            if not self.supports_extension("smtputf8"):
-                raise SMTPNotSupported(
-                    "An address containing non-ASCII characters was provided, but "
-                    "SMTPUTF8 is not supported by this server"
-                )
-            elif "smtputf8" not in [option.lower() for option in mail_options]:
-                mail_options.append("SMTPUTF8")
-
-        if self.supports_extension("8BITMIME"):
-            if "body=8bitmime" not in [option.lower() for option in mail_options]:
-                mail_options.append("BODY=8BITMIME")
-            cte_type = "8bit"
-        else:
-            cte_type = "7bit"
-
-        flat_message = flatten_message(message, utf8=utf8_required, cte_type=cte_type)
-
-        return await self.sendmail(
-            sender,
-            recipients,
-            flat_message,
-            mail_options=mail_options,
-            rcpt_options=rcpt_options,
-            timeout=timeout,
-        )
-
-    def sendmail_sync(
-        self, *args: Any, **kwargs: Any
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
-        """
-        Synchronous version of :meth:`.sendmail`. This method starts
-        an event loop to connect, send the message, and disconnect.
-        """
-
-        async def sendmail_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
-            async with self:
-                return await self.sendmail(*args, **kwargs)
-
-        return asyncio.run(sendmail_coroutine())
-
-    def send_message_sync(
-        self, *args: Any, **kwargs: Any
-    ) -> Tuple[Dict[str, SMTPResponse], str]:
-        """
-        Synchronous version of :meth:`.send_message`. This method
-        starts an event loop to connect, send the message, and disconnect.
-        """
-
-        async def send_message_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
-            async with self:
-                return await self.send_message(*args, **kwargs)
-
-        return asyncio.run(send_message_coroutine())
diff -pruN 3.0.1-1/aiosmtplib/status.py 3.0.2-1/aiosmtplib/status.py
--- 3.0.1-1/aiosmtplib/status.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/status.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,5 +0,0 @@
-from .typing import SMTPStatus
-
-
-# alias SMTPStatus for backwards compatibility
-__all__ = ("SMTPStatus",)
diff -pruN 3.0.1-1/aiosmtplib/typing.py 3.0.2-1/aiosmtplib/typing.py
--- 3.0.1-1/aiosmtplib/typing.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/aiosmtplib/typing.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,60 +0,0 @@
-import enum
-import os
-from typing import Union
-
-
-__all__ = ("Default", "SMTPStatus", "SocketPathType", "_default")
-
-
-SocketPathType = Union[str, bytes, os.PathLike]
-
-
-class Default(enum.Enum):
-    """
-    Used for type hinting kwarg defaults.
-    """
-
-    token = 0
-
-
-_default = Default.token
-
-
-@enum.unique
-class SMTPStatus(enum.IntEnum):
-    """
-    Defines SMTP statuses for code readability.
-
-    See also: http://www.greenend.org.uk/rjk/tech/smtpreplies.html
-    """
-
-    invalid_response = -1
-    system_status_ok = 211
-    help_message = 214
-    ready = 220
-    closing = 221
-    auth_successful = 235
-    completed = 250
-    will_forward = 251
-    cannot_vrfy = 252
-    auth_continue = 334
-    start_input = 354
-    domain_unavailable = 421
-    mailbox_unavailable = 450
-    error_processing = 451
-    insufficient_storage = 452
-    tls_not_available = 454
-    unrecognized_command = 500
-    unrecognized_parameters = 501
-    command_not_implemented = 502
-    bad_command_sequence = 503
-    parameter_not_implemented = 504
-    domain_does_not_accept_mail = 521
-    access_denied = 530  # Sendmail specific
-    auth_failed = 535
-    mailbox_does_not_exist = 550
-    user_not_local = 551
-    storage_exceeded = 552
-    mailbox_name_invalid = 553
-    transaction_failed = 554
-    syntax_error = 555
diff -pruN 3.0.1-1/debian/changelog 3.0.2-1/debian/changelog
--- 3.0.1-1/debian/changelog	2023-12-31 17:46:13.000000000 +0000
+++ 3.0.2-1/debian/changelog	2024-08-23 13:45:30.000000000 +0000
@@ -1,3 +1,11 @@
+aiosmtplib (3.0.2-1) unstable; urgency=medium
+
+  * New upstream release 3.0.2
+  * Bump Standards-Version to 4.7.0
+  * Depends on uvloop for performance
+
+ -- Pierre-Elliott Bécue <peb@debian.org>  Fri, 23 Aug 2024 15:45:30 +0200
+
 aiosmtplib (3.0.1-1) unstable; urgency=medium
 
   * Team upload.
diff -pruN 3.0.1-1/debian/control 3.0.2-1/debian/control
--- 3.0.1-1/debian/control	2023-12-31 17:46:13.000000000 +0000
+++ 3.0.2-1/debian/control	2024-08-23 13:45:30.000000000 +0000
@@ -8,16 +8,17 @@ Build-Depends: debhelper-compat (= 13),
                pybuild-plugin-pyproject,
                python3-aiosmtpd,
                python3-all,
+               python3-hatchling,
                python3-hypothesis,
-               python3-poetry-core,
                python3-pytest,
                python3-pytest-asyncio,
                python3-sphinx,
                python3-sphinx-autodoc-typehints,
                python3-sphinx-copybutton,
                python3-trustme,
+               python3-uvloop,
                furo,
-Standards-Version: 4.6.2
+Standards-Version: 4.7.0
 Rules-Requires-Root: no
 Homepage: https://aiosmtplib.readthedocs.io
 Vcs-Browser: https://salsa.debian.org/python-team/packages/aiosmtplib
@@ -26,6 +27,7 @@ Vcs-Git: https://salsa.debian.org/python
 Package: python3-aiosmtplib
 Architecture: all
 Depends: python3-pkg-resources,
+         python3-uvloop,
          ${misc:Depends},
          ${python3:Depends}
 Suggests: python3-aiosmtplib-doc
diff -pruN 3.0.1-1/debian/patches/0001-Remove-web-images-for-privacy.patch 3.0.2-1/debian/patches/0001-Remove-web-images-for-privacy.patch
--- 3.0.1-1/debian/patches/0001-Remove-web-images-for-privacy.patch	2023-12-31 17:46:13.000000000 +0000
+++ 3.0.2-1/debian/patches/0001-Remove-web-images-for-privacy.patch	2024-08-23 13:45:30.000000000 +0000
@@ -1,8 +1,8 @@
 From: Dale Richards <dale@dalerichards.net>
 Date: Sun, 31 Dec 2023 16:02:14 +0000
 Subject: Remove web images for privacy
-Forwarded: not-needed
 
+Forwarded: not-needed
 ---
  README.rst | 23 -----------------------
  1 file changed, 23 deletions(-)
diff -pruN 3.0.1-1/docs/client.rst 3.0.2-1/docs/client.rst
--- 3.0.1-1/docs/client.rst	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/docs/client.rst	2020-02-02 00:00:00.000000000 +0000
@@ -1,5 +1,4 @@
-.. module:: aiosmtplib
-    :noindex:
+ .. py:currentmodule:: aiosmtplib
 
 The SMTP Client Class
 =====================
diff -pruN 3.0.1-1/docs/conf.py 3.0.2-1/docs/conf.py
--- 3.0.1-1/docs/conf.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/docs/conf.py	2020-02-02 00:00:00.000000000 +0000
@@ -67,7 +67,7 @@ copyright = f"{year}, {author}"
 # |version| and |release|, also used in various other places throughout the
 # built documents.
 
-init = pathlib.Path("../aiosmtplib/__init__.py")
+init = pathlib.Path("../src/aiosmtplib/__init__.py")
 version_match = re.search(VERSION_REGEX, init.read_text("utf8"), re.MULTILINE)
 if not version_match:
     raise RuntimeError("Cannot find version information")
diff -pruN 3.0.1-1/docs/requirements.txt 3.0.2-1/docs/requirements.txt
--- 3.0.1-1/docs/requirements.txt	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/docs/requirements.txt	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,4 @@
+sphinx>=7.0.0
+sphinx_autodoc_typehints>=1.24.0
+sphinx-copybutton>=0.5.0
+furo>=2023.9.10
diff -pruN 3.0.1-1/docs/timeouts.rst 3.0.2-1/docs/timeouts.rst
--- 3.0.1-1/docs/timeouts.rst	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/docs/timeouts.rst	2020-02-02 00:00:00.000000000 +0000
@@ -1,5 +1,4 @@
-.. module:: aiosmtplib
-    :noindex:
+ .. py:currentmodule:: aiosmtplib
 
 Timeouts
 ========
diff -pruN 3.0.1-1/docs/usage.rst 3.0.2-1/docs/usage.rst
--- 3.0.1-1/docs/usage.rst	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/docs/usage.rst	2020-02-02 00:00:00.000000000 +0000
@@ -1,5 +1,4 @@
-.. module:: aiosmtplib
-    :noindex:
+ .. py:currentmodule:: aiosmtplib
 
 The send Coroutine
 ==================
diff -pruN 3.0.1-1/mypy.ini 3.0.2-1/mypy.ini
--- 3.0.1-1/mypy.ini	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/mypy.ini	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,21 @@
+[mypy]
+python_version = 3.8
+
+# --strict
+warn_unused_configs = True
+disallow_subclassing_any = True
+disallow_untyped_calls = True
+disallow_untyped_defs = True
+disallow_incomplete_defs = True
+check_untyped_defs = True
+disallow_untyped_decorators = True
+no_implicit_optional = True
+warn_redundant_casts = True
+warn_unused_ignores = True
+warn_return_any = True
+no_implicit_reexport = True
+strict_equality = True
+
+[mypy-tests.*]
+disallow_subclassing_any = False
+disallow_untyped_decorators = False
diff -pruN 3.0.1-1/pyproject.toml 3.0.2-1/pyproject.toml
--- 3.0.1-1/pyproject.toml	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/pyproject.toml	2020-02-02 00:00:00.000000000 +0000
@@ -1,17 +1,15 @@
 [build-system]
-requires = ["poetry-core>=1.0.0"]
-build-backend = "poetry.core.masonry.api"
+requires = ["hatchling"]
+build-backend = "hatchling.build"
 
-[tool.poetry]
+[project]
+dynamic = ["version"]
 name = "aiosmtplib"
-version = "3.0.1"
 description = "asyncio SMTP client"
-authors = ["Cole Maclean <hi@colemaclean.dev>"]
-license = "MIT"
-packages = [{ include = "aiosmtplib" }]
+authors = [{ name = "Cole Maclean", email = "hi@colemaclean.dev" }]
+license = { text = "MIT" }
 readme = "README.rst"
-repository = "https://github.com/cole/aiosmtplib"
-documentation = "https://aiosmtplib.readthedocs.io/en/stable/"
+requires-python = ">=3.8"
 keywords = ["smtp", "email", "asyncio"]
 classifiers = [
     "Development Status :: 5 - Production/Stable",
@@ -26,57 +24,24 @@ classifiers = [
     "Topic :: Software Development :: Libraries :: Python Modules",
     "Typing :: Typed",
 ]
-exclude = ["docs/requirements.txt"]
-include = [
-    { path = "CHANGELOG.rst", format = "sdist" },
-    { path = "tests", format = "sdist" },
-    { path = "docs", format = "sdist" },
-]
-
-[tool.poetry.dependencies]
-python = "^3.8"
-
-uvloop = { version = "^0.18", optional = true }
-sphinx = { version = "^7.0.0", optional = true }
-furo = { version = "^2023.9.10", optional = true }
-sphinx-copybutton = { version = "^0.5.0", optional = true }
-sphinx_autodoc_typehints = { version = "^1.24.0", optional = true }
-
-[tool.poetry.extras]
-docs = ["sphinx", "sphinx_autodoc_typehints", "sphinx-copybutton", "furo"]
-uvloop = ["uvloop"]
-
-[tool.poetry.group.test]
-optional = true
-
-[tool.poetry.group.test.dependencies]
-pytest = "^7.2"
-pytest-asyncio = "^0.20.1"
-pytest-cov = "^4.0"
-pytest-xdist = "^3.0.0"
-coverage = { extras = ["toml"], version = "^6.5" }
-hypothesis = "^6.56"
-aiosmtpd = "^1.4.2"
-trustme = "^0.9.0"
-
-[tool.poetry.group.typecheck]
-optional = true
-
-[tool.poetry.group.typecheck.dependencies]
-mypy = "^0.982"
-
-[tool.poetry.group.security]
-optional = true
-
-[tool.poetry.group.security.dependencies]
-bandit = "^1.7.4"
-safety = "^2.3.1"
 
-[tool.poetry.group.lint]
-optional = true
+[project.urls]
+Documentation = "https://aiosmtplib.readthedocs.io/en/stable/"
+Changelog = "https://github.com/cole/aiosmtplib/blob/main/CHANGELOG.rst"
+GitHub = "https://github.com/cole/aiosmtplib"
+
+[project.optional-dependencies]
+uvloop = ["uvloop>=0.18"]
+# Docs extra is planned for removal in 4.x
+docs = [
+    "sphinx>=7.0.0",
+    "sphinx_autodoc_typehints>=1.24.0",
+    "sphinx-copybutton>=0.5.0",
+    "furo>=2023.9.10",
+]
 
-[tool.poetry.group.lint.dependencies]
-ruff = "^0.0.244"
+[tool.hatch.version]
+path = "src/aiosmtplib/__init__.py"
 
 [tool.pytest.ini_options]
 asyncio_mode = "strict"
@@ -110,3 +75,10 @@ exclude_lines = [
     "raise NotImplementedError",
     "if __name__ == .__main__.:",
 ]
+
+[tool.coverage.paths]
+source = [
+    "src/aiosmtplib",
+    "/opt/pypy/lib/pypy3.*/site-packages/aiosmtplib",
+    "/usr/local/lib/python3.*/site-packages/aiosmtplib",
+]
diff -pruN 3.0.1-1/requirements-dev.txt 3.0.2-1/requirements-dev.txt
--- 3.0.1-1/requirements-dev.txt	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/requirements-dev.txt	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,8 @@
+pytest>=7.2
+pytest-asyncio>=0.20.1
+pytest-cov>=4.0
+pytest-xdist>=3.0
+coverage[toml]>=6.5
+hypothesis>=6.56
+aiosmtpd>=1.4.2
+trustme>=0.9.0
diff -pruN 3.0.1-1/setup.py 3.0.2-1/setup.py
--- 3.0.1-1/setup.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/setup.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,34 +0,0 @@
-# -*- coding: utf-8 -*-
-from setuptools import setup
-
-packages = \
-['aiosmtplib']
-
-package_data = \
-{'': ['*']}
-
-extras_require = \
-{'docs': ['sphinx>=7.0.0,<8.0.0',
-          'furo>=2023.9.10,<2024.0.0',
-          'sphinx-copybutton>=0.5.0,<0.6.0',
-          'sphinx_autodoc_typehints>=1.24.0,<2.0.0'],
- 'uvloop': ['uvloop>=0.18,<0.19']}
-
-setup_kwargs = {
-    'name': 'aiosmtplib',
-    'version': '3.0.1',
-    'description': 'asyncio SMTP client',
-    'long_description': 'aiosmtplib\n==========\n\n|circleci| |precommit.ci| |codecov| |pypi-version| |pypi-status| |downloads| |pypi-python-versions|\n|pypi-license|\n\n------------\n\naiosmtplib is an asynchronous SMTP client for use with asyncio.\n\nFor documentation, see `Read The Docs`_.\n\nQuickstart\n----------\n\n\n..\n  start quickstart\n\n.. code-block:: python\n\n    import asyncio\n    from email.message import EmailMessage\n\n    import aiosmtplib\n\n    message = EmailMessage()\n    message["From"] = "root@localhost"\n    message["To"] = "somebody@example.com"\n    message["Subject"] = "Hello World!"\n    message.set_content("Sent via aiosmtplib")\n\n    asyncio.run(aiosmtplib.send(message, hostname="127.0.0.1", port=25))\n\n..\n  end quickstart\n\nRequirements\n------------\n\n..\n  start requirements\n\nPython 3.8+ is required.\n\n..\n  end requirements\n\n\nBug Reporting\n-------------\n\n..\n  start bug-reporting\n\nBug reports (and feature requests) are welcome via `Github issues`_.\n\n.. _Github issues: https://github.com/cole/aiosmtplib/issues\n\n..\n  end bug-reporting\n\n\n.. |circleci| image:: https://circleci.com/gh/cole/aiosmtplib/tree/main.svg?style=shield\n           :target: https://circleci.com/gh/cole/aiosmtplib/tree/main\n           :alt: "aiosmtplib CircleCI build status"\n.. |pypi-version| image:: https://img.shields.io/pypi/v/aiosmtplib.svg\n                 :target: https://pypi.python.org/pypi/aiosmtplib\n                 :alt: "aiosmtplib on the Python Package Index"\n.. |pypi-python-versions| image:: https://img.shields.io/pypi/pyversions/aiosmtplib.svg\n.. |pypi-status| image:: https://img.shields.io/pypi/status/aiosmtplib.svg\n.. |pypi-license| image:: https://img.shields.io/pypi/l/aiosmtplib.svg\n.. |codecov| image:: https://codecov.io/gh/cole/aiosmtplib/branch/main/graph/badge.svg\n             :target: https://codecov.io/gh/cole/aiosmtplib\n.. |downloads| image:: https://pepy.tech/badge/aiosmtplib\n               :target: https://pepy.tech/project/aiosmtplib\n               :alt: "aiosmtplib on pypy.tech"\n.. |precommit.ci| image:: https://results.pre-commit.ci/badge/github/cole/aiosmtplib/main.svg\n                  :target: https://results.pre-commit.ci/latest/github/cole/aiosmtplib/main\n                  :alt: "pre-commit.ci status"\n.. _Read The Docs: https://aiosmtplib.readthedocs.io/en/stable/\n',
-    'author': 'Cole Maclean',
-    'author_email': 'hi@colemaclean.dev',
-    'maintainer': 'None',
-    'maintainer_email': 'None',
-    'url': 'https://github.com/cole/aiosmtplib',
-    'packages': packages,
-    'package_data': package_data,
-    'extras_require': extras_require,
-    'python_requires': '>=3.8,<4.0',
-}
-
-
-setup(**setup_kwargs)
diff -pruN 3.0.1-1/src/aiosmtplib/__init__.py 3.0.2-1/src/aiosmtplib/__init__.py
--- 3.0.1-1/src/aiosmtplib/__init__.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/__init__.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,61 @@
+"""
+aiosmtplib
+==========
+
+An asyncio SMTP client.
+
+Originally based on smtplib from the Python 3 standard library by:
+The Dragon De Monsyne <dragondm@integral.org>
+
+Author: Cole Maclean <hi@colemaclean.dev>
+"""
+
+from .api import send
+from .errors import (
+    SMTPAuthenticationError,
+    SMTPConnectError,
+    SMTPConnectTimeoutError,
+    SMTPDataError,
+    SMTPException,
+    SMTPHeloError,
+    SMTPNotSupported,
+    SMTPReadTimeoutError,
+    SMTPRecipientRefused,
+    SMTPRecipientsRefused,
+    SMTPResponseException,
+    SMTPSenderRefused,
+    SMTPServerDisconnected,
+    SMTPTimeoutError,
+    SMTPConnectResponseError,
+)
+from .response import SMTPResponse
+from .smtp import SMTP
+from .typing import SMTPStatus
+
+
+__title__ = "aiosmtplib"
+__version__ = "3.0.2"
+__author__ = "Cole Maclean"
+__license__ = "MIT"
+__copyright__ = "Copyright 2022 Cole Maclean"
+__all__ = (
+    "send",
+    "SMTP",
+    "SMTPResponse",
+    "SMTPStatus",
+    "SMTPAuthenticationError",
+    "SMTPConnectError",
+    "SMTPDataError",
+    "SMTPException",
+    "SMTPHeloError",
+    "SMTPNotSupported",
+    "SMTPRecipientRefused",
+    "SMTPRecipientsRefused",
+    "SMTPResponseException",
+    "SMTPSenderRefused",
+    "SMTPServerDisconnected",
+    "SMTPTimeoutError",
+    "SMTPConnectTimeoutError",
+    "SMTPReadTimeoutError",
+    "SMTPConnectResponseError",
+)
diff -pruN 3.0.1-1/src/aiosmtplib/__main__.py 3.0.2-1/src/aiosmtplib/__main__.py
--- 3.0.1-1/src/aiosmtplib/__main__.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/__main__.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,30 @@
+from aiosmtplib.smtp import SMTP, SMTP_PORT
+
+
+raw_hostname = input("SMTP server hostname [localhost]: ")  # nosec
+raw_port = input(f"SMTP server port [{SMTP_PORT}]: ")  # nosec
+raw_sender = input("From: ")  # nosec
+raw_recipients = input("To: ")  # nosec
+
+hostname = raw_hostname or "localhost"
+port = int(raw_port) if raw_port else SMTP_PORT
+recipients = raw_recipients.split(",")
+lines = []
+
+print("Enter message, end with ^D:")
+while True:
+    try:
+        lines.append(input())  # nosec
+    except EOFError:
+        break
+
+message = "\n".join(lines)
+message_len = len(message.encode("utf-8"))
+print(f"Message length (bytes): {message_len}")
+
+smtp_client = SMTP(hostname=hostname or "localhost", port=port, start_tls=False)
+sendmail_errors, sendmail_response = smtp_client.sendmail_sync(
+    raw_sender, recipients, message
+)
+
+print(f"Server response: {sendmail_response}")
diff -pruN 3.0.1-1/src/aiosmtplib/api.py 3.0.2-1/src/aiosmtplib/api.py
--- 3.0.1-1/src/aiosmtplib/api.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/api.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,140 @@
+"""
+Main public API.
+"""
+
+import email.message
+import socket
+import ssl
+from typing import Dict, Optional, Sequence, Tuple, Union, cast
+
+from .response import SMTPResponse
+from .smtp import DEFAULT_TIMEOUT, SMTP
+from .typing import SocketPathType
+
+
+__all__ = ("send",)
+
+
+async def send(
+    message: Union[email.message.EmailMessage, email.message.Message, str, bytes],
+    /,
+    *,
+    sender: Optional[str] = None,
+    recipients: Optional[Union[str, Sequence[str]]] = None,
+    mail_options: Optional[Sequence[str]] = None,
+    rcpt_options: Optional[Sequence[str]] = None,
+    hostname: Optional[str] = "localhost",
+    port: Optional[int] = None,
+    username: Optional[Union[str, bytes]] = None,
+    password: Optional[Union[str, bytes]] = None,
+    local_hostname: Optional[str] = None,
+    source_address: Optional[Tuple[str, int]] = None,
+    timeout: Optional[float] = DEFAULT_TIMEOUT,
+    use_tls: bool = False,
+    start_tls: Optional[bool] = None,
+    validate_certs: bool = True,
+    client_cert: Optional[str] = None,
+    client_key: Optional[str] = None,
+    tls_context: Optional[ssl.SSLContext] = None,
+    cert_bundle: Optional[str] = None,
+    socket_path: Optional[SocketPathType] = None,
+    sock: Optional[socket.socket] = None,
+) -> Tuple[Dict[str, SMTPResponse], str]:
+    """
+    Send an email message. On await, connects to the SMTP server using the details
+    provided, sends the message, then disconnects.
+
+    :param message:  Message text. Either an :py:class:`email.message.EmailMessage`
+        object, ``str`` or ``bytes``. If an :py:class:`email.message.EmailMessage`
+        object is provided, sender and recipients set in the message headers will be
+        used, unless overridden by the respective keyword arguments.
+    :keyword sender:  From email address. Not required if an
+        :py:class:`email.message.EmailMessage` object is provided for the `message`
+        argument.
+    :keyword recipients: Recipient email addresses. Not required if an
+        :py:class:`email.message.EmailMessage` object is provided for the `message`
+        argument.
+    :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
+    :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
+        ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
+    :keyword username:  Username to login as after connect.
+    :keyword password:  Password for login after connect.
+    :keyword local_hostname: The hostname of the client.  If specified, used as the
+        FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
+        :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
+    :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
+        as its source address before connecting. If the host is '' and port is 0,
+        the OS default behavior will be used.
+    :keyword timeout: Default timeout value for the connection, in seconds.
+        Defaults to 60.
+    :keyword use_tls: If True, make the initial connection to the server
+        over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
+        STARTTLS, ``use_tls`` should be ``False``.
+    :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
+        If ``None`` (the default), upgrade will be initiated if supported by the
+        server.
+        If ``True``, and upgrade will be initiated regardless of server support.
+        If ``False``, no upgrade will occur.
+        Mutually exclusive with ``use_tls``.
+    :keyword validate_certs: Determines if server certificates are
+        validated. Defaults to ``True``.
+    :keyword client_cert: Path to client side certificate, for TLS.
+    :keyword client_key: Path to client side key, for TLS.
+    :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
+        Mutually exclusive with ``client_cert``/``client_key``.
+    :keyword cert_bundle: Path to certificate bundle, for TLS verification.
+    :keyword socket_path: Path to a Unix domain socket. Not compatible with
+        hostname or port. Accepts str, bytes, or a pathlike object.
+    :keyword sock: An existing, connected socket object. If given, none of
+        hostname, port, or socket_path should be provided.
+
+    :raises ValueError: required arguments missing or mutually exclusive options
+        provided
+    """
+    if not isinstance(message, (email.message.EmailMessage, email.message.Message)):
+        if not recipients:
+            raise ValueError("Recipients must be provided with raw messages.")
+        if not sender:
+            raise ValueError("Sender must be provided with raw messages.")
+
+    sender = cast(str, sender)
+    recipients = cast(Union[str, Sequence[str]], recipients)
+
+    client = SMTP(
+        hostname=hostname,
+        port=port,
+        local_hostname=local_hostname,
+        source_address=source_address,
+        timeout=timeout,
+        use_tls=use_tls,
+        start_tls=start_tls,
+        validate_certs=validate_certs,
+        client_cert=client_cert,
+        client_key=client_key,
+        tls_context=tls_context,
+        cert_bundle=cert_bundle,
+        socket_path=socket_path,
+        sock=sock,
+        username=username,
+        password=password,
+    )
+
+    async with client:
+        if isinstance(message, (email.message.EmailMessage, email.message.Message)):
+            result = await client.send_message(
+                message,
+                sender=sender,
+                recipients=recipients,
+                mail_options=mail_options,
+                rcpt_options=rcpt_options,
+            )
+        else:
+            result = await client.sendmail(
+                sender,
+                recipients,
+                message,
+                mail_options=mail_options,
+                rcpt_options=rcpt_options,
+            )
+
+    return result
diff -pruN 3.0.1-1/src/aiosmtplib/auth.py 3.0.2-1/src/aiosmtplib/auth.py
--- 3.0.1-1/src/aiosmtplib/auth.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/auth.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,73 @@
+"""
+Authentication related methods.
+"""
+
+import base64
+import hmac
+from typing import Tuple, Union
+
+
+__all__ = ("auth_crammd5_verify", "auth_plain_encode", "auth_login_encode")
+
+
+def _ensure_bytes(value: Union[str, bytes]) -> bytes:
+    if isinstance(value, bytes):
+        return value
+
+    return value.encode("utf-8")
+
+
+def auth_crammd5_verify(
+    username: Union[str, bytes],
+    password: Union[str, bytes],
+    challenge: Union[str, bytes],
+    /,
+) -> bytes:
+    """
+    CRAM-MD5 auth uses the password as a shared secret to MD5 the server's
+    response, and sends the username combined with that (base64 encoded).
+    """
+    username_bytes = _ensure_bytes(username)
+    password_bytes = _ensure_bytes(password)
+    decoded_challenge = base64.b64decode(challenge)
+
+    md5_digest = hmac.new(password_bytes, msg=decoded_challenge, digestmod="md5")
+    verification = username_bytes + b" " + md5_digest.hexdigest().encode("ascii")
+    encoded_verification = base64.b64encode(verification)
+
+    return encoded_verification
+
+
+def auth_plain_encode(
+    username: Union[str, bytes],
+    password: Union[str, bytes],
+    /,
+) -> bytes:
+    """
+    PLAIN auth base64 encodes the username and password together.
+    """
+    username_bytes = _ensure_bytes(username)
+    password_bytes = _ensure_bytes(password)
+
+    username_and_password = b"\0" + username_bytes + b"\0" + password_bytes
+    encoded = base64.b64encode(username_and_password)
+
+    return encoded
+
+
+def auth_login_encode(
+    username: Union[str, bytes],
+    password: Union[str, bytes],
+    /,
+) -> Tuple[bytes, bytes]:
+    """
+    LOGIN auth base64 encodes the username and password and sends them
+    in sequence.
+    """
+    username_bytes = _ensure_bytes(username)
+    password_bytes = _ensure_bytes(password)
+
+    encoded_username = base64.b64encode(username_bytes)
+    encoded_password = base64.b64encode(password_bytes)
+
+    return encoded_username, encoded_password
diff -pruN 3.0.1-1/src/aiosmtplib/email.py 3.0.2-1/src/aiosmtplib/email.py
--- 3.0.1-1/src/aiosmtplib/email.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/email.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,188 @@
+"""
+Email message and address formatting/parsing functions.
+"""
+
+import copy
+import email.charset
+import email.generator
+import email.header
+import email.headerregistry
+import email.message
+import email.policy
+import email.utils
+import io
+import re
+from typing import List, Optional, Tuple, Union
+
+
+__all__ = (
+    "extract_recipients",
+    "extract_sender",
+    "flatten_message",
+    "parse_address",
+    "quote_address",
+)
+
+
+LINE_SEP = "\r\n"
+SPECIALS_REGEX = re.compile(r'[][\\()<>@,:;".]')
+ESCAPES_REGEX = re.compile(r'[\\"]')
+UTF8_CHARSET = email.charset.Charset("utf-8")
+
+
+def parse_address(address: str) -> str:
+    """
+    Parse an email address, falling back to the raw string given.
+    """
+    display_name, parsed_address = email.utils.parseaddr(address)
+
+    return parsed_address or address.strip()
+
+
+def quote_address(address: str) -> str:
+    """
+    Quote a subset of the email addresses defined by RFC 821.
+    """
+    parsed_address = parse_address(address)
+    return f"<{parsed_address}>"
+
+
+def formataddr(pair: Tuple[str, str]) -> str:
+    """
+    Copied from the standard library, and modified to handle international (UTF-8)
+    email addresses.
+
+    The inverse of parseaddr(), this takes a 2-tuple of the form
+    (realname, email_address) and returns the string value suitable
+    for an RFC 2822 From, To or Cc header.
+    If the first element of pair is false, then the second element is
+    returned unmodified.
+    """
+    name, address = pair
+    if name:
+        encoded_name = UTF8_CHARSET.header_encode(name)
+        return f"{encoded_name} <{address}>"
+    else:
+        quotes = ""
+        if SPECIALS_REGEX.search(name):
+            quotes = '"'
+            name = ESCAPES_REGEX.sub(r"\\\g<0>", name)
+            return f"{quotes}{name}{quotes} <{address}>"
+
+    return address
+
+
+def flatten_message(
+    message: Union[email.message.EmailMessage, email.message.Message],
+    /,
+    *,
+    utf8: bool = False,
+    cte_type: str = "8bit",
+) -> bytes:
+    # Make a local copy so we can delete the bcc headers.
+    message_copy = copy.copy(message)
+    del message_copy["Bcc"]
+    del message_copy["Resent-Bcc"]
+
+    policy: email.policy.Policy
+    if isinstance(message, email.message.EmailMessage):
+        # New message class, default policy
+        policy = email.policy.default.clone(
+            linesep=LINE_SEP,
+            utf8=utf8,
+            cte_type=cte_type,
+        )
+    else:
+        # Old message class, Compat32 policy.
+        # Compat32 cannot use UTF8
+        policy = email.policy.compat32.clone(linesep=LINE_SEP, cte_type=cte_type)
+
+    with io.BytesIO() as messageio:
+        generator = email.generator.BytesGenerator(messageio, policy=policy)
+        generator.flatten(message_copy)
+        flat_message = messageio.getvalue()
+
+    return flat_message
+
+
+def extract_addresses(
+    header: Union[str, email.headerregistry.AddressHeader, email.header.Header],
+    /,
+) -> List[str]:
+    """
+    Convert address headers into raw email addresses, suitable for use in
+    low level SMTP commands.
+    """
+    addresses = []
+    if isinstance(header, email.headerregistry.AddressHeader):
+        for address in header.addresses:
+            # If the object has been assigned an iterable, it's possible to get
+            # a string here
+            if isinstance(address, email.headerregistry.Address):
+                addresses.append(address.addr_spec)
+            else:
+                addresses.append(parse_address(address))
+    elif isinstance(header, email.header.Header):
+        for address_bytes, charset in email.header.decode_header(header):
+            if charset is None:
+                charset = "ascii"
+            addresses.append(parse_address(str(address_bytes, encoding=charset)))
+    else:
+        addresses.extend(addr for _, addr in email.utils.getaddresses([header]))
+
+    return addresses
+
+
+def extract_sender(
+    message: Union[email.message.EmailMessage, email.message.Message],
+    /,
+) -> Optional[str]:
+    """
+    Extract the sender from the message object given.
+    """
+    resent_dates = message.get_all("Resent-Date")
+
+    if resent_dates is not None and len(resent_dates) > 1:
+        raise ValueError("Message has more than one 'Resent-' header block")
+    elif resent_dates:
+        sender_header_name = "Resent-Sender"
+        from_header_name = "Resent-From"
+    else:
+        sender_header_name = "Sender"
+        from_header_name = "From"
+
+    # Prefer the sender field per RFC 2822:3.6.2.
+    if sender_header_name in message:
+        sender_header = message[sender_header_name]
+    else:
+        sender_header = message[from_header_name]
+
+    if sender_header is None:
+        return None
+
+    return extract_addresses(sender_header)[0]
+
+
+def extract_recipients(
+    message: Union[email.message.EmailMessage, email.message.Message],
+    /,
+) -> List[str]:
+    """
+    Extract the recipients from the message object given.
+    """
+    recipients: List[str] = []
+
+    resent_dates = message.get_all("Resent-Date")
+
+    if resent_dates is not None and len(resent_dates) > 1:
+        raise ValueError("Message has more than one 'Resent-' header block")
+    elif resent_dates:
+        recipient_headers = ("Resent-To", "Resent-Cc", "Resent-Bcc")
+    else:
+        recipient_headers = ("To", "Cc", "Bcc")
+
+    for header in recipient_headers:
+        for recipient in message.get_all(header, failobj=[]):
+            recipients.extend(extract_addresses(recipient))
+
+    return recipients
diff -pruN 3.0.1-1/src/aiosmtplib/errors.py 3.0.2-1/src/aiosmtplib/errors.py
--- 3.0.1-1/src/aiosmtplib/errors.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/errors.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,137 @@
+from asyncio import TimeoutError
+from typing import List
+
+
+__all__ = (
+    "SMTPAuthenticationError",
+    "SMTPConnectError",
+    "SMTPDataError",
+    "SMTPException",
+    "SMTPHeloError",
+    "SMTPNotSupported",
+    "SMTPRecipientRefused",
+    "SMTPRecipientsRefused",
+    "SMTPResponseException",
+    "SMTPSenderRefused",
+    "SMTPServerDisconnected",
+    "SMTPTimeoutError",
+    "SMTPConnectTimeoutError",
+    "SMTPReadTimeoutError",
+    "SMTPConnectResponseError",
+)
+
+
+class SMTPException(Exception):
+    """
+    Base class for all SMTP exceptions.
+    """
+
+    def __init__(self, message: str, /) -> None:
+        self.message = message
+        self.args = (message,)
+
+
+class SMTPServerDisconnected(SMTPException, ConnectionError):
+    """
+    The connection was lost unexpectedly, or a command was run that requires
+    a connection.
+    """
+
+
+class SMTPConnectError(SMTPException, ConnectionError):
+    """
+    An error occurred while connecting to the SMTP server.
+    """
+
+
+class SMTPTimeoutError(SMTPException, TimeoutError):
+    """
+    A timeout occurred while performing a network operation.
+    """
+
+
+class SMTPConnectTimeoutError(SMTPTimeoutError, SMTPConnectError):
+    """
+    A timeout occurred while connecting to the SMTP server.
+    """
+
+
+class SMTPReadTimeoutError(SMTPTimeoutError):
+    """
+    A timeout occurred while waiting for a response from the SMTP server.
+    """
+
+
+class SMTPNotSupported(SMTPException):
+    """
+    A command or argument sent to the SMTP server is not supported.
+    """
+
+
+class SMTPResponseException(SMTPException):
+    """
+    Base class for all server responses with error codes.
+    """
+
+    def __init__(self, code: int, message: str, /) -> None:
+        self.code = code
+        self.message = message
+        self.args = (code, message)
+
+
+class SMTPConnectResponseError(SMTPResponseException, SMTPConnectError):
+    """
+    The SMTP server returned an invalid response code after connecting.
+    """
+
+
+class SMTPHeloError(SMTPResponseException):
+    """
+    Server refused HELO or EHLO.
+    """
+
+
+class SMTPDataError(SMTPResponseException):
+    """
+    Server refused DATA content.
+    """
+
+
+class SMTPAuthenticationError(SMTPResponseException):
+    """
+    Server refused our AUTH request; may be caused by invalid credentials.
+    """
+
+
+class SMTPSenderRefused(SMTPResponseException):
+    """
+    SMTP server refused the message sender.
+    """
+
+    def __init__(self, code: int, message: str, sender: str, /) -> None:
+        self.code = code
+        self.message = message
+        self.sender = sender
+        self.args = (code, message, sender)
+
+
+class SMTPRecipientRefused(SMTPResponseException):
+    """
+    SMTP server refused a message recipient.
+    """
+
+    def __init__(self, code: int, message: str, recipient: str, /) -> None:
+        self.code = code
+        self.message = message
+        self.recipient = recipient
+        self.args = (code, message, recipient)
+
+
+class SMTPRecipientsRefused(SMTPException):
+    """
+    SMTP server refused multiple recipients.
+    """
+
+    def __init__(self, recipients: List[SMTPRecipientRefused], /) -> None:
+        self.recipients = recipients
+        self.args = (recipients,)
diff -pruN 3.0.1-1/src/aiosmtplib/esmtp.py 3.0.2-1/src/aiosmtplib/esmtp.py
--- 3.0.1-1/src/aiosmtplib/esmtp.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/esmtp.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,73 @@
+"""
+ESMTP utils
+"""
+
+import re
+from typing import Dict, List, Tuple
+
+
+__all__ = ("parse_esmtp_extensions",)
+
+
+OLDSTYLE_AUTH_REGEX = re.compile(r"auth=(?P<auth>.*)", flags=re.I)
+EXTENSIONS_REGEX = re.compile(r"(?P<ext>[A-Za-z0-9][A-Za-z0-9\-]*) ?")
+
+
+def parse_esmtp_extensions(message: str) -> Tuple[Dict[str, str], List[str]]:
+    """
+    Parse an EHLO response from the server into a dict of {extension: params}
+    and a list of auth method names.
+
+    It might look something like:
+
+         220 size.does.matter.af.MIL (More ESMTP than Crappysoft!)
+         EHLO heaven.af.mil
+         250-size.does.matter.af.MIL offers FIFTEEN extensions:
+         250-8BITMIME
+         250-PIPELINING
+         250-DSN
+         250-ENHANCEDSTATUSCODES
+         250-EXPN
+         250-HELP
+         250-SAML
+         250-SEND
+         250-SOML
+         250-TURN
+         250-XADR
+         250-XSTA
+         250-ETRN
+         250-XGEN
+         250 SIZE 51200000
+    """
+    esmtp_extensions = {}
+    auth_types: List[str] = []
+
+    response_lines = message.split("\n")
+
+    # ignore the first line
+    for line in response_lines[1:]:
+        # To be able to communicate with as many SMTP servers as possible,
+        # we have to take the old-style auth advertisement into account,
+        # because:
+        # 1) Else our SMTP feature parser gets confused.
+        # 2) There are some servers that only advertise the auth methods we
+        #    support using the old style.
+        auth_match = OLDSTYLE_AUTH_REGEX.match(line)
+        if auth_match is not None:
+            auth_type = auth_match.group("auth")
+            auth_types.append(auth_type.lower().strip())
+
+        # RFC 1869 requires a space between ehlo keyword and parameters.
+        # It's actually stricter, in that only spaces are allowed between
+        # parameters, but were not going to check for that here.  Note
+        # that the space isn't present if there are no parameters.
+        extensions = EXTENSIONS_REGEX.match(line)
+        if extensions is not None:
+            extension = extensions.group("ext").lower()
+            params = extensions.string[extensions.end("ext") :].strip()
+            esmtp_extensions[extension] = params
+
+            if extension == "auth":
+                auth_types.extend([param.strip().lower() for param in params.split()])
+
+    return esmtp_extensions, auth_types
diff -pruN 3.0.1-1/src/aiosmtplib/protocol.py 3.0.2-1/src/aiosmtplib/protocol.py
--- 3.0.1-1/src/aiosmtplib/protocol.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/protocol.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,374 @@
+"""
+An ``asyncio.Protocol`` subclass for lower level IO handling.
+"""
+
+import asyncio
+import collections
+import re
+import ssl
+from typing import Deque, Optional, cast
+
+from .errors import (
+    SMTPDataError,
+    SMTPReadTimeoutError,
+    SMTPResponseException,
+    SMTPServerDisconnected,
+    SMTPTimeoutError,
+)
+from .response import SMTPResponse
+from .typing import SMTPStatus
+
+
+__all__ = ("SMTPProtocol",)
+
+
+MAX_LINE_LENGTH = 8192
+LINE_ENDINGS_REGEX = re.compile(rb"(?:\r\n|\n|\r(?!\n))")
+PERIOD_REGEX = re.compile(rb"(?m)^\.")
+
+
+class FlowControlMixin(asyncio.Protocol):
+    """
+    Reusable flow control logic for StreamWriter.drain().
+    This implements the protocol methods pause_writing(),
+    resume_writing() and connection_lost().  If the subclass overrides
+    these it must call the super methods.
+    StreamWriter.drain() must wait for _drain_helper() coroutine.
+
+    Copied from stdlib as per recommendation: https://bugs.python.org/msg343685.
+    Logging and asserts removed, type annotations added.
+    """
+
+    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None):
+        if loop is None:
+            self._loop = asyncio.get_event_loop()
+        else:
+            self._loop = loop
+
+        self._paused = False
+        self._drain_waiters: Deque[asyncio.Future[None]] = collections.deque()
+        self._connection_lost = False
+
+    def pause_writing(self) -> None:
+        self._paused = True
+
+    def resume_writing(self) -> None:
+        self._paused = False
+
+        for waiter in self._drain_waiters:
+            if not waiter.done():
+                waiter.set_result(None)
+
+    def connection_lost(self, exc: Optional[Exception]) -> None:
+        self._connection_lost = True
+        # Wake up the writer(s) if currently paused.
+        if not self._paused:
+            return
+
+        for waiter in self._drain_waiters:
+            if not waiter.done():
+                if exc is None:
+                    waiter.set_result(None)
+                else:
+                    waiter.set_exception(exc)
+
+    async def _drain_helper(self) -> None:
+        if self._connection_lost:
+            raise ConnectionResetError("Connection lost")
+        if not self._paused:
+            return
+        waiter = self._loop.create_future()
+        self._drain_waiters.append(waiter)
+        try:
+            await waiter
+        finally:
+            self._drain_waiters.remove(waiter)
+
+    def _get_close_waiter(self, stream: asyncio.StreamWriter) -> "asyncio.Future[None]":
+        raise NotImplementedError
+
+
+class SMTPProtocol(FlowControlMixin, asyncio.BaseProtocol):
+    def __init__(
+        self,
+        loop: Optional[asyncio.AbstractEventLoop] = None,
+    ) -> None:
+        super().__init__(loop=loop)
+        self._over_ssl = False
+        self._buffer = bytearray()
+        self._response_waiter: Optional[asyncio.Future[SMTPResponse]] = None
+
+        self.transport: Optional[asyncio.BaseTransport] = None
+        self._command_lock: Optional[asyncio.Lock] = None
+        self._closed: "asyncio.Future[None]" = self._loop.create_future()
+        self._quit_sent = False
+
+    def _get_close_waiter(self, stream: asyncio.StreamWriter) -> "asyncio.Future[None]":
+        return self._closed
+
+    def __del__(self) -> None:
+        # Avoid 'Future exception was never retrieved' warnings
+        # Some unknown race conditions can sometimes trigger these :(
+        self._retrieve_response_exception()
+
+    @property
+    def is_connected(self) -> bool:
+        """
+        Check if our transport is still connected.
+        """
+        return bool(self.transport is not None and not self.transport.is_closing())
+
+    def connection_made(self, transport: asyncio.BaseTransport) -> None:
+        self.transport = cast(asyncio.Transport, transport)
+        self._over_ssl = transport.get_extra_info("sslcontext") is not None
+        self._response_waiter = self._loop.create_future()
+        self._command_lock = asyncio.Lock()
+        self._quit_sent = False
+
+    def connection_lost(self, exc: Optional[Exception]) -> None:
+        super().connection_lost(exc)
+
+        if not self._quit_sent:
+            smtp_exc = SMTPServerDisconnected("Connection lost")
+            if exc:
+                smtp_exc.__cause__ = exc
+
+            if self._response_waiter and not self._response_waiter.done():
+                self._response_waiter.set_exception(smtp_exc)
+
+        self.transport = None
+        self._command_lock = None
+
+    def data_received(self, data: bytes) -> None:
+        if self._response_waiter is None:
+            raise RuntimeError(
+                f"data_received called without a response waiter set: {data!r}"
+            )
+        elif self._response_waiter.done():
+            # We got a response without issuing a command; ignore it.
+            return
+
+        self._buffer.extend(data)
+
+        # If we got an obvious partial message, don't try to parse the buffer
+        last_linebreak = data.rfind(b"\n")
+        if (
+            last_linebreak == -1
+            or data[last_linebreak + 3 : last_linebreak + 4] == b"-"
+        ):
+            return
+
+        try:
+            response = self._read_response_from_buffer()
+        except Exception as exc:
+            self._response_waiter.set_exception(exc)
+        else:
+            if response is not None:
+                self._response_waiter.set_result(response)
+
+    def eof_received(self) -> bool:
+        exc = SMTPServerDisconnected("Unexpected EOF received")
+        if self._response_waiter and not self._response_waiter.done():
+            self._response_waiter.set_exception(exc)
+
+        # Returning false closes the transport
+        return False
+
+    def _retrieve_response_exception(self) -> Optional[BaseException]:
+        """
+        Return any exception that has been set on the response waiter.
+
+        Used to avoid 'Future exception was never retrieved' warnings
+        """
+        if (
+            self._response_waiter
+            and self._response_waiter.done()
+            and not self._response_waiter.cancelled()
+        ):
+            return self._response_waiter.exception()
+
+        return None
+
+    def _read_response_from_buffer(self) -> Optional[SMTPResponse]:
+        """Parse the actual response (if any) from the data buffer"""
+        code = -1
+        message = bytearray()
+        offset = 0
+        message_complete = False
+
+        while True:
+            line_end_index = self._buffer.find(b"\n", offset)
+            if line_end_index == -1:
+                break
+
+            line = bytes(self._buffer[offset : line_end_index + 1])
+
+            if len(line) > MAX_LINE_LENGTH:
+                raise SMTPResponseException(
+                    SMTPStatus.unrecognized_command, "Response too long"
+                )
+
+            try:
+                code = int(line[:3])
+            except ValueError:
+                raise SMTPResponseException(
+                    SMTPStatus.invalid_response.value,
+                    f"Malformed SMTP response line: {line!r}",
+                ) from None
+
+            offset += len(line)
+            if len(message):
+                message.extend(b"\n")
+            message.extend(line[4:].strip(b" \t\r\n"))
+            if line[3:4] != b"-":
+                message_complete = True
+                break
+
+        if message_complete:
+            response = SMTPResponse(
+                code, bytes(message).decode("utf-8", "surrogateescape")
+            )
+            del self._buffer[:offset]
+            return response
+        else:
+            return None
+
+    async def read_response(self, timeout: Optional[float] = None) -> SMTPResponse:
+        """
+        Get a status response from the server.
+
+        This method must be awaited once per command sent; if multiple commands
+        are written to the transport without awaiting, response data will be lost.
+
+        Returns an :class:`.response.SMTPResponse` namedtuple consisting of:
+          - server response code (e.g. 250, or such, if all goes well)
+          - server response string (multiline responses are converted to a
+            single, multiline string).
+        """
+        if self._response_waiter is None:
+            raise SMTPServerDisconnected("Connection lost")
+
+        try:
+            result = await asyncio.wait_for(self._response_waiter, timeout)
+        except (TimeoutError, asyncio.TimeoutError) as exc:
+            raise SMTPReadTimeoutError("Timed out waiting for server response") from exc
+        finally:
+            # If we were disconnected, don't create a new waiter
+            if self.transport is None:
+                self._response_waiter = None
+            else:
+                self._response_waiter = self._loop.create_future()
+
+        return result
+
+    def write(self, data: bytes) -> None:
+        if self.transport is None or self.transport.is_closing():
+            raise SMTPServerDisconnected("Connection lost")
+        if not hasattr(self.transport, "write"):
+            raise RuntimeError(
+                f"Transport {self.transport!r} does not support writing."
+            )
+
+        self.transport.write(data)  # type: ignore
+
+    async def execute_command(
+        self, *args: bytes, timeout: Optional[float] = None
+    ) -> SMTPResponse:
+        """
+        Sends an SMTP command along with any args to the server, and returns
+        a response.
+        """
+        if self._command_lock is None:
+            raise SMTPServerDisconnected("Server not connected")
+        command = b" ".join(args) + b"\r\n"
+
+        async with self._command_lock:
+            self.write(command)
+
+            if command == b"QUIT\r\n":
+                self._quit_sent = True
+
+            response = await self.read_response(timeout=timeout)
+
+        return response
+
+    async def execute_data_command(
+        self, message: bytes, timeout: Optional[float] = None
+    ) -> SMTPResponse:
+        """
+        Sends an SMTP DATA command to the server, followed by encoded message content.
+
+        Automatically quotes lines beginning with a period per RFC821.
+        Lone \\\\r and \\\\n characters are converted to \\\\r\\\\n
+        characters.
+        """
+        if self._command_lock is None:
+            raise SMTPServerDisconnected("Server not connected")
+
+        message = LINE_ENDINGS_REGEX.sub(b"\r\n", message)
+        message = PERIOD_REGEX.sub(b"..", message)
+        if not message.endswith(b"\r\n"):
+            message += b"\r\n"
+        message += b".\r\n"
+
+        async with self._command_lock:
+            self.write(b"DATA\r\n")
+            start_response = await self.read_response(timeout=timeout)
+            if start_response.code != SMTPStatus.start_input:
+                raise SMTPDataError(start_response.code, start_response.message)
+
+            self.write(message)
+            response = await self.read_response(timeout=timeout)
+            if response.code != SMTPStatus.completed:
+                raise SMTPDataError(response.code, response.message)
+
+        return response
+
+    async def start_tls(
+        self,
+        tls_context: ssl.SSLContext,
+        server_hostname: Optional[str] = None,
+        timeout: Optional[float] = None,
+    ) -> SMTPResponse:
+        """
+        Puts the connection to the SMTP server into TLS mode.
+        """
+        if self._over_ssl:
+            raise RuntimeError("Already using TLS.")
+        if self._command_lock is None:
+            raise SMTPServerDisconnected("Server not connected")
+
+        async with self._command_lock:
+            self.write(b"STARTTLS\r\n")
+            response = await self.read_response(timeout=timeout)
+            if response.code != SMTPStatus.ready:
+                raise SMTPResponseException(response.code, response.message)
+
+            # Check for disconnect after response
+            if self.transport is None or self.transport.is_closing():
+                raise SMTPServerDisconnected("Connection lost")
+
+            try:
+                tls_transport = await self._loop.start_tls(
+                    self.transport,
+                    self,
+                    tls_context,
+                    server_side=False,
+                    server_hostname=server_hostname,
+                    ssl_handshake_timeout=timeout,
+                )
+            except (TimeoutError, asyncio.TimeoutError) as exc:
+                raise SMTPTimeoutError("Timed out while upgrading transport") from exc
+            # SSLProtocol only raises ConnectionAbortedError on timeout
+            except ConnectionAbortedError as exc:
+                raise SMTPTimeoutError(exc.args[0]) from exc
+            except ConnectionResetError as exc:
+                if exc.args:
+                    message = exc.args[0]
+                else:
+                    message = "Connection was reset while upgrading transport"
+                raise SMTPServerDisconnected(message) from exc
+
+            self.transport = tls_transport
+
+        return response
diff -pruN 3.0.1-1/src/aiosmtplib/py.typed 3.0.2-1/src/aiosmtplib/py.typed
--- 3.0.1-1/src/aiosmtplib/py.typed	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/py.typed	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1 @@
+This file exists to help mypy (and other tools) find inline type hints. See PR #141 and PEP 561.
diff -pruN 3.0.1-1/src/aiosmtplib/response.py 3.0.2-1/src/aiosmtplib/response.py
--- 3.0.1-1/src/aiosmtplib/response.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/response.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,34 @@
+"""
+SMTPResponse class, a simple namedtuple of (code, message).
+"""
+
+from typing import NamedTuple
+
+
+__all__ = ("SMTPResponse",)
+
+
+class SMTPResponse(NamedTuple):
+    """
+    NamedTuple of server response code and server response message.
+
+    ``code`` and ``message`` can be accessed via attributes or indexes:
+
+        >>> response = SMTPResponse(200, "OK")
+        >>> response.message
+        'OK'
+        >>> response[0]
+        200
+        >>> response.code
+        200
+
+    """
+
+    code: int
+    message: str
+
+    def __repr__(self) -> str:
+        return f"({self.code}, {self.message})"
+
+    def __str__(self) -> str:
+        return f"{self.code} {self.message}"
diff -pruN 3.0.1-1/src/aiosmtplib/smtp.py 3.0.2-1/src/aiosmtplib/smtp.py
--- 3.0.1-1/src/aiosmtplib/smtp.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/smtp.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,1459 @@
+"""
+Main SMTP client class.
+
+Implements SMTP, ESMTP & Auth methods.
+"""
+
+import asyncio
+import email.message
+import socket
+import ssl
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple, Type, Union
+
+from .auth import auth_crammd5_verify, auth_login_encode, auth_plain_encode
+from .email import (
+    extract_recipients,
+    extract_sender,
+    flatten_message,
+    parse_address,
+    quote_address,
+)
+from .errors import (
+    SMTPAuthenticationError,
+    SMTPConnectError,
+    SMTPConnectTimeoutError,
+    SMTPException,
+    SMTPHeloError,
+    SMTPNotSupported,
+    SMTPRecipientRefused,
+    SMTPRecipientsRefused,
+    SMTPResponseException,
+    SMTPSenderRefused,
+    SMTPServerDisconnected,
+    SMTPTimeoutError,
+    SMTPConnectResponseError,
+)
+from .esmtp import parse_esmtp_extensions
+from .protocol import SMTPProtocol
+from .response import SMTPResponse
+from .typing import Default, SMTPStatus, SocketPathType, _default
+
+
+__all__ = ("SMTP", "SMTP_PORT", "SMTP_TLS_PORT", "SMTP_STARTTLS_PORT")
+
+SMTP_PORT = 25
+SMTP_TLS_PORT = 465
+SMTP_STARTTLS_PORT = 587
+DEFAULT_TIMEOUT = 60
+
+
+class SMTP:
+    """
+    Main SMTP client class.
+
+    Basic usage:
+
+        >>> event_loop = asyncio.get_event_loop()
+        >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
+        >>> event_loop.run_until_complete(smtp.connect())
+        (220, ...)
+        >>> sender = "root@localhost"
+        >>> recipients = ["somebody@localhost"]
+        >>> message = "Hello World"
+        >>> send = smtp.sendmail(sender, recipients, "Hello World")
+        >>> event_loop.run_until_complete(send)
+        ({}, 'OK')
+
+    Keyword arguments can be provided either on :meth:`__init__` or when
+    calling the :meth:`connect` method. Note that in both cases these options,
+    except for ``timeout``, are saved for later use; subsequent calls to
+    :meth:`connect` will use the same options, unless new ones are provided.
+    ``timeout`` is saved for later use when provided on :meth:`__init__`, but
+    not when calling the :meth:`connect` method.
+    """
+
+    # Preferred methods first
+    AUTH_METHODS: Tuple[str, ...] = (
+        "cram-md5",
+        "plain",
+        "login",
+    )
+
+    def __init__(
+        self,
+        *,
+        hostname: Optional[str] = "localhost",
+        port: Optional[int] = None,
+        username: Optional[Union[str, bytes]] = None,
+        password: Optional[Union[str, bytes]] = None,
+        local_hostname: Optional[str] = None,
+        source_address: Optional[Tuple[str, int]] = None,
+        timeout: Optional[float] = DEFAULT_TIMEOUT,
+        use_tls: bool = False,
+        start_tls: Optional[bool] = None,
+        validate_certs: bool = True,
+        client_cert: Optional[str] = None,
+        client_key: Optional[str] = None,
+        tls_context: Optional[ssl.SSLContext] = None,
+        cert_bundle: Optional[str] = None,
+        socket_path: Optional[SocketPathType] = None,
+        sock: Optional[socket.socket] = None,
+    ) -> None:
+        """
+        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
+        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
+            ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
+        :keyword username:  Username to login as after connect.
+        :keyword password:  Password for login after connect.
+        :keyword local_hostname: The hostname of the client.  If specified, used as the
+            FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
+            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
+        :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
+            as its source address before connecting. If the host is '' and port is 0,
+            the OS default behavior will be used.
+        :keyword timeout: Default timeout value for the connection, in seconds.
+            Defaults to 60.
+        :keyword use_tls: If True, make the initial connection to the server
+            over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
+            STARTTLS, ``use_tls`` should be ``False``.
+        :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
+            If ``None`` (the default), upgrade will be initiated if supported by the
+            server.
+            If ``True``, and upgrade will be initiated regardless of server support.
+            If ``False``, no upgrade will occur.
+            Mutually exclusive with ``use_tls``.
+        :keyword validate_certs: Determines if server certificates are
+            validated. Defaults to ``True``.
+        :keyword client_cert: Path to client side certificate, for TLS.
+        :keyword client_key: Path to client side key, for TLS.
+        :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
+            Mutually exclusive with ``client_cert``/``client_key``.
+        :keyword cert_bundle: Path to certificate bundle, for TLS verification.
+        :keyword socket_path: Path to a Unix domain socket. Not compatible with
+            hostname or port. Accepts str, bytes, or a pathlike object.
+        :keyword sock: An existing, connected socket object. If given, none of
+            hostname, port, or socket_path should be provided.
+
+        :raises ValueError: mutually exclusive options provided
+        """
+        self.protocol: Optional[SMTPProtocol] = None
+        self.transport: Optional[asyncio.BaseTransport] = None
+
+        # Kwarg defaults are provided here, and saved for connect.
+        self.hostname = hostname
+        self.port = port
+        self._login_username = username
+        self._login_password = password
+        self._local_hostname = local_hostname
+        self.timeout = timeout
+        self.use_tls = use_tls
+        self._start_tls_on_connect = start_tls
+        self.validate_certs = validate_certs
+        self.client_cert = client_cert
+        self.client_key = client_key
+        self.tls_context = tls_context
+        self.cert_bundle = cert_bundle
+        self.socket_path = socket_path
+        self.sock = sock
+        self.source_address = source_address
+
+        self.loop: Optional[asyncio.AbstractEventLoop] = None
+        self._connect_lock: Optional[asyncio.Lock] = None
+        self.last_helo_response: Optional[SMTPResponse] = None
+        self._last_ehlo_response: Optional[SMTPResponse] = None
+        self.esmtp_extensions: Dict[str, str] = {}
+        self.supports_esmtp = False
+        self.server_auth_methods: List[str] = []
+        self._sendmail_lock: Optional[asyncio.Lock] = None
+
+        self._validate_config()
+
+    async def __aenter__(self) -> "SMTP":
+        if not self.is_connected:
+            await self.connect()
+
+        return self
+
+    async def __aexit__(
+        self, exc_type: Type[BaseException], exc: BaseException, traceback: Any
+    ) -> None:
+        if isinstance(exc, (ConnectionError, TimeoutError)):
+            self.close()
+            return
+
+        try:
+            await self.quit()
+        except (SMTPServerDisconnected, SMTPResponseException, SMTPTimeoutError):
+            pass
+
+    @property
+    def is_connected(self) -> bool:
+        """
+        Check if our transport is still connected.
+        """
+        return bool(self.protocol is not None and self.protocol.is_connected)
+
+    @property
+    def local_hostname(self) -> str:
+        """
+        Get the system hostname to be sent to the SMTP server.
+        Simply caches the result of :func:`socket.getfqdn`.
+        """
+        if self._local_hostname is None:
+            self._local_hostname = socket.getfqdn()
+
+        return self._local_hostname
+
+    @property
+    def last_ehlo_response(self) -> Union[SMTPResponse, None]:
+        return self._last_ehlo_response
+
+    @last_ehlo_response.setter
+    def last_ehlo_response(self, response: SMTPResponse) -> None:
+        """
+        When setting the last EHLO response, parse the message for supported
+        extensions and auth methods.
+        """
+        extensions, auth_methods = parse_esmtp_extensions(response.message)
+        self._last_ehlo_response = response
+        self.esmtp_extensions = extensions
+        self.server_auth_methods = auth_methods
+        self.supports_esmtp = True
+
+    @property
+    def is_ehlo_or_helo_needed(self) -> bool:
+        """
+        Check if we've already received a response to an EHLO or HELO command.
+        """
+        return self.last_ehlo_response is None and self.last_helo_response is None
+
+    @property
+    def supported_auth_methods(self) -> List[str]:
+        """
+        Get all AUTH methods supported by the both server and by us.
+        """
+        return [auth for auth in self.AUTH_METHODS if auth in self.server_auth_methods]
+
+    def _update_settings_from_kwargs(
+        self,
+        hostname: Optional[Union[str, Default]] = _default,
+        port: Optional[Union[int, Default]] = _default,
+        username: Optional[Union[str, bytes, Default]] = _default,
+        password: Optional[Union[str, bytes, Default]] = _default,
+        local_hostname: Optional[Union[str, Default]] = _default,
+        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
+        use_tls: Optional[bool] = None,
+        start_tls: Optional[Union[bool, Default]] = _default,
+        validate_certs: Optional[bool] = None,
+        client_cert: Optional[Union[str, Default]] = _default,
+        client_key: Optional[Union[str, Default]] = _default,
+        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
+        cert_bundle: Optional[Union[str, Default]] = _default,
+        socket_path: Optional[Union[SocketPathType, Default]] = _default,
+        sock: Optional[Union[socket.socket, Default]] = _default,
+    ) -> None:
+        """Update our configuration from the kwargs provided.
+
+        This method can be called multiple times.
+        """
+        if hostname is not _default:
+            self.hostname = hostname
+        if use_tls is not None:
+            self.use_tls = use_tls
+        if start_tls is not _default:
+            self._start_tls_on_connect = start_tls
+        if validate_certs is not None:
+            self.validate_certs = validate_certs
+        if port is not _default:
+            self.port = port
+        if username is not _default:
+            self._login_username = username
+        if password is not _default:
+            self._login_password = password
+
+        if local_hostname is not _default:
+            self._local_hostname = local_hostname
+        if source_address is not _default:
+            self.source_address = source_address
+        if client_cert is not _default:
+            self.client_cert = client_cert
+        if client_key is not _default:
+            self.client_key = client_key
+        if tls_context is not _default:
+            self.tls_context = tls_context
+        if cert_bundle is not _default:
+            self.cert_bundle = cert_bundle
+        if socket_path is not _default:
+            self.socket_path = socket_path
+        if sock is not _default:
+            self.sock = sock
+
+    def _validate_config(self) -> None:
+        if self._start_tls_on_connect and self.use_tls:
+            raise ValueError("The start_tls and use_tls options are not compatible.")
+
+        if self.tls_context is not None and self.client_cert is not None:
+            raise ValueError(
+                "Either a TLS context or a certificate/key must be provided"
+            )
+
+        if self.sock is not None and any([self.hostname, self.port, self.socket_path]):
+            raise ValueError(
+                "The socket option is not compatible with hostname, port or socket_path"
+            )
+
+        if self.socket_path is not None and any([self.hostname, self.port]):
+            raise ValueError(
+                "The socket_path option is not compatible with hostname/port"
+            )
+
+        if self._local_hostname is not None and (
+            "\r" in self._local_hostname or "\n" in self._local_hostname
+        ):
+            raise ValueError(
+                "The local_hostname param contains prohibited newline characters"
+            )
+
+        if self.hostname is not None and (
+            "\r" in self.hostname or "\n" in self.hostname
+        ):
+            raise ValueError(
+                "The hostname param contains prohibited newline characters"
+            )
+
+    def _get_default_port(self) -> int:
+        """
+        Return an appropriate default port, based on options selected.
+        """
+        if self.use_tls:
+            return SMTP_TLS_PORT
+        elif self._start_tls_on_connect:
+            return SMTP_STARTTLS_PORT
+
+        return SMTP_PORT
+
+    async def connect(
+        self,
+        *,
+        hostname: Optional[Union[str, Default]] = _default,
+        port: Optional[Union[int, Default]] = _default,
+        username: Optional[Union[str, bytes, Default]] = _default,
+        password: Optional[Union[str, bytes, Default]] = _default,
+        local_hostname: Optional[Union[str, Default]] = _default,
+        source_address: Optional[Union[Tuple[str, int], Default]] = _default,
+        timeout: Optional[Union[float, Default]] = _default,
+        use_tls: Optional[bool] = None,
+        start_tls: Optional[Union[bool, Default]] = _default,
+        validate_certs: Optional[bool] = None,
+        client_cert: Optional[Union[str, Default]] = _default,
+        client_key: Optional[Union[str, Default]] = _default,
+        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
+        cert_bundle: Optional[Union[str, Default]] = _default,
+        socket_path: Optional[Union[SocketPathType, Default]] = _default,
+        sock: Optional[Union[socket.socket, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Initialize a connection to the server. Options provided to
+        :meth:`.connect` take precedence over those used to initialize the
+        class.
+
+        :keyword hostname:  Server name (or IP) to connect to. Defaults to "localhost".
+        :keyword port: Server port. Defaults ``465`` if ``use_tls`` is ``True``,
+            ``587`` if ``start_tls`` is ``True``, or ``25`` otherwise.
+        :keyword username:  Username to login as after connect.
+        :keyword password:  Password for login after connect.
+        :keyword local_hostname: The hostname of the client.  If specified, used as the
+            FQDN of the local host in the HELO/EHLO command. Otherwise, the result of
+            :func:`socket.getfqdn`. **Note that getfqdn will block the event loop.**
+        :keyword source_address: Takes a 2-tuple (host, port) for the socket to bind to
+            as its source address before connecting. If the host is '' and port is 0,
+            the OS default behavior will be used.
+        :keyword timeout: Default timeout value for the connection, in seconds.
+            Defaults to 60.
+        :keyword use_tls: If True, make the initial connection to the server
+            over TLS/SSL. Mutually exclusive with ``start_tls``; if the server uses
+            STARTTLS, ``use_tls`` should be ``False``.
+        :keyword start_tls: Flag to initiate a STARTTLS upgrade on connect.
+            If ``None`` (the default), upgrade will be initiated if supported by the
+            server.
+            If ``True``, and upgrade will be initiated regardless of server support.
+            If ``False``, no upgrade will occur.
+            Mutually exclusive with ``use_tls``.
+        :keyword validate_certs: Determines if server certificates are
+            validated. Defaults to ``True``.
+        :keyword client_cert: Path to client side certificate, for TLS.
+        :keyword client_key: Path to client side key, for TLS.
+        :keyword tls_context: An existing :py:class:`ssl.SSLContext`, for TLS.
+            Mutually exclusive with ``client_cert``/``client_key``.
+        :keyword cert_bundle: Path to certificate bundle, for TLS verification.
+        :keyword socket_path: Path to a Unix domain socket. Not compatible with
+            hostname or port. Accepts str, bytes, or a pathlike object.
+        :keyword sock: An existing, connected socket object. If given, none of
+            hostname, port, or socket_path should be provided.
+
+        :raises ValueError: mutually exclusive options provided
+        """
+        self._update_settings_from_kwargs(
+            hostname=hostname,
+            port=port,
+            local_hostname=local_hostname,
+            source_address=source_address,
+            use_tls=use_tls,
+            start_tls=start_tls,
+            validate_certs=validate_certs,
+            client_cert=client_cert,
+            client_key=client_key,
+            tls_context=tls_context,
+            cert_bundle=cert_bundle,
+            socket_path=socket_path,
+            sock=sock,
+            username=username,
+            password=password,
+        )
+        self._validate_config()
+
+        self.loop = asyncio.get_running_loop()
+        if self._connect_lock is None:
+            self._connect_lock = asyncio.Lock()
+        await self._connect_lock.acquire()
+
+        # Set default port last in case use_tls or start_tls is provided,
+        # and only if we're not using a socket.
+        if self.port is None and self.sock is None and self.socket_path is None:
+            self.port = self._get_default_port()
+
+        try:
+            response = await self._create_connection(
+                timeout=self.timeout if timeout is _default else timeout
+            )
+            await self._maybe_start_tls_on_connect()
+            await self._maybe_login_on_connect()
+        except Exception as exc:
+            self.close()  # Reset our state to disconnected
+            raise exc
+
+        return response
+
+    async def _create_connection(self, timeout: Optional[float]) -> SMTPResponse:
+        if self.loop is None:
+            raise RuntimeError("No event loop set")
+
+        protocol = SMTPProtocol(loop=self.loop)
+
+        tls_context: Optional[ssl.SSLContext] = None
+        ssl_handshake_timeout: Optional[float] = None
+        if self.use_tls:
+            tls_context = self._get_tls_context()
+            ssl_handshake_timeout = timeout
+
+        if self.sock is not None:
+            connect_coro = self.loop.create_connection(
+                lambda: protocol,
+                sock=self.sock,
+                ssl=tls_context,
+                ssl_handshake_timeout=ssl_handshake_timeout,
+            )
+        elif self.socket_path is not None:
+            connect_coro = self.loop.create_unix_connection(
+                lambda: protocol,
+                path=self.socket_path,  # type: ignore
+                ssl=tls_context,
+                ssl_handshake_timeout=ssl_handshake_timeout,
+            )
+        else:
+            if self.hostname is None:
+                raise RuntimeError("No hostname provided; default should have been set")
+            if self.port is None:
+                raise RuntimeError("No port provided; default should have been set")
+
+            connect_coro = self.loop.create_connection(
+                lambda: protocol,
+                host=self.hostname,
+                port=self.port,
+                ssl=tls_context,
+                ssl_handshake_timeout=ssl_handshake_timeout,
+                local_addr=self.source_address,
+            )
+
+        try:
+            transport, _ = await asyncio.wait_for(connect_coro, timeout=timeout)
+        except (TimeoutError, asyncio.TimeoutError) as exc:
+            raise SMTPConnectTimeoutError(
+                f"Timed out connecting to {self.hostname} on port {self.port}"
+            ) from exc
+        except OSError as exc:
+            raise SMTPConnectError(
+                f"Error connecting to {self.hostname} on port {self.port}: {exc}"
+            ) from exc
+
+        self.protocol = protocol
+        self.transport = transport
+
+        try:
+            response = await protocol.read_response(timeout=timeout)
+        except SMTPServerDisconnected as exc:
+            raise SMTPConnectError(
+                f"Error connecting to {self.hostname} on port {self.port}: {exc}"
+            ) from exc
+        except SMTPTimeoutError as exc:
+            raise SMTPConnectTimeoutError(
+                "Timed out waiting for server ready message"
+            ) from exc
+
+        if response.code != SMTPStatus.ready:
+            raise SMTPConnectResponseError(response.code, response.message)
+
+        return response
+
+    def _connection_lost(self, waiter: "asyncio.Future[None]") -> None:
+        self.close()
+
+    async def _maybe_start_tls_on_connect(self) -> None:
+        """
+        Depending on config, upgrade the connection via STARTTLS.
+        """
+        if self._start_tls_on_connect is True:
+            await self.starttls()
+        # If _start_tls_on_connect hasn't been set either way,
+        # try to STARTTLS if supported, with graceful failure handling
+        elif self._start_tls_on_connect is None:
+            already_using_tls = self.get_transport_info("sslcontext") is not None
+            if not (self.use_tls or already_using_tls):
+                await self._ehlo_or_helo_if_needed()
+                if self.supports_extension("starttls"):
+                    await self.starttls()
+
+    async def _maybe_login_on_connect(self) -> None:
+        """
+        Depending on config, login after connecting.
+        """
+        if self._login_username is not None:
+            login_password = (
+                self._login_password if self._login_password is not None else ""
+            )
+            await self.login(self._login_username, login_password)
+
+    async def execute_command(
+        self, *args: bytes, timeout: Optional[Union[float, Default]] = _default
+    ) -> SMTPResponse:
+        """
+        Check that we're connected, if we got a timeout value, and then
+        pass the command to the protocol.
+
+        :raises SMTPServerDisconnected: connection lost
+        """
+        if self.protocol is None:
+            raise SMTPServerDisconnected("Server not connected")
+
+        response = await self.protocol.execute_command(
+            *args, timeout=self.timeout if timeout is _default else timeout
+        )
+
+        # If the server is unavailable, be nice and close the connection
+        if response.code == SMTPStatus.domain_unavailable:
+            self.close()
+
+        return response
+
+    def _get_tls_context(self) -> ssl.SSLContext:
+        """
+        Build an SSLContext object from the options we've been given.
+        """
+        if self.tls_context is not None:
+            context = self.tls_context
+        else:
+            # SERVER_AUTH is what we want for a client side socket
+            context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
+            context.check_hostname = bool(self.validate_certs)
+            if self.validate_certs:
+                context.verify_mode = ssl.CERT_REQUIRED
+            else:
+                context.verify_mode = ssl.CERT_NONE
+
+            if self.cert_bundle is not None:
+                context.load_verify_locations(cafile=self.cert_bundle)
+
+            if self.client_cert is not None:
+                context.load_cert_chain(self.client_cert, keyfile=self.client_key)
+
+        return context
+
+    def close(self) -> None:
+        """
+        Closes the connection.
+        """
+        if self.transport is not None and not self.transport.is_closing():
+            self.transport.close()
+
+        if self._connect_lock is not None and self._connect_lock.locked():
+            self._connect_lock.release()
+
+        self.protocol = None
+        self.transport = None
+
+        # Reset ESMTP state
+        self._reset_server_state()
+
+    def get_transport_info(self, key: str) -> Any:
+        """
+        Get extra info from the transport.
+        Supported keys:
+
+            - ``peername``
+            - ``socket``
+            - ``sockname``
+            - ``compression``
+            - ``cipher``
+            - ``peercert``
+            - ``sslcontext``
+            - ``sslobject``
+
+        :raises SMTPServerDisconnected: connection lost
+        """
+        if not (self.is_connected and self.transport):
+            raise SMTPServerDisconnected("Server not connected")
+
+        return self.transport.get_extra_info(key)
+
+    # Base SMTP commands #
+
+    async def helo(
+        self,
+        *,
+        hostname: Optional[str] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send the SMTP HELO command.
+        Hostname to send for this command defaults to the FQDN of the local
+        host.
+
+        :raises SMTPHeloError: on unexpected server response code
+        """
+        response = self.last_helo_response = await self.execute_command(
+            b"HELO", (hostname or self.local_hostname).encode("ascii"), timeout=timeout
+        )
+
+        if response.code != SMTPStatus.completed:
+            raise SMTPHeloError(response.code, response.message)
+
+        return response
+
+    async def help(self, *, timeout: Optional[Union[float, Default]] = _default) -> str:
+        """
+        Send the SMTP HELP command, which responds with help text.
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        response = await self.execute_command(b"HELP", timeout=timeout)
+        if response.code not in (
+            SMTPStatus.system_status_ok,
+            SMTPStatus.help_message,
+            SMTPStatus.completed,
+        ):
+            raise SMTPResponseException(response.code, response.message)
+
+        return response.message
+
+    async def rset(
+        self, *, timeout: Optional[Union[float, Default]] = _default
+    ) -> SMTPResponse:
+        """
+        Send an SMTP RSET command, which resets the server's envelope
+        (the envelope contains the sender, recipient, and mail data).
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        response = await self.execute_command(b"RSET", timeout=timeout)
+        if response.code != SMTPStatus.completed:
+            raise SMTPResponseException(response.code, response.message)
+
+        return response
+
+    async def noop(
+        self, *, timeout: Optional[Union[float, Default]] = _default
+    ) -> SMTPResponse:
+        """
+        Send an SMTP NOOP command, which does nothing.
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        response = await self.execute_command(b"NOOP", timeout=timeout)
+        if response.code != SMTPStatus.completed:
+            raise SMTPResponseException(response.code, response.message)
+
+        return response
+
+    async def vrfy(
+        self,
+        address: str,
+        /,
+        *,
+        options: Optional[Iterable[str]] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send an SMTP VRFY command, which tests an address for validity.
+        Not many servers support this command.
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if options is None:
+            options = []
+
+        parsed_address = parse_address(address)
+        if any(option.lower() == "smtputf8" for option in options):
+            if not self.supports_extension("smtputf8"):
+                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
+            addr_bytes = parsed_address.encode("utf-8")
+        else:
+            addr_bytes = parsed_address.encode("ascii")
+        options_bytes = [option.encode("ascii") for option in options]
+
+        response = await self.execute_command(
+            b"VRFY", addr_bytes, *options_bytes, timeout=timeout
+        )
+
+        if response.code not in (
+            SMTPStatus.completed,
+            SMTPStatus.will_forward,
+            SMTPStatus.cannot_vrfy,
+        ):
+            raise SMTPResponseException(response.code, response.message)
+
+        return response
+
+    async def expn(
+        self,
+        address: str,
+        /,
+        *,
+        options: Optional[Iterable[str]] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send an SMTP EXPN command, which expands a mailing list.
+        Not many servers support this command.
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if options is None:
+            options = []
+
+        parsed_address = parse_address(address)
+        if any(option.lower() == "smtputf8" for option in options):
+            if not self.supports_extension("smtputf8"):
+                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
+            addr_bytes = parsed_address.encode("utf-8")
+        else:
+            addr_bytes = parsed_address.encode("ascii")
+        options_bytes = [option.encode("ascii") for option in options]
+
+        response = await self.execute_command(
+            b"EXPN", addr_bytes, *options_bytes, timeout=timeout
+        )
+
+        if response.code != SMTPStatus.completed:
+            raise SMTPResponseException(response.code, response.message)
+
+        return response
+
+    async def quit(
+        self, *, timeout: Optional[Union[float, Default]] = _default
+    ) -> SMTPResponse:
+        """
+        Send the SMTP QUIT command, which closes the connection.
+        Also closes the connection from our side after a response is received.
+
+        :raises SMTPResponseException: on unexpected server response code
+        """
+        response = await self.execute_command(b"QUIT", timeout=timeout)
+        if response.code != SMTPStatus.closing:
+            raise SMTPResponseException(response.code, response.message)
+
+        self.close()
+
+        return response
+
+    async def mail(
+        self,
+        sender: str,
+        /,
+        *,
+        options: Optional[Iterable[str]] = None,
+        encoding: str = "ascii",
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send an SMTP MAIL command, which specifies the message sender and
+        begins a new mail transfer session ("envelope").
+
+        :raises SMTPSenderRefused: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if options is None:
+            options = []
+
+        quoted_sender = quote_address(sender)
+        addr_bytes = quoted_sender.encode(encoding)
+        options_bytes = [option.encode("ascii") for option in options]
+
+        response = await self.execute_command(
+            b"MAIL", b"FROM:" + addr_bytes, *options_bytes, timeout=timeout
+        )
+
+        if response.code != SMTPStatus.completed:
+            raise SMTPSenderRefused(response.code, response.message, sender)
+
+        return response
+
+    async def rcpt(
+        self,
+        recipient: str,
+        /,
+        *,
+        options: Optional[Iterable[str]] = None,
+        encoding: str = "ascii",
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send an SMTP RCPT command, which specifies a single recipient for
+        the message. This command is sent once per recipient and must be
+        preceded by 'MAIL'.
+
+        :raises SMTPRecipientRefused: on unexpected server response code
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if options is None:
+            options = []
+
+        quoted_recipient = quote_address(recipient)
+        addr_bytes = quoted_recipient.encode(encoding)
+        options_bytes = [option.encode("ascii") for option in options]
+
+        response = await self.execute_command(
+            b"RCPT", b"TO:" + addr_bytes, *options_bytes, timeout=timeout
+        )
+
+        if response.code not in (SMTPStatus.completed, SMTPStatus.will_forward):
+            raise SMTPRecipientRefused(response.code, response.message, recipient)
+
+        return response
+
+    async def data(
+        self,
+        message: Union[str, bytes],
+        /,
+        *,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send an SMTP DATA command, followed by the message given.
+        This method transfers the actual email content to the server.
+
+        :raises SMTPDataError: on unexpected server response code
+        :raises SMTPServerDisconnected: connection lost
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        # As data accesses protocol directly, some handling is required
+        if self.protocol is None:
+            raise SMTPServerDisconnected("Connection lost")
+
+        if timeout is _default:
+            timeout = self.timeout
+
+        if isinstance(message, str):
+            message = message.encode("ascii")
+
+        return await self.protocol.execute_data_command(message, timeout=timeout)
+
+    # ESMTP commands #
+
+    async def ehlo(
+        self,
+        *,
+        hostname: Optional[str] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Send the SMTP EHLO command.
+        Hostname to send for this command defaults to the FQDN of the local
+        host.
+
+        :raises SMTPHeloError: on unexpected server response code
+        """
+        if hostname is None:
+            hostname = self.local_hostname
+
+        response = await self.execute_command(
+            b"EHLO", hostname.encode("ascii"), timeout=timeout
+        )
+        self.last_ehlo_response = response
+
+        if response.code != SMTPStatus.completed:
+            raise SMTPHeloError(response.code, response.message)
+
+        return response
+
+    def supports_extension(self, extension: str, /) -> bool:
+        """
+        Tests if the server supports the ESMTP service extension given.
+        """
+        return extension.lower() in self.esmtp_extensions
+
+    async def _ehlo_or_helo_if_needed(self) -> None:
+        """
+        Call self.ehlo() and/or self.helo() if needed.
+
+        If there has been no previous EHLO or HELO command this session, this
+        method tries ESMTP EHLO first.
+        """
+        if self.is_ehlo_or_helo_needed:
+            try:
+                await self.ehlo()
+            except SMTPHeloError as exc:
+                if self.is_connected:
+                    await self.helo()
+                else:
+                    raise exc
+
+    def _reset_server_state(self) -> None:
+        """
+        Clear stored information about the server.
+        """
+        self.last_helo_response = None
+        self._last_ehlo_response = None
+        self.esmtp_extensions = {}
+        self.supports_esmtp = False
+        self.server_auth_methods = []
+
+    async def starttls(
+        self,
+        *,
+        server_hostname: Optional[str] = None,
+        validate_certs: Optional[bool] = None,
+        client_cert: Optional[Union[str, Default]] = _default,
+        client_key: Optional[Union[str, Default]] = _default,
+        cert_bundle: Optional[Union[str, Default]] = _default,
+        tls_context: Optional[Union[ssl.SSLContext, Default]] = _default,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Puts the connection to the SMTP server into TLS mode.
+
+        If there has been no previous EHLO or HELO command this session, this
+        method tries ESMTP EHLO first.
+
+        If the server supports TLS, this will encrypt the rest of the SMTP
+        session. If you provide the keyfile and certfile parameters,
+        the identity of the SMTP server and client can be checked (if
+        validate_certs is True). You can also provide a custom SSLContext
+        object. If no certs or SSLContext is given, and TLS config was
+        provided when initializing the class, STARTTLS will use to that,
+        otherwise it will use the Python defaults.
+
+        :raises SMTPException: server does not support STARTTLS
+        :raises SMTPServerDisconnected: connection lost
+        :raises ValueError: invalid options provided
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if self.protocol is None:
+            raise SMTPServerDisconnected("Server not connected")
+
+        if self.get_transport_info("sslcontext") is not None:
+            raise SMTPException("Connection already using TLS")
+
+        self._update_settings_from_kwargs(
+            validate_certs=validate_certs,
+            client_cert=client_cert,
+            client_key=client_key,
+            cert_bundle=cert_bundle,
+            tls_context=tls_context,
+        )
+        self._validate_config()
+
+        if server_hostname is None:
+            server_hostname = self.hostname
+
+        if timeout is _default:
+            timeout = self.timeout
+
+        tls_context = self._get_tls_context()
+
+        if not self.supports_extension("starttls"):
+            raise SMTPException("SMTP STARTTLS extension not supported by server.")
+
+        response = await self.protocol.start_tls(
+            tls_context, server_hostname=server_hostname, timeout=timeout
+        )
+        if self.protocol is None:
+            raise SMTPServerDisconnected("Connection lost")
+        # Update our transport reference
+        self.transport = self.protocol.transport
+
+        # RFC 3207 part 4.2:
+        # The client MUST discard any knowledge obtained from the server, such
+        # as the list of SMTP service extensions, which was not obtained from
+        # the TLS negotiation itself.
+        self._reset_server_state()
+
+        return response
+
+    # Auth commands
+
+    async def login(
+        self,
+        username: Union[str, bytes],
+        password: Union[str, bytes],
+        /,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        Tries to login with supported auth methods.
+
+        Some servers advertise authentication methods they don't really
+        support, so if authentication fails, we continue until we've tried
+        all methods.
+        """
+        await self._ehlo_or_helo_if_needed()
+
+        if not self.supports_extension("auth"):
+            if self.is_connected and self.get_transport_info("sslcontext") is None:
+                raise SMTPException(
+                    "The SMTP AUTH extension is not supported by this server. Try "
+                    "connecting via TLS (or STARTTLS)."
+                )
+            raise SMTPException(
+                "The SMTP AUTH extension is not supported by this server."
+            )
+
+        response: Optional[SMTPResponse] = None
+        exception: Optional[SMTPAuthenticationError] = None
+        for auth_name in self.supported_auth_methods:
+            method_name = f'auth_{auth_name.replace("-", "")}'
+            try:
+                auth_method = getattr(self, method_name)
+            except AttributeError as err:
+                raise RuntimeError(
+                    f"Missing handler for auth method {auth_name}"
+                ) from err
+            try:
+                response = await auth_method(username, password, timeout=timeout)
+            except SMTPAuthenticationError as exc:
+                exception = exc
+            else:
+                # No exception means we're good
+                break
+
+        if response is None:
+            raise exception or SMTPException("No suitable authentication method found.")
+
+        return response
+
+    async def auth_crammd5(
+        self,
+        username: Union[str, bytes],
+        password: Union[str, bytes],
+        /,
+        *,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        CRAM-MD5 auth uses the password as a shared secret to MD5 the server's
+        response.
+
+        Example::
+
+            250 AUTH CRAM-MD5
+            auth cram-md5
+            334 PDI0NjA5LjEwNDc5MTQwNDZAcG9wbWFpbC5TcGFjZS5OZXQ+
+            dGltIGI5MTNhNjAyYzdlZGE3YTQ5NWI0ZTZlNzMzNGQzODkw
+
+        """
+        initial_response = await self.execute_command(
+            b"AUTH", b"CRAM-MD5", timeout=timeout
+        )
+
+        if initial_response.code != SMTPStatus.auth_continue:
+            raise SMTPAuthenticationError(
+                initial_response.code, initial_response.message
+            )
+
+        verification_bytes = auth_crammd5_verify(
+            username, password, initial_response.message
+        )
+        response = await self.execute_command(verification_bytes)
+
+        if response.code != SMTPStatus.auth_successful:
+            raise SMTPAuthenticationError(response.code, response.message)
+
+        return response
+
+    async def auth_plain(
+        self,
+        username: Union[str, bytes],
+        password: Union[str, bytes],
+        /,
+        *,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        PLAIN auth encodes the username and password in one Base64 encoded
+        string. No verification message is required.
+
+        Example::
+
+            220-esmtp.example.com
+            AUTH PLAIN dGVzdAB0ZXN0AHRlc3RwYXNz
+            235 ok, go ahead (#2.0.0)
+
+        """
+        encoded = auth_plain_encode(username, password)
+        response = await self.execute_command(
+            b"AUTH", b"PLAIN", encoded, timeout=timeout
+        )
+
+        if response.code != SMTPStatus.auth_successful:
+            raise SMTPAuthenticationError(response.code, response.message)
+
+        return response
+
+    async def auth_login(
+        self,
+        username: Union[str, bytes],
+        password: Union[str, bytes],
+        /,
+        *,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> SMTPResponse:
+        """
+        LOGIN auth sends the Base64 encoded username and password in sequence.
+
+        Example::
+
+            250 AUTH LOGIN PLAIN CRAM-MD5
+            auth login avlsdkfj
+            334 UGFzc3dvcmQ6
+            avlsdkfj
+
+        Note that there is an alternate version sends the username
+        as a separate command::
+
+            250 AUTH LOGIN PLAIN CRAM-MD5
+            auth login
+            334 VXNlcm5hbWU6
+            avlsdkfj
+            334 UGFzc3dvcmQ6
+            avlsdkfj
+
+        However, since most servers seem to support both, we send the username
+        with the initial request.
+        """
+        encoded_username, encoded_password = auth_login_encode(username, password)
+        initial_response = await self.execute_command(
+            b"AUTH", b"LOGIN", encoded_username, timeout=timeout
+        )
+
+        if initial_response.code != SMTPStatus.auth_continue:
+            raise SMTPAuthenticationError(
+                initial_response.code, initial_response.message
+            )
+
+        response = await self.execute_command(encoded_password, timeout=timeout)
+
+        if response.code != SMTPStatus.auth_successful:
+            raise SMTPAuthenticationError(response.code, response.message)
+
+        return response
+
+    async def sendmail(
+        self,
+        sender: str,
+        recipients: Union[str, Sequence[str]],
+        message: Union[str, bytes],
+        /,
+        *,
+        mail_options: Optional[Iterable[str]] = None,
+        rcpt_options: Optional[Iterable[str]] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        """
+        This command performs an entire mail transaction.
+
+        The arguments are:
+            - sender: The address sending this mail.
+            - recipients: A list of addresses to send this mail to.  A bare
+                string will be treated as a list with 1 address.
+            - message: The message string to send.
+            - mail_options: List of options (such as ESMTP 8bitmime) for the
+                MAIL command.
+            - rcpt_options: List of options (such as DSN commands) for all the
+                RCPT commands.
+
+        message must be a string containing characters in the ASCII range.
+        The string is encoded to bytes using the ascii codec, and lone \\\\r
+        and \\\\n characters are converted to \\\\r\\\\n characters.
+
+        If there has been no previous HELO or EHLO command this session, this
+        method tries EHLO first.
+
+        This method will return normally if the mail is accepted for at least
+        one recipient.  It returns a tuple consisting of:
+
+            - an error dictionary, with one entry for each recipient that was
+                refused.  Each entry contains a tuple of the SMTP error code
+                and the accompanying error message sent by the server.
+            - the message sent by the server in response to the DATA command
+                (often containing a message id)
+
+        Example:
+
+             >>> event_loop = asyncio.get_event_loop()
+             >>> smtp = aiosmtplib.SMTP(hostname="127.0.0.1", port=1025)
+             >>> event_loop.run_until_complete(smtp.connect())
+             (220, ...)
+             >>> recipients = ["one@one.org", "two@two.org", "3@three.org"]
+             >>> message = "From: Me@my.org\\nSubject: testing\\nHello World"
+             >>> send_coro = smtp.sendmail("me@my.org", recipients, message)
+             >>> event_loop.run_until_complete(send_coro)
+             ({}, 'OK')
+             >>> event_loop.run_until_complete(smtp.quit())
+             (221, Bye)
+
+        In the above example, the message was accepted for delivery for all
+        three addresses. If delivery had been only successful to two
+        of the three addresses, and one was rejected, the response would look
+        something like::
+
+            (
+                {"nobody@three.org": (550, "User unknown")},
+                "Written safely to disk. #902487694.289148.12219.",
+            )
+
+
+        If delivery is not successful to any addresses,
+        :exc:`.SMTPRecipientsRefused` is raised.
+
+        If :exc:`.SMTPResponseException` is raised by this method, we try to
+        send an RSET command to reset the server envelope automatically for
+        the next attempt.
+
+        :raises SMTPRecipientsRefused: delivery to all recipients failed
+        :raises SMTPResponseException: on invalid response
+        """
+        if isinstance(recipients, str):
+            recipients = [recipients]
+        if mail_options is None:
+            mail_options = []
+        else:
+            mail_options = list(mail_options)
+        if rcpt_options is None:
+            rcpt_options = []
+        else:
+            rcpt_options = list(rcpt_options)
+
+        if any(option.lower() == "smtputf8" for option in mail_options):
+            mailbox_encoding = "utf-8"
+        else:
+            mailbox_encoding = "ascii"
+
+        if self._sendmail_lock is None:
+            self._sendmail_lock = asyncio.Lock()
+
+        async with self._sendmail_lock:
+            # Make sure we've done an EHLO for extension checks
+            await self._ehlo_or_helo_if_needed()
+
+            if mailbox_encoding == "utf-8" and not self.supports_extension("smtputf8"):
+                raise SMTPNotSupported("SMTPUTF8 is not supported by this server")
+
+            if self.supports_extension("size"):
+                message_len = len(message)
+                size_option = f"size={message_len}"
+                mail_options.insert(0, size_option)
+
+            try:
+                await self.mail(
+                    sender,
+                    options=mail_options,
+                    encoding=mailbox_encoding,
+                    timeout=timeout,
+                )
+                recipient_errors = await self._send_recipients(
+                    recipients, rcpt_options, encoding=mailbox_encoding, timeout=timeout
+                )
+                response = await self.data(message, timeout=timeout)
+            except (SMTPResponseException, SMTPRecipientsRefused) as exc:
+                # If we got an error, reset the envelope.
+                try:
+                    await self.rset(timeout=timeout)
+                except (ConnectionError, SMTPResponseException):
+                    # If we're disconnected on the reset, or we get a bad
+                    # status, don't raise that as it's confusing
+                    pass
+                raise exc
+
+        return recipient_errors, response.message
+
+    async def _send_recipients(
+        self,
+        recipients: Sequence[str],
+        options: Iterable[str],
+        encoding: str = "ascii",
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> Dict[str, SMTPResponse]:
+        """
+        Send the recipients given to the server. Used as part of
+        :meth:`.sendmail`.
+        """
+        recipient_errors = []
+        for address in recipients:
+            try:
+                await self.rcpt(
+                    address, options=options, encoding=encoding, timeout=timeout
+                )
+            except SMTPRecipientRefused as exc:
+                recipient_errors.append(exc)
+
+        if len(recipient_errors) == len(recipients):
+            raise SMTPRecipientsRefused(recipient_errors)
+
+        formatted_errors = {
+            err.recipient: SMTPResponse(err.code, err.message)
+            for err in recipient_errors
+        }
+
+        return formatted_errors
+
+    async def send_message(
+        self,
+        message: Union[email.message.EmailMessage, email.message.Message],
+        /,
+        *,
+        sender: Optional[str] = None,
+        recipients: Optional[Union[str, Sequence[str]]] = None,
+        mail_options: Optional[Iterable[str]] = None,
+        rcpt_options: Optional[Iterable[str]] = None,
+        timeout: Optional[Union[float, Default]] = _default,
+    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        r"""
+        Sends an :py:class:`email.message.EmailMessage` object.
+
+        Arguments are as for :meth:`.sendmail`, except that message is an
+        :py:class:`email.message.EmailMessage` object.  If sender is None or
+        recipients is None, these arguments are taken from the headers of the
+        EmailMessage as described in RFC 2822.  Regardless of the values of sender
+        and recipients, any Bcc field (or Resent-Bcc field, when the message is a
+        resent) of the EmailMessage object will not be transmitted.  The EmailMessage
+        object is then serialized using :py:class:`email.generator.Generator` and
+        :meth:`.sendmail` is called to transmit the message.
+
+        'Resent-Date' is a mandatory field if the message is resent (RFC 2822
+        Section 3.6.6). In such a case, we use the 'Resent-\*' fields.
+        However, if there is more than one 'Resent-' block there's no way to
+        unambiguously determine which one is the most recent in all cases,
+        so rather than guess we raise a ``ValueError`` in that case.
+
+        :raises ValueError:
+            on more than one Resent header block
+            on no sender kwarg or From header in message
+            on no recipients kwarg or To, Cc or Bcc header in message
+        :raises SMTPRecipientsRefused: delivery to all recipients failed
+        :raises SMTPResponseException: on invalid response
+        """
+        if mail_options is None:
+            mail_options = []
+        else:
+            mail_options = list(mail_options)
+
+        if sender is None:
+            sender = extract_sender(message)
+        if sender is None:
+            raise ValueError("No From header provided in message")
+
+        if isinstance(recipients, str):
+            recipients = [recipients]
+        elif recipients is None:
+            recipients = extract_recipients(message)
+        if not recipients:
+            raise ValueError("No recipient headers provided in message")
+
+        # Make sure we've done an EHLO for extension checks
+        await self._ehlo_or_helo_if_needed()
+
+        try:
+            sender.encode("ascii")
+            "".join(recipients).encode("ascii")
+        except UnicodeEncodeError:
+            utf8_required = True
+        else:
+            utf8_required = False
+
+        if utf8_required:
+            if not self.supports_extension("smtputf8"):
+                raise SMTPNotSupported(
+                    "An address containing non-ASCII characters was provided, but "
+                    "SMTPUTF8 is not supported by this server"
+                )
+            elif "smtputf8" not in [option.lower() for option in mail_options]:
+                mail_options.append("SMTPUTF8")
+
+        if self.supports_extension("8BITMIME"):
+            if "body=8bitmime" not in [option.lower() for option in mail_options]:
+                mail_options.append("BODY=8BITMIME")
+            cte_type = "8bit"
+        else:
+            cte_type = "7bit"
+
+        flat_message = flatten_message(message, utf8=utf8_required, cte_type=cte_type)
+
+        return await self.sendmail(
+            sender,
+            recipients,
+            flat_message,
+            mail_options=mail_options,
+            rcpt_options=rcpt_options,
+            timeout=timeout,
+        )
+
+    def sendmail_sync(
+        self, *args: Any, **kwargs: Any
+    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        """
+        Synchronous version of :meth:`.sendmail`. This method starts
+        an event loop to connect, send the message, and disconnect.
+        """
+
+        async def sendmail_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
+            async with self:
+                return await self.sendmail(*args, **kwargs)
+
+        return asyncio.run(sendmail_coroutine())
+
+    def send_message_sync(
+        self, *args: Any, **kwargs: Any
+    ) -> Tuple[Dict[str, SMTPResponse], str]:
+        """
+        Synchronous version of :meth:`.send_message`. This method
+        starts an event loop to connect, send the message, and disconnect.
+        """
+
+        async def send_message_coroutine() -> Tuple[Dict[str, SMTPResponse], str]:
+            async with self:
+                return await self.send_message(*args, **kwargs)
+
+        return asyncio.run(send_message_coroutine())
diff -pruN 3.0.1-1/src/aiosmtplib/status.py 3.0.2-1/src/aiosmtplib/status.py
--- 3.0.1-1/src/aiosmtplib/status.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/status.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,5 @@
+from .typing import SMTPStatus
+
+
+# alias SMTPStatus for backwards compatibility
+__all__ = ("SMTPStatus",)
diff -pruN 3.0.1-1/src/aiosmtplib/typing.py 3.0.2-1/src/aiosmtplib/typing.py
--- 3.0.1-1/src/aiosmtplib/typing.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/src/aiosmtplib/typing.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,64 @@
+import enum
+import os
+import sys
+from typing import Union
+
+
+__all__ = ("Default", "SMTPStatus", "SocketPathType", "_default")
+
+# 3.9 changed the os.PathLike type
+if sys.version_info >= (3, 9):
+    SocketPathType = Union[str, bytes, os.PathLike[str]]
+else:
+    SocketPathType = Union[str, bytes, os.PathLike]
+
+
+class Default(enum.Enum):
+    """
+    Used for type hinting kwarg defaults.
+    """
+
+    token = 0
+
+
+_default = Default.token
+
+
+@enum.unique
+class SMTPStatus(enum.IntEnum):
+    """
+    Defines SMTP statuses for code readability.
+
+    See also: http://www.greenend.org.uk/rjk/tech/smtpreplies.html
+    """
+
+    invalid_response = -1
+    system_status_ok = 211
+    help_message = 214
+    ready = 220
+    closing = 221
+    auth_successful = 235
+    completed = 250
+    will_forward = 251
+    cannot_vrfy = 252
+    auth_continue = 334
+    start_input = 354
+    domain_unavailable = 421
+    mailbox_unavailable = 450
+    error_processing = 451
+    insufficient_storage = 452
+    tls_not_available = 454
+    unrecognized_command = 500
+    unrecognized_parameters = 501
+    command_not_implemented = 502
+    bad_command_sequence = 503
+    parameter_not_implemented = 504
+    domain_does_not_accept_mail = 521
+    access_denied = 530  # Sendmail specific
+    auth_failed = 535
+    mailbox_does_not_exist = 550
+    user_not_local = 551
+    storage_exceeded = 552
+    mailbox_name_invalid = 553
+    transaction_failed = 554
+    syntax_error = 555
diff -pruN 3.0.1-1/tests/conftest.py 3.0.2-1/tests/conftest.py
--- 3.0.1-1/tests/conftest.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/conftest.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Pytest fixtures and config.
 """
+
 import asyncio
 import email.header
 import email.message
@@ -98,7 +99,7 @@ def pytest_sessionfinish(session: pytest
 
 @pytest_asyncio.fixture
 def debug_event_loop(
-    event_loop: asyncio.AbstractEventLoop
+    event_loop: asyncio.AbstractEventLoop,
 ) -> asyncio.AbstractEventLoop:
     previous_debug = event_loop.get_debug()
     event_loop.set_debug(True)
@@ -448,7 +449,7 @@ def smtpd_mock_response_error_disconnect
 @pytest.fixture(scope="session")
 def smtpd_mock_response_bad_data() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
     async def mock_response_bad_data(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd._writer.write(b"250 \xFF\xFF\xFF\xFF\r\n")
+        smtpd._writer.write(b"250 \xff\xff\xff\xff\r\n")
         await smtpd._writer.drain()
 
     return mock_response_bad_data
diff -pruN 3.0.1-1/tests/pyright.py 3.0.2-1/tests/pyright.py
--- 3.0.1-1/tests/pyright.py	1970-01-01 00:00:00.000000000 +0000
+++ 3.0.2-1/tests/pyright.py	2020-02-02 00:00:00.000000000 +0000
@@ -0,0 +1,43 @@
+# pyright: strict
+
+import email.message
+import pathlib
+import socket
+import ssl
+
+import aiosmtplib
+
+
+async def send_str():
+    await aiosmtplib.send("test")
+
+
+async def send_message():
+    message = email.message.EmailMessage()
+    await aiosmtplib.send(message)
+
+
+async def send_all_options():
+    await aiosmtplib.send(
+        "test",
+        sender="test@example.com",
+        recipients=["user1@example.com"],
+        mail_options=["FOO"],
+        rcpt_options=["BAR"],
+        hostname="smtp.example.com",
+        port=1234,
+        username="root@example.com",
+        password="changeme",
+        local_hostname="test",
+        source_address=("foo", 123),
+        timeout=124.34,
+        use_tls=True,
+        start_tls=False,
+        validate_certs=False,
+        client_cert="test",
+        client_key="test",
+        tls_context=ssl.create_default_context(),
+        cert_bundle="test",
+        socket_path=pathlib.PurePath("/tmp/foo"),
+        sock=socket.socket(),
+    )
diff -pruN 3.0.1-1/tests/smtpd.py 3.0.2-1/tests/smtpd.py
--- 3.0.1-1/tests/smtpd.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/smtpd.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Implements handlers required on top of aiosmtpd for testing.
 """
+
 import asyncio
 import logging
 from email.errors import HeaderParseError
diff -pruN 3.0.1-1/tests/test_api.py 3.0.2-1/tests/test_api.py
--- 3.0.1-1/tests/test_api.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_api.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 send coroutine testing.
 """
+
 import asyncio
 import email
 import pathlib
diff -pruN 3.0.1-1/tests/test_auth_methods.py 3.0.2-1/tests/test_auth_methods.py
--- 3.0.1-1/tests/test_auth_methods.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_auth_methods.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Tests for auth methods on the SMTP class.
 """
+
 import asyncio
 import base64
 
diff -pruN 3.0.1-1/tests/test_auth_utils.py 3.0.2-1/tests/test_auth_utils.py
--- 3.0.1-1/tests/test_auth_utils.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_auth_utils.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Tests for authentication encoding utils.
 """
+
 import base64
 import hmac
 
diff -pruN 3.0.1-1/tests/test_commands.py 3.0.2-1/tests/test_commands.py
--- 3.0.1-1/tests/test_commands.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_commands.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Lower level SMTP command tests.
 """
+
 import asyncio
 from typing import Any, Callable, Coroutine, List, Tuple, Type
 
diff -pruN 3.0.1-1/tests/test_config.py 3.0.2-1/tests/test_config.py
--- 3.0.1-1/tests/test_config.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_config.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Tests covering SMTP configuration options.
 """
+
 import asyncio
 import socket
 import ssl
diff -pruN 3.0.1-1/tests/test_connect.py 3.0.2-1/tests/test_connect.py
--- 3.0.1-1/tests/test_connect.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_connect.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Connectivity tests.
 """
+
 import asyncio
 import pathlib
 import socket
diff -pruN 3.0.1-1/tests/test_email_utils.py 3.0.2-1/tests/test_email_utils.py
--- 3.0.1-1/tests/test_email_utils.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_email_utils.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Test message and address parsing/formatting functions.
 """
+
 from email.header import Header
 from email.headerregistry import Address
 from email.message import EmailMessage, Message
diff -pruN 3.0.1-1/tests/test_errors.py 3.0.2-1/tests/test_errors.py
--- 3.0.1-1/tests/test_errors.py	2023-11-02 18:36:16.219468600 +0000
+++ 3.0.2-1/tests/test_errors.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Test error class imports, arguments, and inheritance.
 """
+
 import asyncio
 from typing import List, Tuple, Type, Union
 
diff -pruN 3.0.1-1/tests/test_live.py 3.0.2-1/tests/test_live.py
--- 3.0.1-1/tests/test_live.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_live.py	2020-02-02 00:00:00.000000000 +0000
@@ -3,6 +3,7 @@ Tests run against live mail providers.
 
 These aren't generally run as part of the test suite.
 """
+
 import os
 from email.message import EmailMessage
 
diff -pruN 3.0.1-1/tests/test_protocol.py 3.0.2-1/tests/test_protocol.py
--- 3.0.1-1/tests/test_protocol.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_protocol.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Protocol level tests.
 """
+
 import asyncio
 import gc
 import socket
diff -pruN 3.0.1-1/tests/test_sendmail.py 3.0.2-1/tests/test_sendmail.py
--- 3.0.1-1/tests/test_sendmail.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_sendmail.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 SMTP.sendmail and SMTP.send_message method testing.
 """
+
 import asyncio
 import copy
 import email.generator
@@ -303,9 +304,10 @@ async def test_send_message_with_cc_reci
     assert not errors
 
     assert len(received_messages) == 1
-    assert received_messages[0][
-        "X-RcptTo"
-    ] == f'{recipient_str}, {", ".join(cc_recipients)}'
+    assert (
+        received_messages[0]["X-RcptTo"]
+        == f'{recipient_str}, {", ".join(cc_recipients)}'
+    )
 
     assert received_commands[2][0] == "RCPT"
     assert received_commands[2][1][0] == recipient_str
diff -pruN 3.0.1-1/tests/test_sync.py 3.0.2-1/tests/test_sync.py
--- 3.0.1-1/tests/test_sync.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_sync.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Sync method tests.
 """
+
 import email.message
 
 from aiosmtplib import SMTP
diff -pruN 3.0.1-1/tests/test_timeouts.py 3.0.2-1/tests/test_timeouts.py
--- 3.0.1-1/tests/test_timeouts.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_timeouts.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 Timeout tests.
 """
+
 import asyncio
 import socket
 import ssl
diff -pruN 3.0.1-1/tests/test_tls.py 3.0.2-1/tests/test_tls.py
--- 3.0.1-1/tests/test_tls.py	2023-11-02 18:36:16.223468500 +0000
+++ 3.0.2-1/tests/test_tls.py	2020-02-02 00:00:00.000000000 +0000
@@ -1,6 +1,7 @@
 """
 TLS and STARTTLS handling.
 """
+
 import asyncio
 import copy
 import ssl
